@@ -12,7 +12,7 @@
 #include "dialog.h"
 #include "hash_dialog.h"
 
-#include "cfg_gui.h"
+#include "configure.h"
 #include "cfg_module.h"
 #include "cfg_cmdseq.h"
 
@@ -97,9 +97,9 @@ static void	reset_row_widgets(P_CmdSeq *page);
 /* ----------------------------------------------------------------------------------------- */
 
 /* 1998-09-26 -	Compare two command sequences, lexicographically on their names. */
-static gint cmp_cmdseq(gpointer a, gpointer b)
+static gint cmp_cmdseq(gconstpointer a, gconstpointer b)
 {
-	CmdSeq	*sa = (CmdSeq *) a, *sb = (CmdSeq *) b;
+	const CmdSeq	*sa = a, *sb = b;
 
 	if(sa != NULL && sb != NULL)
 		return strcmp(sa->name, sb->name);
@@ -109,9 +109,9 @@ static gint cmp_cmdseq(gpointer a, gpointer b)
 /* 1998-09-26 -	Insert command sequence named <key> into list at <user>, sorted. */
 static void sort_cmdseq(gpointer key, gpointer data, gpointer user)
 {
-	GList	**sorted = (GList **) user;
+	GList	**sorted = user;
 
-	*sorted = g_list_insert_sorted(*sorted, data, (GCompareFunc) cmp_cmdseq);
+	*sorted = g_list_insert_sorted(*sorted, data, cmp_cmdseq);
 }
 
 /* 1998-09-26 -	Zzap the clist and reinitialize its contents, based on the current contents
@@ -119,7 +119,7 @@ static void sort_cmdseq(gpointer key, gpointer data, gpointer user)
 */
 static void repopulate_clist(P_CmdSeq *page)
 {
-	char	*line[] = { NULL };
+	gchar	*line[] = { NULL };
 	GList	*sorted = NULL, *iter;
 	gint	row;
 
@@ -128,7 +128,7 @@ static void repopulate_clist(P_CmdSeq *page)
 
 	gtk_clist_freeze(GTK_CLIST(page->clist));
 	gtk_clist_clear(GTK_CLIST(page->clist));
-	g_hash_table_foreach(page->cmdseq, sort_cmdseq, (gpointer) &sorted);
+	g_hash_table_foreach(page->cmdseq, sort_cmdseq, &sorted);
 	for(iter = sorted; iter != NULL; iter = g_list_next(iter))
 	{
 		line[0] = ((CmdSeq *) iter->data)->name;
@@ -144,10 +144,10 @@ static void repopulate_clist(P_CmdSeq *page)
 */
 static void copy_cmdseq(gpointer key, gpointer data, gpointer user)
 {
-	P_CmdSeq	*page = (P_CmdSeq *) user;
+	P_CmdSeq	*page = user;
 	CmdSeq		*seq;
 
-	if((seq = csq_cmdseq_copy((CmdSeq *) data)) != NULL)
+	if((seq = csq_cmdseq_copy(data)) != NULL)
 		csq_cmdseq_hash(&page->cmdseq, seq);
 }
 
@@ -159,7 +159,7 @@ static void populate_clist(MainInfo *min, P_CmdSeq *page)
 	page->cmdseq = NULL;
 	if(min->cfg.commands.cmdseq != NULL)
 	{
-		g_hash_table_foreach(min->cfg.commands.cmdseq, copy_cmdseq, (gpointer) page);
+		g_hash_table_foreach(min->cfg.commands.cmdseq, copy_cmdseq, page);
 		repopulate_clist(page);
 	}
 }
@@ -180,7 +180,7 @@ static void populate_def_clist(P_CmdSeq *page)
 		gtk_clist_clear(GTK_CLIST(page->dclist));
 		for(iter = page->curr_seq->rows; iter != NULL; iter = g_list_next(iter))
 		{
-			row = (CmdRow *) iter->data;
+			row = iter->data;
 
 			line[0] = (gchar *) csq_cmdrow_type_to_string(row->type);
 			line[1] = row->def->str;
@@ -226,7 +226,7 @@ static void set_repeat(P_CmdSeq *page)
 
 		for(iter = page->curr_seq->rows; iter != NULL; iter = g_list_next(iter))
 		{
-			row = (CmdRow *) iter->data;
+			row = iter->data;
 			if((row->type == CRTP_EXTERNAL) && (row->extra.external.gflags & CGF_RUNINBG))
 				break;
 		}
@@ -255,9 +255,9 @@ static void set_widgets(P_CmdSeq *page)
 /* ----------------------------------------------------------------------------------------- */
 
 /* 1998-09-26 -	User just selected a new command sequence to work with. */
-static gint evt_cmdseq_selected(GtkWidget *wid, gint row, gint column, GdkEventButton *evt)
+static gint evt_cmdseq_selected(GtkWidget *wid, gint row, gint column, GdkEventButton *evt, gpointer user)
 {
-	P_CmdSeq	*page = (P_CmdSeq *) gtk_object_get_user_data(GTK_OBJECT(wid));
+	P_CmdSeq	*page = user;
 	CmdSeq		*cs;
 
 	if((cs = gtk_clist_get_row_data(GTK_CLIST(wid), row)) != NULL)
@@ -272,9 +272,9 @@ static gint evt_cmdseq_selected(GtkWidget *wid, gint row, gint column, GdkEventB
 }
 
 /* 1998-09-26 -	There is no longer a current command sequence. */
-static gint evt_cmdseq_unselected(GtkWidget *wid, gint row, gint column, GdkEventButton *evt)
+static gint evt_cmdseq_unselected(GtkWidget *wid, gint row, gint column, GdkEventButton *evt, gpointer user)
 {
-	P_CmdSeq	*page = (P_CmdSeq *) gtk_object_get_user_data(GTK_OBJECT(wid));
+	P_CmdSeq	*page = user;
 
 	page->curr_seq = NULL;
 	page->curr_crow = -1;
@@ -287,10 +287,10 @@ static gint evt_cmdseq_unselected(GtkWidget *wid, gint row, gint column, GdkEven
 /* ----------------------------------------------------------------------------------------- */
 
 /* 1998-09-26 -	User is editing the name of the current sequence. */
-static gint evt_name_changed(GtkWidget *wid, gpointer data)
+static gint evt_name_changed(GtkWidget *wid, gpointer user)
 {
-	char		*text;
-	P_CmdSeq	*page = (P_CmdSeq *) data;
+	P_CmdSeq	*page = user;
+	const gchar	*text;
 
 	if(page != NULL && page->curr_seq != NULL)
 	{
@@ -388,25 +388,24 @@ static void reset_row_widgets(P_CmdSeq *page)
 }
 
 /* 1998-09-26 -	Called when the user selects a row of the current sequence. */
-static gint evt_row_selected(GtkWidget *wid, gint row, gint column, GdkEventButton *evt)
+static gint evt_row_selected(GtkWidget *wid, gint row, gint column, GdkEventButton *evt, gpointer user)
 {
-	P_CmdSeq	*page = (P_CmdSeq *) gtk_object_get_user_data(GTK_OBJECT(wid));
+	P_CmdSeq	*page = user;
 
 	if(page != NULL)
 	{
-		page->curr_row = (CmdRow *) gtk_clist_get_row_data(GTK_CLIST(page->dclist), row);
+		page->curr_row = gtk_clist_get_row_data(GTK_CLIST(page->dclist), row);
 		page->curr_rrow = row;
 		page->curr_type = page->curr_row->type;
 		set_row_widgets(page);
 	}
-
 	return TRUE;
 }
 
 /* 1998-09-26 -	Called as the row clist loses its selection. */
-static gint evt_row_unselected(GtkWidget *wid, gint row, gint column, GdkEventButton *evt)
+static gint evt_row_unselected(GtkWidget *wid, gint row, gint column, GdkEventButton *evt, gpointer user)
 {
-	P_CmdSeq	*page = (P_CmdSeq *) gtk_object_get_user_data(GTK_OBJECT(wid));
+	P_CmdSeq	*page = user;
 
 	if(page != NULL)
 	{
@@ -414,7 +413,6 @@ static gint evt_row_unselected(GtkWidget *wid, gint row, gint column, GdkEventBu
 		page->curr_rrow = -1;
 		reset_row_widgets(page);
 	}
-
 	return TRUE;
 }
 
@@ -438,9 +436,9 @@ static gint evt_repeat_clicked(GtkWidget *wid, gpointer user)
 /* ----------------------------------------------------------------------------------------- */
 
 /* 1998-09-27 -	User just hit the "Add Row" button, so let's do just that. */
-static gint evt_addrow_clicked(GtkWidget *wid, gpointer data)
+static gint evt_addrow_clicked(GtkWidget *wid, gpointer user)
 {
-	P_CmdSeq	*page = (P_CmdSeq *) data;
+	P_CmdSeq	*page = user;
 	CmdRow		*nr;
 	gint		pos;
 
@@ -460,9 +458,9 @@ static gint evt_addrow_clicked(GtkWidget *wid, gpointer data)
 }
 
 /* 1998-09-27 -	Delete the current row from the current command sequence. */
-static gint evt_delrow_clicked(GtkWidget *wid, gpointer data)
+static gint evt_delrow_clicked(GtkWidget *wid, gpointer user)
 {
-	P_CmdSeq	*page = (P_CmdSeq *) data;
+	P_CmdSeq	*page = user;
 	gint		opos, npos;
 
 	if(page != NULL && page->curr_seq != NULL && page->curr_row != NULL)
@@ -479,9 +477,9 @@ static gint evt_delrow_clicked(GtkWidget *wid, gpointer data)
 }
 
 /* 1998-09-27 -	User wants to duplicate the current row. Fine. */
-static gint evt_duprow_clicked(GtkWidget *wid, gpointer data)
+static gint evt_duprow_clicked(GtkWidget *wid, gpointer user)
 {
-	P_CmdSeq	*page = (P_CmdSeq *) data;
+	P_CmdSeq	*page = user;
 	CmdRow		*nr;
 	gint		pos;
 
@@ -500,9 +498,9 @@ static gint evt_duprow_clicked(GtkWidget *wid, gpointer data)
 }
 
 /* 1998-09-27 -	Move currently selected row up one row (exchange with previous row). */
-static gint evt_uprow_clicked(GtkWidget *wid, gpointer data)
+static gint evt_uprow_clicked(GtkWidget *wid, gpointer user)
 {
-	P_CmdSeq	*page = (P_CmdSeq *) data;
+	P_CmdSeq	*page = user;
 	gint		pos;
 
 	if(page != NULL && page->curr_seq != NULL && page->curr_row != NULL)
@@ -517,9 +515,9 @@ static gint evt_uprow_clicked(GtkWidget *wid, gpointer data)
 }
 
 /* 1998-09-27 -	Move currently selected row down one row (exchange with next row). */
-static gint evt_downrow_clicked(GtkWidget *wid, gpointer data)
+static gint evt_downrow_clicked(GtkWidget *wid, gpointer user)
 {
-	P_CmdSeq	*page = (P_CmdSeq *) data;
+	P_CmdSeq	*page = user;
 	gint		pos;
 
 	if(page != NULL && page->curr_seq != NULL && page->curr_row != NULL)
@@ -536,9 +534,9 @@ static gint evt_downrow_clicked(GtkWidget *wid, gpointer data)
 /* ----------------------------------------------------------------------------------------- */
 
 /* 1998-09-26 -	User choose a new type for the current row. */
-static gint evt_type_selected(GtkWidget *wid, gpointer data)
+static gint evt_type_selected(GtkWidget *wid, gpointer user)
 {
-	P_CmdSeq	*page = (P_CmdSeq *) data;
+	P_CmdSeq	*page = user;
 	gint		pos;
 
 	if(page != NULL && page->curr_seq != NULL && page->curr_row != NULL)
@@ -560,10 +558,10 @@ static gint evt_type_selected(GtkWidget *wid, gpointer data)
 **		Notice how this routine avoids to rebuild the entire clist; this is
 **		probably a good idea since the changes may be rapid.
 */
-static gint evt_def_changed(GtkWidget *wid, gpointer data)
+static gint evt_def_changed(GtkWidget *wid, gpointer user)
 {
-	P_CmdSeq	*page = (P_CmdSeq *) data;
-	char		*def;
+	P_CmdSeq	*page = user;
+	const gchar	*def;
 
 	if(page != NULL && page->curr_seq != NULL && page->curr_row != NULL)
 	{
@@ -629,9 +627,9 @@ static void pick_external(P_CmdSeq *page)
 					"Is:\"label\"=\"default\"",	N_("Input string"),
 					"Ix:\"label\"",			N_("Input check button (gives TRUE or FALSE)"),
 					"It:\"label\"",			N_("Set title text of input window"),
-					"$NAME",			N_("Value of $NAME (environment)"),
+					N_("$NAME"),			N_("Value of $NAME (environment)"),
 					"#",				N_("gentoo's PID"),
-					"~NAME",			N_("Home directory for user NAME"),
+					N_("~NAME"),			N_("Home directory for user NAME"),
 					NULL },	*row[2];
 	gchar		*sel_code = NULL;
 	guint		i;
@@ -651,9 +649,8 @@ static void pick_external(P_CmdSeq *page)
 		gtk_clist_append(GTK_CLIST(clist), (gchar **) row);
 	}
 	gtk_container_add(GTK_CONTAINER(scwin), clist);
-	gtk_widget_show(clist);
 
-	dlg = dlg_dialog_sync_new(scwin, _("Pick Code"), _("OK|Cancel"));
+	dlg = dlg_dialog_sync_new(scwin, _("Pick Code"), NULL);
 	gtk_signal_connect(GTK_OBJECT(clist), "select_row", GTK_SIGNAL_FUNC(evt_extcode_selected), dlg);
 	if((dlg_dialog_sync_wait(dlg) == DLG_POSITIVE) && (sel_code != NULL))
 	{
@@ -671,9 +668,9 @@ static void pick_external(P_CmdSeq *page)
 /* 1998-09-27 -	The details (formerly "...") button has been clicked; pop up a quick-selection
 **		window where the user can select something s?he's too lazy to type.
 */
-static gint evt_pick_clicked(GtkWidget *wid, gpointer data)
+static gint evt_pick_clicked(GtkWidget *wid, gpointer user)
 {
-	P_CmdSeq	*page = (P_CmdSeq *) data;
+	P_CmdSeq	*page = user;
 
 	if(page != NULL && page->curr_seq != NULL && page->curr_row != NULL)
 	{
@@ -689,11 +686,11 @@ static gint evt_pick_clicked(GtkWidget *wid, gpointer data)
 /* ----------------------------------------------------------------------------------------- */
 
 /* 1998-09-27 -	Add a new command sequence to the happy bunch. */
-static gint evt_add_clicked(GtkWidget *wid, gpointer data)
+static gint evt_add_clicked(GtkWidget *wid, gpointer user)
 {
+	P_CmdSeq	*page = user;
 	gchar		*line[] = { NULL };
 	gint		row;
-	P_CmdSeq	*page = (P_CmdSeq *) data;
 	CmdSeq		*ns;
 
 	if(page != NULL)
@@ -703,7 +700,7 @@ static gint evt_add_clicked(GtkWidget *wid, gpointer data)
 		{
 			csq_cmdseq_hash(&page->cmdseq, ns);
 			row = gtk_clist_append(GTK_CLIST(page->clist), line);
-			gtk_clist_set_row_data(GTK_CLIST(page->clist), row, (gpointer) ns);
+			gtk_clist_set_row_data(GTK_CLIST(page->clist), row, ns);
 			gtk_clist_select_row(GTK_CLIST(page->clist), row, 0);
 			gtk_entry_select_region(GTK_ENTRY(page->name), 0, -1);
 			gtk_widget_grab_focus(page->name);
@@ -713,9 +710,9 @@ static gint evt_add_clicked(GtkWidget *wid, gpointer data)
 }
 
 /* 1998-09-27 -	Delete the currently selected command sequence. */
-static gint evt_del_clicked(GtkWidget *wid, gpointer data)
+static gint evt_del_clicked(GtkWidget *wid, gpointer user)
 {
-	P_CmdSeq	*page = (P_CmdSeq *) data;
+	P_CmdSeq	*page = user;
 
 	if(page != NULL && page->curr_seq != NULL)
 	{
@@ -732,16 +729,15 @@ static gint evt_del_clicked(GtkWidget *wid, gpointer data)
 /* 1998-09-26 -	Build the (currently very pointless) extra controls for builtin rows. */
 static void build_px_builtin(P_CmdSeq *page, PX_Builtin *pxb)
 {
-	pxb->vbox = gtk_vbox_new(FALSE, 0);
+	pxb->vbox  = gtk_vbox_new(FALSE, 0);
 	pxb->label = gtk_label_new(_("(No Options Available)"));
 	gtk_box_pack_start(GTK_BOX(pxb->vbox), pxb->label, TRUE, TRUE, 0);
-	gtk_widget_show(pxb->label);
 }
 
 /* 1998-09-27 -	One of the general flag check buttons was clicked. */
-static gint evt_gf_clicked(GtkWidget *wid, gpointer data)
+static gint evt_gf_clicked(GtkWidget *wid, gpointer user)
 {
-	P_CmdSeq	*page = (P_CmdSeq *) data;
+	P_CmdSeq	*page = user;
 	guint32		flags = GPOINTER_TO_UINT(gtk_object_get_user_data(GTK_OBJECT(wid)));
 
 	if(page != NULL && page->curr_seq != NULL && page->curr_row != NULL && page->curr_row->type == CRTP_EXTERNAL)
@@ -771,25 +767,20 @@ static void build_pxe_general(P_CmdSeq *page, PXE_General *pxg)
 	gtk_object_set_user_data(GTK_OBJECT(pxg->runbg), GINT_TO_POINTER(CGF_RUNINBG));
 	gtk_signal_connect(GTK_OBJECT(pxg->runbg), "clicked", GTK_SIGNAL_FUNC(evt_gf_clicked), page);
 	gtk_box_pack_start(GTK_BOX(hbox), pxg->runbg, FALSE, FALSE, 0);
-	gtk_widget_show(pxg->runbg);
 	pxg->killprev = gtk_check_button_new_with_label(_("Kill Previous Instance?"));
 	gtk_object_set_user_data(GTK_OBJECT(pxg->killprev), GINT_TO_POINTER(CGF_KILLPREV));
 	gtk_signal_connect(GTK_OBJECT(pxg->killprev), "clicked", GTK_SIGNAL_FUNC(evt_gf_clicked), page);
 	gtk_box_pack_start(GTK_BOX(hbox), pxg->killprev, FALSE, FALSE, 0);
-	gtk_widget_show(pxg->killprev);
 	pxg->survive = gtk_check_button_new_with_label(_("Survive Quit?"));
 	gtk_object_set_user_data(GTK_OBJECT(pxg->survive), GINT_TO_POINTER(CGF_SURVIVE));
 	gtk_signal_connect(GTK_OBJECT(pxg->survive), "clicked", GTK_SIGNAL_FUNC(evt_gf_clicked), page);
 	gtk_box_pack_start(GTK_BOX(hbox), pxg->survive, FALSE, FALSE, 0);
-	gtk_widget_show(pxg->survive);
 	gtk_box_pack_start(GTK_BOX(pxg->vbox), hbox, FALSE, FALSE, 0);
-	gtk_widget_show(hbox);
 
 	pxg->graboutput = gtk_check_button_new_with_label(_("Capture Output?"));
 	gtk_object_set_user_data(GTK_OBJECT(pxg->graboutput), GINT_TO_POINTER(CGF_GRABOUTPUT));
 	gtk_signal_connect(GTK_OBJECT(pxg->graboutput), "clicked", GTK_SIGNAL_FUNC(evt_gf_clicked), page);
 	gtk_box_pack_start(GTK_BOX(pxg->vbox), pxg->graboutput, FALSE, FALSE, 0);
-	gtk_widget_show(pxg->graboutput);
 }
 
 /* 1999-03-09 -	One of the CD check buttons was clicked. Do the pseudo-radio logic, and alter the
@@ -861,16 +852,12 @@ static void build_px_baf(P_CmdSeq *page, PXE_BAF *pxbaf)
 	pxbaf->cdsrc = gtk_check_button_new_with_label(_("CD Source?"));
 	gtk_signal_connect(GTK_OBJECT(pxbaf->cdsrc), "clicked", GTK_SIGNAL_FUNC(evt_baf_cd_clicked), page);
 	gtk_box_pack_start(GTK_BOX(vbox), pxbaf->cdsrc, TRUE, TRUE, 0);
-	gtk_widget_show(pxbaf->cdsrc);
 	pxbaf->cddst = gtk_check_button_new_with_label(_("CD Destination?"));
 	gtk_object_set_data(GTK_OBJECT(pxbaf->cddst), "destination", GINT_TO_POINTER(1));	/* Just a silly flag. */
 	gtk_signal_connect(GTK_OBJECT(pxbaf->cddst), "clicked", GTK_SIGNAL_FUNC(evt_baf_cd_clicked), page);
 	gtk_box_pack_start(GTK_BOX(vbox), pxbaf->cddst, TRUE, TRUE, 0);
-	gtk_widget_show(pxbaf->cddst);
 	gtk_container_add(GTK_CONTAINER(pxbaf->bframe), vbox);
-	gtk_widget_show(vbox);
 	gtk_box_pack_start(GTK_BOX(pxbaf->hbox), pxbaf->bframe, TRUE, TRUE, 0);
-	gtk_widget_show(pxbaf->bframe);
 
 	pxbaf->aframe = gtk_frame_new(_("After Execution"));
 	vbox = gtk_vbox_new(FALSE, 0);
@@ -878,16 +865,12 @@ static void build_px_baf(P_CmdSeq *page, PXE_BAF *pxbaf)
 	gtk_object_set_user_data(GTK_OBJECT(pxbaf->rssrc), GINT_TO_POINTER(CBAF_RESCAN_SOURCE));
 	gtk_signal_connect(GTK_OBJECT(pxbaf->rssrc), "clicked", GTK_SIGNAL_FUNC(evt_baf_rescan_clicked), page);
 	gtk_box_pack_start(GTK_BOX(vbox), pxbaf->rssrc, TRUE, TRUE, 0);
-	gtk_widget_show(pxbaf->rssrc);
 	pxbaf->rsdst = gtk_check_button_new_with_label(_("Rescan Destination?"));
 	gtk_object_set_user_data(GTK_OBJECT(pxbaf->rsdst), GINT_TO_POINTER(CBAF_RESCAN_DEST));
 	gtk_signal_connect(GTK_OBJECT(pxbaf->rsdst), "clicked", GTK_SIGNAL_FUNC(evt_baf_rescan_clicked), page);
 	gtk_box_pack_start(GTK_BOX(vbox), pxbaf->rsdst, TRUE, TRUE, 0);
-	gtk_widget_show(pxbaf->rsdst);
 	gtk_container_add(GTK_CONTAINER(pxbaf->aframe), vbox);
-	gtk_widget_show(vbox);
 	gtk_box_pack_start(GTK_BOX(pxbaf->hbox), pxbaf->aframe, TRUE, TRUE, 0);
-	gtk_widget_show(pxbaf->aframe);
 }
 
 /* 1998-09-26 -	Build the extra configuration widgetry for external commands. Plenty of stuff. */
@@ -897,11 +880,9 @@ static void build_px_external(P_CmdSeq *page, PX_External *pxe)
 	gtk_notebook_set_tab_pos(GTK_NOTEBOOK(pxe->nbook), GTK_POS_LEFT);
 	build_pxe_general(page, &pxe->gflags);
 	gtk_notebook_append_page(GTK_NOTEBOOK(pxe->nbook), pxe->gflags.vbox, gtk_label_new(_("General")));
-	gtk_widget_show(pxe->gflags.vbox);
 
 	build_px_baf(page, &pxe->baf);
 	gtk_notebook_append_page(GTK_NOTEBOOK(pxe->nbook), pxe->baf.hbox, gtk_label_new(_("Before&After")));
-	gtk_widget_show(pxe->baf.hbox);
 }
 
 static GtkWidget * ccs_init(MainInfo *min, gchar **name)
@@ -915,34 +896,23 @@ static GtkWidget * ccs_init(MainInfo *min, gchar **name)
 	GtkWidget	*label, *vbox, *hbox, *vbox2, *sep;
 	P_CmdSeq	*page = &the_page;
 
-	if(name == NULL)
-		return NULL;
-
-	*name = _("Commands");
-
 	page->vbox = gtk_vbox_new(FALSE, 0);
 
 	page->scwin = gtk_scrolled_window_new(NULL, NULL);
 	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(page->scwin), GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
 	page->clist = gtk_clist_new(1);
-	gtk_object_set_user_data(GTK_OBJECT(page->clist), (gpointer) page);
-	gtk_signal_connect(GTK_OBJECT(page->clist), "select_row", GTK_SIGNAL_FUNC(evt_cmdseq_selected), (gpointer) page);
-	gtk_signal_connect(GTK_OBJECT(page->clist), "unselect_row", GTK_SIGNAL_FUNC(evt_cmdseq_unselected), (gpointer) page);
+	gtk_signal_connect(GTK_OBJECT(page->clist), "select_row", GTK_SIGNAL_FUNC(evt_cmdseq_selected), page);
+	gtk_signal_connect(GTK_OBJECT(page->clist), "unselect_row", GTK_SIGNAL_FUNC(evt_cmdseq_unselected), page);
 	gtk_container_add(GTK_CONTAINER(page->scwin), page->clist);
-	gtk_widget_show(page->clist);
 	gtk_box_pack_start(GTK_BOX(page->vbox), page->scwin, TRUE, TRUE, 0);
-	gtk_widget_show(page->scwin);
 
 	page->nhbox = gtk_hbox_new(FALSE, 0);
 	label = gtk_label_new(_("Name"));
 	gtk_box_pack_start(GTK_BOX(page->nhbox), label, FALSE, FALSE, 0);
-	gtk_widget_show(label);
 	page->name = gtk_entry_new_with_max_length(CSQ_NAME_SIZE - 1);
-	gtk_signal_connect(GTK_OBJECT(page->name), "changed", GTK_SIGNAL_FUNC(evt_name_changed), (gpointer) page);
+	gtk_signal_connect(GTK_OBJECT(page->name), "changed", GTK_SIGNAL_FUNC(evt_name_changed), page);
 	gtk_box_pack_start(GTK_BOX(page->nhbox), page->name, TRUE, TRUE, 0);
-	gtk_widget_show(page->name);
 	gtk_box_pack_start(GTK_BOX(page->vbox), page->nhbox, FALSE, FALSE, 0);
-	gtk_widget_show(page->nhbox);
 
 	page->dframe = gtk_frame_new(_("Definition"));
 	vbox = gtk_vbox_new(FALSE, 0);
@@ -959,55 +929,43 @@ static GtkWidget * ccs_init(MainInfo *min, gchar **name)
 			for(; dblab[i][0] == '('; i++)
 			{
 				page->dbtn[i] = gui_arrow_button_new(dblab[i][1] == 'U' ? GTK_ARROW_UP : GTK_ARROW_DOWN);
-				gtk_signal_connect(GTK_OBJECT(page->dbtn[i]), "clicked", dbfunc[i], (gpointer) page);
+				gtk_signal_connect(GTK_OBJECT(page->dbtn[i]), "clicked", dbfunc[i], page);
 				gtk_box_pack_start(GTK_BOX(hbox), page->dbtn[i], TRUE, TRUE, 0);
-				gtk_widget_show(page->dbtn[i]);
 			}
 			gtk_box_pack_start(GTK_BOX(vbox2), hbox, FALSE, FALSE, 0);
-			gtk_widget_show(hbox);
 			i--;		/* Compensate for outermost for loop. Told you it was gross... */
 		}
 		else
 		{
 			page->dbtn[i] = gtk_button_new_with_label(_(dblab[i]));
-			gtk_signal_connect(GTK_OBJECT(page->dbtn[i]), "clicked", dbfunc[i], (gpointer) page);
+			gtk_signal_connect(GTK_OBJECT(page->dbtn[i]), "clicked", dbfunc[i], page);
 			gtk_box_pack_start(GTK_BOX(vbox2), page->dbtn[i], FALSE, FALSE, 0);
-			gtk_widget_show(page->dbtn[i]);
 		}
 	}
 	gtk_box_pack_start(GTK_BOX(hbox), vbox2, FALSE, FALSE, 0);
-	gtk_widget_show(vbox2);
 
 	page->dscwin = gtk_scrolled_window_new(NULL, NULL);
 	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(page->dscwin), GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
 	page->dclist = gtk_clist_new(2);
 	gtk_clist_set_column_width(GTK_CLIST(page->dclist), 0, 56);
-	gtk_object_set_user_data(GTK_OBJECT(page->dclist), (gpointer) page);
-	gtk_signal_connect(GTK_OBJECT(page->dclist), "select_row", GTK_SIGNAL_FUNC(evt_row_selected), (gpointer) page);
-	gtk_signal_connect(GTK_OBJECT(page->dclist), "unselect_row", GTK_SIGNAL_FUNC(evt_row_unselected), (gpointer) page);
+	gtk_signal_connect(GTK_OBJECT(page->dclist), "select_row", GTK_SIGNAL_FUNC(evt_row_selected), page);
+	gtk_signal_connect(GTK_OBJECT(page->dclist), "unselect_row", GTK_SIGNAL_FUNC(evt_row_unselected), page);
 	gtk_container_add(GTK_CONTAINER(page->dscwin), page->dclist);
-	gtk_widget_show(page->dclist);
 	gtk_box_pack_start(GTK_BOX(hbox), page->dscwin, TRUE, TRUE, 0);
-	gtk_widget_show(page->dscwin);
 	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 5);
-	gtk_widget_show(hbox);
 
 	page->dhbox = gtk_hbox_new(FALSE, 0);
-	page->dtmenu = gui_build_menu(tlab, GTK_SIGNAL_FUNC(evt_type_selected), (gpointer) page);
+	page->dtmenu = gui_build_menu(tlab, GTK_SIGNAL_FUNC(evt_type_selected), page);
 	page->dtype = gtk_option_menu_new();
 	gtk_option_menu_set_menu(GTK_OPTION_MENU(page->dtype), page->dtmenu);
 	gtk_box_pack_start(GTK_BOX(page->dhbox), page->dtype, FALSE, FALSE, 0);
-	gtk_widget_show(page->dtype);
 	page->ddef = gtk_entry_new();
-	gtk_signal_connect(GTK_OBJECT(page->ddef), "changed", GTK_SIGNAL_FUNC(evt_def_changed), (gpointer) page);
+	gtk_signal_connect(GTK_OBJECT(page->ddef), "changed", GTK_SIGNAL_FUNC(evt_def_changed), page);
 	gtk_box_pack_start(GTK_BOX(page->dhbox), page->ddef, TRUE, TRUE, 0);
-	gtk_widget_show(page->ddef);
 	page->dpick = gui_details_button_new(min->gui->window->window);
-	gtk_signal_connect(GTK_OBJECT(page->dpick), "clicked", GTK_SIGNAL_FUNC(evt_pick_clicked), (gpointer) page);
+	gtk_signal_connect(GTK_OBJECT(page->dpick), "clicked", GTK_SIGNAL_FUNC(evt_pick_clicked), page);
 	gtk_box_pack_start(GTK_BOX(page->dhbox), page->dpick, FALSE, FALSE, 0);
-	gtk_widget_show(page->dpick);	
 	gtk_box_pack_start(GTK_BOX(vbox), page->dhbox, FALSE, FALSE, 0);
-	gtk_widget_show(page->dhbox);
 
 	page->dextra = gtk_notebook_new();
 	gtk_notebook_set_show_tabs(GTK_NOTEBOOK(page->dextra), FALSE);
@@ -1015,42 +973,34 @@ static GtkWidget * ccs_init(MainInfo *min, gchar **name)
 
 	build_px_builtin(page, &page->px_builtin);
 	gtk_notebook_append_page(GTK_NOTEBOOK(page->dextra), page->px_builtin.vbox, NULL);
-	gtk_widget_show(page->px_builtin.vbox);
 
 	build_px_external(page, &page->px_external);	
 	gtk_notebook_append_page(GTK_NOTEBOOK(page->dextra), page->px_external.nbook, NULL);
-	gtk_widget_show(page->px_external.nbook);
 	gtk_box_pack_start(GTK_BOX(vbox), page->dextra, FALSE, FALSE, 0);
-	gtk_widget_show(page->dextra);
 
 	sep = gtk_hseparator_new();
 	gtk_box_pack_start(GTK_BOX(vbox), sep, FALSE, FALSE, 5);
-	gtk_widget_show(sep);
 	page->drepeat = gtk_check_button_new_with_label(_("Repeat Sequence Until No Source Selection?"));
 	gtk_signal_connect(GTK_OBJECT(page->drepeat), "clicked", GTK_SIGNAL_FUNC(evt_repeat_clicked), page);
 	gtk_box_pack_start(GTK_BOX(vbox), page->drepeat, FALSE, FALSE, 0);
-	gtk_widget_show(page->drepeat);
 
 	gtk_container_add(GTK_CONTAINER(page->dframe), vbox);
-	gtk_widget_show(vbox);
 	gtk_box_pack_start(GTK_BOX(page->vbox), page->dframe, FALSE, FALSE, 0);
-	gtk_widget_show(page->dframe);
 
 	page->bhbox = gtk_hbox_new(FALSE, 0);
 	page->badd = gtk_button_new_with_label(_("Add"));
-	gtk_signal_connect(GTK_OBJECT(page->badd), "clicked", GTK_SIGNAL_FUNC(evt_add_clicked), (gpointer) page);
+	gtk_signal_connect(GTK_OBJECT(page->badd), "clicked", GTK_SIGNAL_FUNC(evt_add_clicked), page);
 	gtk_box_pack_start(GTK_BOX(page->bhbox), page->badd, TRUE, TRUE, 5);
-	gtk_widget_show(page->badd);
 	page->bdel = gtk_button_new_with_label(_("Delete"));
-	gtk_signal_connect(GTK_OBJECT(page->bdel), "clicked", GTK_SIGNAL_FUNC(evt_del_clicked), (gpointer) page);
+	gtk_signal_connect(GTK_OBJECT(page->bdel), "clicked", GTK_SIGNAL_FUNC(evt_del_clicked), page);
 	gtk_box_pack_start(GTK_BOX(page->bhbox), page->bdel, TRUE, TRUE, 5);
-	gtk_widget_show(page->bdel);
 	gtk_box_pack_start(GTK_BOX(page->vbox), page->bhbox, FALSE, FALSE, 5);
-	gtk_widget_show(page->bhbox);
 
-	gtk_widget_show(page->vbox);
+	gtk_widget_show_all(page->vbox);
 
-	return page->vbox;
+	cfg_tree_level_begin(_("Commands"));
+	cfg_tree_level_append(_("Definitions"), page->vbox);	/* Rely on cfg_cmdcfg.c to close level. */
+	return NULL;
 }
 
 /* ----------------------------------------------------------------------------------------- */
@@ -1124,8 +1074,8 @@ static void save_cmdrow(CmdRow *row, FILE *out)
 /* 1998-09-27 -	Save out a single command sequence. */
 static void save_cmdseq(gpointer key, gpointer data, gpointer user)
 {
-	CmdSeq	*cs = (CmdSeq *) data;
-	FILE	*out = (FILE *) user;
+	CmdSeq	*cs = data;
+	FILE	*out = user;
 	GList	*iter;
 
 	xml_put_node_open(out, "CmdSeq");
@@ -1135,7 +1085,7 @@ static void save_cmdseq(gpointer key, gpointer data, gpointer user)
 	{
 		xml_put_node_open(out, "CmdRows");
 		for(iter = cs->rows; iter != NULL; iter = g_list_next(iter))
-			save_cmdrow((CmdRow *) iter->data, out);
+			save_cmdrow(iter->data, out);
 		xml_put_node_close(out, "CmdRows");
 	}
 	xml_put_node_close(out, "CmdSeq");
@@ -1145,7 +1095,7 @@ static void save_cmdseq(gpointer key, gpointer data, gpointer user)
 static gint ccs_save(MainInfo *min, FILE *out)
 {
 	xml_put_node_open(out, NODE);
-	g_hash_table_foreach(min->cfg.commands.cmdseq, save_cmdseq, (gpointer) out);
+	g_hash_table_foreach(min->cfg.commands.cmdseq, save_cmdseq, out);
 	xml_put_node_close(out, NODE);
 	return TRUE;
 }
@@ -1163,7 +1113,7 @@ static void load_external(CX_Ext *cext, const XmlNode *node)
 /* 1998-09-27 -	Load a single command sequence row. */
 static void load_cmdrow(const XmlNode *node, gpointer user)
 {
-	CmdSeq		*cs = (CmdSeq *) user;
+	CmdSeq		*cs = user;
 	CmdRow		*row;
 	const XmlNode	*data;
 	CRType		type = CRTP_BUILTIN;
@@ -1186,13 +1136,13 @@ static void load_cmdrow(const XmlNode *node, gpointer user)
 /* 1998-09-27 -	Trampoline. */
 static void load_cmdseq_rows(const XmlNode *node, CmdSeq *cs)
 {
-	xml_node_visit_children(node, load_cmdrow, (gpointer) cs);
+	xml_node_visit_children(node, load_cmdrow, cs);
 }
 
 /* 1998-09-27 -	Load a command sequence. */
 static void load_cmdseq(const XmlNode *node, gpointer user)
 {
-	MainInfo	*min = (MainInfo *) user;
+	MainInfo	*min = user;
 	const gchar	*name = "Unknown";
 	guint32		flags = 0UL;
 	CmdSeq		*cs;
@@ -1218,14 +1168,14 @@ static void ccs_load(MainInfo *min, const XmlNode *node)
 		g_hash_table_destroy(min->cfg.commands.cmdseq);
 	}
 	min->cfg.commands.cmdseq = NULL;
-	xml_node_visit_children(node, load_cmdseq, (gpointer) min);
+	xml_node_visit_children(node, load_cmdseq, min);
 }
 
 /* ----------------------------------------------------------------------------------------- */
 
-CfgPage * ccs_describe(MainInfo *min)
+const CfgModule * ccs_describe(MainInfo *min)
 {
-	static CfgPage	desc = { NODE, ccs_init, ccs_update, ccs_accept, ccs_save, ccs_load, NULL };
+	static const CfgModule	desc = { NODE, ccs_init, ccs_update, ccs_accept, ccs_save, ccs_load, NULL };
 
 	return &desc;
 }
