@@ -14,7 +14,7 @@
 #include "guiutil.h"
 #include "strutil.h"
 
-#include "cfg_gui.h"
+#include "configure.h"
 #include "cfg_module.h"
 #include "cfg_cmdseq.h"				/* For ccs_current(). */
 #include "cfg_controls.h"
@@ -153,8 +153,8 @@ static void populate_key_clist(P_Controls *page)
 
 		for(iter = list; iter != NULL; iter = g_slist_next(iter))
 		{
-			crow[0] = (gchar *) ctrl_key_get_keyname((CtrlKey *) iter->data);
-			crow[1] = (gchar *) ctrl_key_get_cmdseq((CtrlKey *) iter->data);
+			crow[0] = (gchar *) ctrl_key_get_keyname(iter->data);
+			crow[1] = (gchar *) ctrl_key_get_cmdseq(iter->data);
 			row = gtk_clist_append(GTK_CLIST(page->kclist), crow);
 			gtk_clist_set_row_data(GTK_CLIST(page->kclist), row, iter->data);
 		}
@@ -283,7 +283,6 @@ static gint evt_kadd_clicked(GtkWidget *wid, gpointer user)
 		gtk_widget_grab_focus(page->kdkey);
 		page->modified = TRUE;
 	}
-
 	return TRUE;
 }
 
@@ -345,26 +344,29 @@ static void evt_mstate_clicked(GtkWidget *wid, gpointer user)
 {
 	P_Controls	*page = user;
 	Dialog		*dlg;
-	guint		i, mask[] = { GDK_SHIFT_MASK, GDK_CONTROL_MASK, GDK_MOD1_MASK };
-	GtkWidget	*vbox, *label, *check[sizeof mask / sizeof mask[0]];
+	guint		i, x, y,
+			mask[] = { GDK_SHIFT_MASK, GDK_CONTROL_MASK, GDK_MOD1_MASK, GDK_MOD2_MASK,
+				   GDK_MOD3_MASK, GDK_MOD4_MASK, GDK_MOD5_MASK };
+	GtkWidget	*table, *label, *check[sizeof mask / sizeof mask[0]];
 
 	if((page == NULL) || (page->curr_mouse == NULL))
 		return;
 
-	vbox = gtk_vbox_new(FALSE, 0);
-	label = gtk_label_new(_("The following modifier key(s) must\nbe held down when the mouse button"
-				"\n is clicked to trigger the command:"));
-	gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);
-	gtk_widget_show(label);
-	for(i = 0; i < sizeof mask / sizeof mask[0]; i++)
+	table = gtk_table_new(1 + (sizeof mask / sizeof *mask) / 2 + 1, 2, FALSE);
+	label = gtk_label_new(_("The following modifier key(s) must\n"
+				"be held down when the mouse button\n"
+				"is clicked to trigger the command"));
+	gtk_table_attach(GTK_TABLE(table), label, 0, 2, 0, 1,  GTK_EXPAND|GTK_FILL,0,0,0);
+	for(i = 0, x = 0, y = 1; i < sizeof mask / sizeof mask[0]; i++, y++)
 	{
 		check[i] = gtk_check_button_new_with_label(gtk_accelerator_name(0, mask[i]));
 		if(ctrl_mouse_get_state(page->curr_mouse) & mask[i])
 			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check[i]), TRUE);
-		gtk_box_pack_start(GTK_BOX(vbox), check[i], FALSE, FALSE, 0);
-		gtk_widget_show(check[i]);
+		gtk_table_attach(GTK_TABLE(table), check[i], x, x + 1, y, y + 1,  GTK_EXPAND|GTK_FILL,0,0,0);
+		if(i == (sizeof mask / sizeof *mask) / 2)
+			x++, y = 0;
 	}
-	dlg = dlg_dialog_sync_new(vbox, _("Edit Modifiers"), _("OK|Cancel"));
+	dlg = dlg_dialog_sync_new(table, _("Edit Modifiers"), NULL);
 	if(dlg_dialog_sync_wait(dlg) == DLG_POSITIVE)
 	{
 		guint	ns = 0U;
@@ -467,36 +469,30 @@ static void evt_nonumlock_clicked(GtkWidget *wid, gpointer user)
 
 static void build_mouse_menu(P_Controls *page)
 {
-	gchar		**gfx[] = { icon_mouse1_xpm, icon_mouse2_xpm, icon_mouse3_xpm, icon_mouse4_xpm, icon_mouse5_xpm },
+	gchar		**gfx[]  = { icon_mouse1_xpm, icon_mouse2_xpm, icon_mouse3_xpm, icon_mouse4_xpm, icon_mouse5_xpm },
 			*label[] = { N_("Left"), N_("Middle"), N_("Right"), N_("Wheel Up"), N_("Wheel Down") };
 	GtkWidget	*menu, *item, *ihbox, *pmap, *lab;
 	guint		i;
 
 	menu = gtk_menu_new();
-	gtk_menu_set_title(GTK_MENU(menu), _("Menu"));
 	for(i = 0; i < 5; i++)
 	{
-		item = gtk_menu_item_new();
+		item  = gtk_menu_item_new();
 		ihbox = gtk_hbox_new(FALSE, 0);
 		page->mpix[i] = gdk_pixmap_colormap_create_from_xpm_d(NULL, gtk_widget_get_default_colormap(), &page->mmsk[i], NULL, gfx[i]);
 		pmap = gtk_pixmap_new(page->mpix[i], page->mmsk[i]);
 		gtk_box_pack_start(GTK_BOX(ihbox), pmap, FALSE, FALSE, 0);
-		gtk_widget_show(pmap);
 		lab = gtk_label_new(_(label[i]));
 		gtk_box_pack_start(GTK_BOX(ihbox), lab, FALSE, FALSE, 0);
-		gtk_widget_show(lab);
 		gtk_container_add(GTK_CONTAINER(item), ihbox);
-		gtk_widget_show(ihbox);
 		gtk_menu_append(GTK_MENU(menu), item);
-		gtk_widget_show(item);
 		gtk_object_set_user_data(GTK_OBJECT(item), GUINT_TO_POINTER(i));
 		gtk_signal_connect(GTK_OBJECT(item), "activate", GTK_SIGNAL_FUNC(evt_mbutton_activated), page);
+		gtk_widget_show_all(item);
 	}
 	page->mdmenu = gtk_option_menu_new();
 	gtk_option_menu_set_menu(GTK_OPTION_MENU(page->mdmenu), menu);
-	gtk_widget_show(menu);
 	gtk_table_attach(GTK_TABLE(page->mdtable), page->mdmenu, 1, 2, 0, 1,  GTK_FILL|GTK_EXPAND,0,0,0);
-	gtk_widget_show(page->mdmenu);
 }
 
 static GtkWidget * cct_init(MainInfo *min, gchar **name)
@@ -506,7 +502,6 @@ static GtkWidget * cct_init(MainInfo *min, gchar **name)
 
 	if(name == NULL)
 		return NULL;
-
 	*name = _("Controls");
 
 	page->min = min;
@@ -514,7 +509,6 @@ static GtkWidget * cct_init(MainInfo *min, gchar **name)
 
 	page->vbox = gtk_vbox_new(FALSE, 0);
 
-
 	page->kframe = gtk_frame_new(_("Global Keyboard Shortcuts"));
 	vbox = gtk_vbox_new(FALSE, 0);
 	page->kscwin = gtk_scrolled_window_new(NULL, NULL);
@@ -628,7 +622,7 @@ static void cct_accept(MainInfo *min)
 						    "are ambiguous: the same button+modifier\n"
 						    "is used for more than one function. This\n"
 						    "might make their behaviour pretty weird..."),
-							_("Warning"), _("OK"), NULL, NULL);
+							_("Warning"), _("_OK"), NULL, NULL);
 		ctrl_destroy(min->cfg.ctrlinfo);
 		min->cfg.ctrlinfo = page->ctrlinfo;
 		page->ctrlinfo = NULL;
@@ -649,8 +643,8 @@ static void keys_save(CtrlInfo *ctrl, FILE *out)
 		for(iter = keys; iter != NULL; iter = g_slist_next(iter))
 		{
 			xml_put_node_open(out, "Key");
-			xml_put_text(out, "keyname", ctrl_key_get_keyname((CtrlKey *) iter->data));
-			xml_put_text(out, "cmdseq", ctrl_key_get_cmdseq((CtrlKey *) iter->data));
+			xml_put_text(out, "keyname", ctrl_key_get_keyname(iter->data));
+			xml_put_text(out, "cmdseq", ctrl_key_get_cmdseq(iter->data));
 			xml_put_node_close(out, "Key");
 		}
 		g_slist_free(keys);
@@ -751,9 +745,9 @@ static void cct_load(MainInfo *min, const XmlNode *node)
 
 /* ----------------------------------------------------------------------------------------- */	
 
-CfgPage * cct_describe(MainInfo *min)
+const CfgModule * cct_describe(MainInfo *min)
 {
-	static CfgPage	desc = { NODE, cct_init, cct_update, cct_accept, cct_save, cct_load, NULL };
+	static const CfgModule	desc = { NODE, cct_init, cct_update, cct_accept, cct_save, cct_load, NULL };
 
 	return &desc;
 }
