@@ -33,7 +33,7 @@
 #include "color_dialog.h"
 #include "icon_dialog.h"
 
-#include "cfg_gui.h"
+#include "configure.h"
 #include "cfg_module.h"
 #include "cfg_paths.h"		/* For cpt_get_path(). */
 #include "cfg_types.h"		/* For ctp_get_types(). */
@@ -106,6 +106,11 @@ static void set_widgets_preview(P_Styles *page, Style *stl)
 	}
 	if((col = stl_style_property_get_color(stl, SPN_COL_UNSEL_BG)) != NULL)
 		gtk_clist_set_background(GTK_CLIST(page->dpvisual.preview), 0, (GdkColor *) col);
+	else
+	{
+		col = &gtk_widget_get_style(GTK_WIDGET(page->min->gui->pane[0].list))->base[GTK_STATE_NORMAL];
+		gtk_clist_set_background(GTK_CLIST(page->dpvisual.preview), 0, (GdkColor *) col);
+	}
 	if((col = stl_style_property_get_color(stl, SPN_COL_UNSEL_FG)) != NULL)
 		gtk_clist_set_foreground(GTK_CLIST(page->dpvisual.preview), 0, (GdkColor *) col);
 }
@@ -156,7 +161,7 @@ static void set_widgets(P_Styles *page, Style *stl)
 	{
 		or = stl_style_property_get_override(stl, vpname[i]);
 		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(page->dpvisual.override[i]), or);
-		gtk_widget_set_sensitive(page->dpvisual.override[i], pname != NULL);
+		gtk_widget_set_sensitive(page->dpvisual.override[i], (i < 1) || (pname != NULL));
 		gtk_widget_set_sensitive(page->dpvisual.edit[i], or);
 	}
 	set_widgets_action(page, stl);
@@ -292,7 +297,14 @@ static void evt_vprop_override_toggled(GtkWidget *wid, gpointer user)
 			if(strcmp(pname, SPN_ICON_UNSEL) == 0)
 				stl_style_property_set_icon(page->curr_style, pname, stl_style_property_get_icon(page->curr_style, pname));
 			else
-				stl_style_property_set_color(page->curr_style, pname, stl_style_property_get_color(page->curr_style, pname));
+			{
+				const GdkColor	*def;
+
+				/* FIXME: This assumes only background can be non-overridden in root. */
+				if((def = stl_style_property_get_color(page->curr_style, pname)) == NULL)
+					def = &(gtk_widget_get_style(GTK_WIDGET(page->min->gui->pane[0].list))->base[GTK_STATE_NORMAL]);
+				stl_style_property_set_color(page->curr_style, pname, def);
+			}
 		}
 		else
 			stl_style_property_remove(page->curr_style, pname);
@@ -448,12 +460,9 @@ static void evt_aprop_add_clicked(GtkWidget *wid, gpointer user)
 		hbox = gtk_hbox_new(FALSE, 0);
 		label = gtk_label_new(_("Name"));
 		gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
-		gtk_widget_show(label);
 		entry = gtk_entry_new_with_max_length(STL_PROPERTY_NAME_SIZE - 1);
 		gtk_box_pack_start(GTK_BOX(hbox), entry, TRUE, TRUE, 0);
-		gtk_widget_show(entry);
-		gtk_widget_show(hbox);
-		dlg = dlg_dialog_sync_new(hbox, _("New Action Property"), _("OK|Cancel"));
+		dlg = dlg_dialog_sync_new(hbox, _("New Action Property"), NULL);
 		gtk_widget_grab_focus(entry);
 		if(dlg_dialog_sync_wait(dlg) == DLG_POSITIVE)
 		{
@@ -525,7 +534,7 @@ static void evt_style_del_clicked(GtkWidget *wid, gpointer user)
 
 		if(stl_styleinfo_style_has_children(page->si, page->curr_style))
 			ok = dlg_dialog_sync_new_simple_wait(_("Deleting this style will also delete\n"
-				"all its children. Are you sure?"), _("Confirm Delete"), _("Delete|Cancel"));
+				"all its children. Are you sure?"), _("Confirm Delete"), _("_Delete|_Cancel"));
 		if(ok == DLG_POSITIVE)
 		{
 			GList	*chlist, *iter;
@@ -559,7 +568,6 @@ static void build_pvisual(P_Styles *page)
 	hbox = gtk_hbox_new(FALSE, 0);
 	label = gtk_label_new(_("Preview"));
 	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
-	gtk_widget_show(label);
 	pv->preview = gtk_clist_new(2);
 	gtk_widget_set_name(pv->preview, "cstPreview");
 	gtk_clist_set_column_width(GTK_CLIST(pv->preview), 0, 16);
@@ -567,9 +575,7 @@ static void build_pvisual(P_Styles *page)
 	gtk_clist_append(GTK_CLIST(pv->preview), ctxt);
 	gtk_clist_set_selectable(GTK_CLIST(pv->preview), 0, FALSE);
 	gtk_box_pack_start(GTK_BOX(hbox), pv->preview, TRUE, TRUE, 0);
-	gtk_widget_show(pv->preview);
 	gtk_box_pack_start(GTK_BOX(pv->vbox), hbox, FALSE, FALSE, 2);
-	gtk_widget_show(hbox);
 
 	hbox = gtk_hbox_new(FALSE, 0);
 	for(i = 0; i < 3; i++)
@@ -581,19 +587,14 @@ static void build_pvisual(P_Styles *page)
 		gtk_object_set_user_data(GTK_OBJECT(pv->override[i]), (gpointer) vpname[i]);
 		gtk_signal_connect(GTK_OBJECT(pv->override[i]), "toggled", GTK_SIGNAL_FUNC(evt_vprop_override_toggled), page);
 		gtk_box_pack_start(GTK_BOX(vbox), pv->override[i], FALSE, FALSE, 0);
-		gtk_widget_show(pv->override[i]);
 		pv->edit[i] = gtk_button_new_with_label(i < 2 ? _("Edit...") : _("Pick..."));
 		gtk_object_set_user_data(GTK_OBJECT(pv->edit[i]), (gpointer) vpname[i]);
 		gtk_signal_connect(GTK_OBJECT(pv->edit[i]), "clicked", GTK_SIGNAL_FUNC(evt_vprop_edit_clicked), page);
 		gtk_box_pack_start(GTK_BOX(vbox), pv->edit[i], FALSE, FALSE, 0);
-		gtk_widget_show(pv->edit[i]);
 		gtk_container_add(GTK_CONTAINER(frame), vbox);
-		gtk_widget_show(vbox);
 		gtk_box_pack_start(GTK_BOX(hbox), frame, TRUE, TRUE, 5);
-		gtk_widget_show(frame);
 	}
 	gtk_box_pack_start(GTK_BOX(pv->vbox), hbox, FALSE, FALSE, 0);
-	gtk_widget_show(hbox);
 }
 
 /* 1999-05-22 -	Build action property editing widgetry. */
@@ -624,43 +625,32 @@ static void build_paction(P_Styles *page)
 	gtk_signal_connect(GTK_OBJECT(pa->actions), "select_row", GTK_SIGNAL_FUNC(evt_aprop_selected), page);
 	gtk_signal_connect(GTK_OBJECT(pa->actions), "unselect_row", GTK_SIGNAL_FUNC(evt_aprop_unselected), page);
 	gtk_container_add(GTK_CONTAINER(scwin), pa->actions);
-	gtk_widget_show(pa->actions);
 	gtk_box_pack_start(GTK_BOX(pa->vbox), scwin, TRUE, TRUE, 0);
-	gtk_widget_show(scwin);
 
 	table = gtk_table_new(2, 3, FALSE);
 	label = gtk_label_new(_("Name"));
 	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 0, 1,  0,0,0,0);
-/*	gtk_widget_show(label);*/
 	pa->aname = gtk_entry_new_with_max_length(STL_PROPERTY_NAME_SIZE - 1);
 	gtk_entry_set_editable(GTK_ENTRY(pa->aname), FALSE);
 	gtk_table_attach(GTK_TABLE(table), pa->aname, 1, 3, 0, 1,  GTK_EXPAND|GTK_FILL,0,0,0);
-/*	gtk_widget_show(pa->aname);*/
 	label = gtk_label_new(_("Command"));
 	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 1, 2,  0,0,0,0);
-	gtk_widget_show(label);
 	pa->acmdseq = gtk_entry_new();
 	gtk_signal_connect(GTK_OBJECT(pa->acmdseq), "changed", GTK_SIGNAL_FUNC(evt_aprop_cmdseq_changed), page);
 	gtk_table_attach(GTK_TABLE(table), pa->acmdseq, 1, 2, 1, 2,  GTK_EXPAND|GTK_FILL,0,0,0);
 	pa->acmdpick = gui_details_button_new(page->min->gui->window->window);
 	gtk_signal_connect(GTK_OBJECT(pa->acmdpick), "clicked", GTK_SIGNAL_FUNC(evt_aprop_cmdpick_clicked), page);
 	gtk_table_attach(GTK_TABLE(table), pa->acmdpick, 2, 3, 1, 2,  0,0,0,0);
-	gtk_widget_show(pa->acmdpick);
-	gtk_widget_show(pa->acmdseq);
 	gtk_box_pack_start(GTK_BOX(pa->vbox), table, FALSE, FALSE, 0);
-	gtk_widget_show(table);
 
 	hbox = gtk_hbox_new(FALSE, 0);
 	btn = gtk_button_new_with_label(_("Add Action..."));
 	gtk_signal_connect(GTK_OBJECT(btn), "clicked", GTK_SIGNAL_FUNC(evt_aprop_add_clicked), page);
 	gtk_box_pack_start(GTK_BOX(hbox), btn, TRUE, TRUE, 5);
-	gtk_widget_show(btn);
 	pa->adel = gtk_button_new_with_label(_("Delete Action"));
 	gtk_signal_connect(GTK_OBJECT(pa->adel), "clicked", GTK_SIGNAL_FUNC(evt_aprop_delete_clicked), page);
 	gtk_box_pack_start(GTK_BOX(hbox), pa->adel, TRUE, TRUE, 5);
-	gtk_widget_show(pa->adel);
 	gtk_box_pack_start(GTK_BOX(pa->vbox), hbox, FALSE, FALSE, 5);
-	gtk_widget_show(hbox);
 }
 
 /* 1999-05-22 -	Build style configuration GUI page. */
@@ -669,11 +659,6 @@ static GtkWidget * cst_init(MainInfo *min, gchar **name)
 	P_Styles	*page = &the_page;
 	GtkWidget	*table, *label, *frame, *hbox, *btn;
 
-	if(name == NULL)
-		return NULL;
-
-	*name = _("File Styles");
-
 	page->min = min;
 	page->si  = NULL;
 	page->curr_style = NULL;
@@ -684,65 +669,52 @@ static GtkWidget * cst_init(MainInfo *min, gchar **name)
 	page->scwin = gtk_scrolled_window_new(FALSE, FALSE);
 	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(page->scwin), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
 	gtk_box_pack_start(GTK_BOX(page->vbox), page->scwin, TRUE, TRUE, 0);
-	gtk_widget_show(page->scwin);
 
 	page->dvbox = gtk_vbox_new(FALSE, 0);
 
 	table = gtk_table_new(2, 3, FALSE);
 	label = gtk_label_new(_("Name"));
 	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 0, 1,  0,0,0,0);
-	gtk_widget_show(label);
 	page->dname = gtk_entry_new_with_max_length(STL_STYLE_NAME_SIZE - 1);
 	gtk_signal_connect(GTK_OBJECT(page->dname), "changed", GTK_SIGNAL_FUNC(evt_style_name_changed), page);
 	gtk_table_attach(GTK_TABLE(table), page->dname, 1, 3, 0, 1,  GTK_EXPAND|GTK_FILL,0,0,0);
-	gtk_widget_show(page->dname);
 	label = gtk_label_new(_("Parent"));
 	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 1, 2,  0,0,0,0);
-	gtk_widget_show(label);
 	page->dparent = gtk_entry_new_with_max_length(STL_STYLE_NAME_SIZE - 1);
 	gtk_entry_set_editable(GTK_ENTRY(page->dparent), FALSE);
 	gtk_table_attach(GTK_TABLE(table), page->dparent, 1, 2, 1, 2,  GTK_EXPAND|GTK_FILL,0,0,0);
-	gtk_widget_show(page->dparent);
 	page->dreparent = gui_details_button_new(min->gui->window->window);
 	gtk_signal_connect(GTK_OBJECT(page->dreparent), "clicked", GTK_SIGNAL_FUNC(evt_style_parent_clicked), page);
 	gtk_table_attach(GTK_TABLE(table), page->dreparent, 2, 3, 1, 2,  0,0,0,0);
-	gtk_widget_show(page->dreparent);
 	gtk_box_pack_start(GTK_BOX(page->dvbox), table, FALSE, FALSE, 0);
-	gtk_widget_show(table);
 
 	frame = gtk_frame_new(_("Inherited Properties"));
 	page->dpnbook = gtk_notebook_new();
 	gtk_notebook_set_tab_pos(GTK_NOTEBOOK(page->dpnbook), GTK_POS_LEFT);
 	build_pvisual(page);
 	gtk_notebook_append_page(GTK_NOTEBOOK(page->dpnbook), page->dpvisual.vbox, gtk_label_new(_("Visual")));
-	gtk_widget_show(page->dpvisual.vbox);
 	build_paction(page);
 	gtk_notebook_append_page(GTK_NOTEBOOK(page->dpnbook), page->dpaction.vbox, gtk_label_new(_("Actions")));
-	gtk_widget_show(page->dpaction.vbox);
 	gtk_container_add(GTK_CONTAINER(frame), page->dpnbook);
-	gtk_widget_show(page->dpnbook);
 
 	gtk_box_pack_start(GTK_BOX(page->dvbox), frame, FALSE, FALSE, 5);
-	gtk_widget_show(frame);
-
 	gtk_box_pack_start(GTK_BOX(page->vbox), page->dvbox, FALSE, FALSE, 0);
-	gtk_widget_show(page->dvbox);
 
 	hbox = gtk_hbox_new(FALSE, 0);
 	btn = gtk_button_new_with_label(_("Add"));
 	gtk_signal_connect(GTK_OBJECT(btn), "clicked", GTK_SIGNAL_FUNC(evt_style_add_clicked), page);
 	gtk_box_pack_start(GTK_BOX(hbox), btn, TRUE, TRUE, 5);
-	gtk_widget_show(btn);
 	page->del = gtk_button_new_with_label(_("Delete"));
 	gtk_signal_connect(GTK_OBJECT(page->del), "clicked", GTK_SIGNAL_FUNC(evt_style_del_clicked), page);
 	gtk_box_pack_start(GTK_BOX(hbox), page->del, TRUE, TRUE, 5);
-	gtk_widget_show(page->del);
 	gtk_box_pack_start(GTK_BOX(page->vbox), hbox, FALSE, FALSE, 5);
-	gtk_widget_show(hbox);
 
-	gtk_widget_show(page->vbox);
+	gtk_widget_show_all(page->vbox);
 
-	return page->vbox;
+	cfg_tree_level_begin(_("File Recognition"));
+	cfg_tree_level_append(_("Styles"), page->vbox);
+
+	return NULL;
 }
 
 /* ----------------------------------------------------------------------------------------- */
@@ -787,7 +759,7 @@ static void cst_accept(MainInfo *min)
 /* 1999-05-27 -	Save the current style configuration. Thanks to the styles module, this
 **		is really not complicated. :)
 */
-static int cst_save(MainInfo *min, FILE *out)
+static gint cst_save(MainInfo *min, FILE *out)
 {
 	stl_styleinfo_save(min, min->cfg.style, out, NODE);
 
@@ -817,9 +789,9 @@ static void cst_hide(MainInfo *min)
 
 /* ----------------------------------------------------------------------------------------- */
 
-CfgPage * cst_describe(MainInfo *min)
+const CfgModule * cst_describe(MainInfo *min)
 {
-	static CfgPage	desc = { NODE, cst_init, cst_update, cst_accept, cst_save, cst_load, cst_hide };
+	static const CfgModule	desc = { NODE, cst_init, cst_update, cst_accept, cst_save, cst_load, cst_hide };
 
 	return &desc;
 }
