@@ -25,13 +25,14 @@
 #include <fcntl.h>
 #include <utime.h>
 
+#include "cmd_delete.h"
+#include "cmdseq_config.h"
+#include "dirpane.h"
 #include "errors.h"
 #include "fileutil.h"
-#include "dirpane.h"
-#include "strutil.h"
 #include "overwrite.h"
 #include "progress.h"
-#include "cmdseq_config.h"
+#include "strutil.h"
 
 #include "cmd_copy.h"
 
@@ -54,7 +55,7 @@ static CmdCfg	*copy_cmc = NULL;
 /* 2001-04-29 -	Copy access and modification dates from <from> to (freshly created) <to>.
 **		If <from_stat> is NULL, a local stat() call is made.
 */
-void copy_dates(const gchar *from, const struct stat *from_stat, const gchar *to)
+static void copy_dates(const gchar *from, const struct stat *from_stat, const gchar *to)
 {
 	struct stat	sbuf;
 	struct utimbuf	tim;
@@ -72,6 +73,20 @@ void copy_dates(const gchar *from, const struct stat *from_stat, const gchar *to
 	utime(to, &tim);	/* Implicit error handling through errno assumed. */
 }
 
+/* 2002-03-09 -	Copy mode from <from> to <to>. */
+static void copy_mode(const gchar *from, const struct stat *from_stat, const gchar *to)
+{
+	struct stat	sbuf;
+
+	if(from_stat == NULL)
+	{
+		if(lstat(from, &sbuf) != 0)
+			return;
+		from_stat = &sbuf;
+	}
+	chmod(to, from_stat->st_mode);
+}
+
 /* ----------------------------------------------------------------------------------------- */
 
 /* 1998-05-21 -	Copy a single regular file from <from> to <to>. The destination name should not
@@ -92,10 +107,10 @@ void copy_dates(const gchar *from, const struct stat *from_stat, const gchar *to
 **		size as the source, and if so ignore the error.
 ** 1999-03-15 -	Cleaned up the implementation of the leave_fullsize feature a great deal.
 */
-gboolean copy_file(MainInfo *min, const gchar *from, const gchar *full_to, struct stat *sstat)
+gboolean copy_file(MainInfo *min, const gchar *from, const gchar *full_to, const struct stat *sstat)
 {
 	gint		fd_in, fd_out;
-	gsize		put = 0;
+	off_t		put = 0;
 	struct stat	stat;
 
 	if(sstat == NULL)
@@ -118,7 +133,10 @@ gboolean copy_file(MainInfo *min, const gchar *from, const gchar *full_to, struc
 		}
 		close(fd_in);
 		if((errno == 0) && (put == sstat->st_size))
+		{
 			copy_dates(from, sstat, full_to);
+			copy_mode(from, sstat, full_to);
+		}
 	}
 	pgs_progress_item_end(min);
 	if(errno)
@@ -139,7 +157,7 @@ gboolean copy_file(MainInfo *min, const gchar *from, const gchar *full_to, struc
 **		to worry about. Note that non-root users rarely have enough power for this.
 ** 1998-09-12 -	Modified; the <full_to> parameter now gives the complete destination name.
 */
-gboolean copy_device(MainInfo *min, const gchar *from, const gchar *full_to, struct stat *sstat)
+gboolean copy_device(MainInfo *min, const gchar *from, const gchar *full_to, const struct stat *sstat)
 {
 	struct stat	stat;
 
@@ -153,10 +171,12 @@ gboolean copy_device(MainInfo *min, const gchar *from, const gchar *full_to, str
 			return FALSE;
 		}
 	}
-
 	pgs_progress_item_begin(min, full_to, stat.st_size);
 	if(mknod(full_to, sstat->st_mode, sstat->st_rdev) == 0)
+	{
 		copy_dates(from, sstat, full_to);
+		copy_mode(from, sstat, full_to);
+	}
 	pgs_progress_item_update(min, sstat->st_size);
 	pgs_progress_item_end(min);
 	if(errno)
@@ -195,7 +215,7 @@ gboolean copy_link(MainInfo *min, const gchar *from, const gchar *full_to)
 				strcpy(dbuf, full_to);
 				if((dslash = strrchr(dbuf, G_DIR_SEPARATOR)) != NULL)
 					*dslash = '\0';
-				strcat(dbuf, "/");
+				strcat(dbuf, G_DIR_SEPARATOR_S);
 				strcat(dbuf, buf);
 				if(fut_exists(dbuf))			/* Does link target exist? */
 					strcpy(newbuf, dbuf);		/* Then create relative link to it. */
@@ -215,24 +235,39 @@ gboolean copy_link(MainInfo *min, const gchar *from, const gchar *full_to)
 **		destination directory, and then copying all the source's contents
 **		there. When directories are found in the source, this function is
 **		called again to copy them.
-** 1998-06-07 -	Added more error checking (now aborts on first failure, returning
-**		0). Does not remove partially failed directories.
+** 1998-06-07 -	Added more error checking (now aborts on first failure, returning 0).
+**		Does not remove partially failed directories.
 ** 1998-09-12 -	Modified; the <dest_name> is now the COMPLETE destination; it is
 **		created directly, without appending anything. Also changed the order of
 **		the fut_cd() and mkdir() calls, for better handling of unreable dirs.
+** 2002-07-13 -	Will now completely delete an existing directory before copying.
 */
 gboolean copy_dir(MainInfo *min, const gchar *from, const gchar *dest_name)
 {
 	gchar		old_dir[PATH_MAX];
 	DIR		*dir;
 	struct dirent	*de;
-	struct stat	stat;
+	struct stat	stat, dstat;
 
 	if(lstat(from, &stat) != 0)
 	{
 		err_set(min, errno, CMD_ID, from);
 		return FALSE;
 	}
+	if(lstat(dest_name, &dstat) == 0)
+	{
+		if(S_ISDIR(dstat.st_mode) && !del_delete_directory(min, dest_name, FALSE))
+		{
+			err_set(min, errno, CMD_ID, dest_name);
+			return FALSE;
+		}
+		else if(!del_delete_file(min, dest_name))
+		{
+			err_set(min, errno, CMD_ID, dest_name);
+			return FALSE;
+		}
+	}
+	err_clear(min);
 	pgs_progress_item_begin(min, dest_name, stat.st_size);
 	if(fut_cd(from, old_dir, sizeof old_dir))
 	{
@@ -264,6 +299,7 @@ gboolean copy_dir(MainInfo *min, const gchar *from, const gchar *dest_name)
 		}
 		fut_cd(old_dir, NULL, 0);
 		copy_dates(from, NULL, dest_name);
+		copy_mode(from, NULL, dest_name);
 	}
 	pgs_progress_item_end(min);
 	if(errno)
@@ -273,14 +309,8 @@ gboolean copy_dir(MainInfo *min, const gchar *from, const gchar *dest_name)
 
 /* ----------------------------------------------------------------------------------------- */
 
-/* 1998-05-30 -	Debugged this one. It was poking the selection flag directly, which is not
-**		legal. The correct way is to call dp_unselect(). Fixing this removed the
-**		need to rescan (via dp_rescan()) the *source* directory after the copy
-**		is complete, which of course boosts performance. Neat.
-** 1998-09-06 -	Now only rescans the dest if some file(s) were actually selected to begin with.
-** 1998-09-17 -	Added overwrite protection.
-*/
-int cmd_copy(MainInfo *min, DirPane *src, DirPane *dst, CmdArg *ca)
+/* 2002-07-23 -	Killed historical comments here, they were boring. Also FAM-adapted rescan. */
+gint cmd_copy(MainInfo *min, DirPane *src, DirPane *dst, const CmdArg *ca)
 {
 	gchar	old_path[PATH_MAX], dest[PATH_MAX];
 	guint	num = 0;
@@ -294,11 +324,10 @@ int cmd_copy(MainInfo *min, DirPane *src, DirPane *dst, CmdArg *ca)
 		return 1;
 	if(!fut_cd(src->dir.path, old_path, sizeof old_path))
 		return 0;
-
 	if((slist = dp_get_selection(src)) == NULL)
 		return 1;
 
-	ovw_overwrite_begin(min, _("\"%s\" Exists - Proceed With Copy?"), 0UL);
+	ovw_overwrite_begin(min, _("\"%s\" Already Exists - Proceed With Copy?"), 0UL);
 	pgs_progress_begin(min, _("Copying..."), PFLG_COUNT_RECURSIVE | PFLG_ITEM_VISIBLE | PFLG_BYTE_VISIBLE);
 	for(iter = slist; !errno && (iter != NULL); iter = g_slist_next(iter))
 	{
@@ -308,11 +337,7 @@ int cmd_copy(MainInfo *min, DirPane *src, DirPane *dst, CmdArg *ca)
 		if(res == OVW_SKIP)
 			continue;
 		else if(res == OVW_CANCEL)
-		{
-			if(errno)
-				err_set(min, errno, "copy", dp_full_name(src, DP_SEL_INDEX(src, iter)));
 			break;
-		}
 		if(S_ISLNK(mode))
 			copy_link(min, DP_SEL_NAME(iter), dest);
 		else if(S_ISDIR(mode))
@@ -326,14 +351,16 @@ int cmd_copy(MainInfo *min, DirPane *src, DirPane *dst, CmdArg *ca)
 		num++;
 	}
 	if(num)
-		dp_rescan(dst);
+		dp_rescan_post_cmd(dst);
 	pgs_progress_end(min);
 	ovw_overwrite_end(min);
-	dp_free_selection(slist);
-
 	fut_cd(old_path, NULL, 0);
 
+	if(errno && iter != NULL)
+		err_set(min, errno, "copy", dp_full_name(src, DP_SEL_INDEX(src, iter)));
+	dp_free_selection(slist);
 	err_show(min);
+
 	return errno == 0;
 }
 
