@@ -38,22 +38,17 @@ typedef struct {
 /* ----------------------------------------------------------------------------------------- */
 
 /* 1998-09-12 -	Update body of copy as GUI. */
-static void cpa_copy_body(MainInfo *min, DirPane *src, DirRow *row, GtkWindow *win, gpointer user)
+static void cpa_copy_body(MainInfo *min, DirPane *src, DirRow *row, gpointer user)
 {
-	gchar	buf[FILENAME_MAX + 32], *text;
-	CpaInfo	*cpa = user;
+	gchar		buf[FILENAME_MAX + 32];
+	const gchar	*text;
+	CpaInfo		*cpa = user;
 
 	text = DP_ROW_NAME(row);
 	if(cpa->action == CPA_COPYAS)
-	{
-		gtk_window_set_title(win, "Copy As");
 		g_snprintf(buf, sizeof buf, _("Enter Name for Copy of \"%s\""), text);
-	}
 	else
-	{
-		gtk_window_set_title(win, "Clone");
 		g_snprintf(buf, sizeof buf, _("Enter Name for Clone of \"%s\""), text);
-	}
 	gtk_label_set_text(GTK_LABEL(cpa->label), buf);
 	gtk_entry_set_text(GTK_ENTRY(cpa->entry), text);
 	gtk_entry_select_region(GTK_ENTRY(cpa->entry), 0, -1);
@@ -65,28 +60,23 @@ static void cpa_copy_body(MainInfo *min, DirPane *src, DirRow *row, GtkWindow *w
 			ovw_overwrite_begin(min, _("\"%s\" Already Exists - Proceed With Copy?"), 0U);
 		else
 			ovw_overwrite_begin(min, _("\"%s\" Already Exists - Continue With Clone?"), 0U);
-		pgs_progress_begin(min, cpa->action == CPA_COPYAS ? _("Copying As...") : _("Cloning..."), -1);
+		pgs_progress_begin(min, cpa->action == CPA_COPYAS ? _("Copying As...") : _("Cloning..."), PFLG_COUNT_RECURSIVE | PFLG_ITEM_VISIBLE | PFLG_BYTE_VISIBLE);
 		cpa->ovw_open = TRUE;
 	}
 }
 
 /* 1999-05-30 -	Update body for symlink operation. */
-static void cpa_link_body(MainInfo *min, DirPane *src, DirRow *row, GtkWindow *win, gpointer user)
+static void cpa_link_body(MainInfo *min, DirPane *src, DirRow *row, gpointer user)
 {
-	gchar	buf[FILENAME_MAX + 32], *text;
-	CpaInfo	*cpa = user;
+	gchar		buf[FILENAME_MAX + 32];
+	const gchar	*text;
+	CpaInfo		*cpa = user;
 
 	text = DP_ROW_NAME(row);
 	if(cpa->action == CPA_SYMLINKAS)
-	{
-		gtk_window_set_title(win, "SymLinkAs");
 		g_snprintf(buf, sizeof buf, _("Enter Name to Link \"%s\" As"), text);
-	}
 	else if(cpa->action == CPA_SYMLINKCLONE)
-	{
-		gtk_window_set_title(win, "SymLinkClone");
 		g_snprintf(buf, sizeof buf, _("Enter Name for Link Clone of \"%s\""), text);
-	}
 	gtk_label_set_text(GTK_LABEL(cpa->label), buf);
 	gtk_entry_set_text(GTK_ENTRY(cpa->entry), text);
 	gtk_entry_select_region(GTK_ENTRY(cpa->entry), 0, -1);
@@ -113,11 +103,12 @@ static const gchar * op_name(CpaAction action)
 }
 
 /* 1998-09-12 -	Perform action for copy as/clone command. */
-static int cpa_action(MainInfo *min, DirPane *src, DirPane *dst, DirRow *row, gpointer user)
+static gint cpa_action(MainInfo *min, DirPane *src, DirPane *dst, DirRow *row, gpointer user)
 {
-	gchar	dest[PATH_MAX], *text, *name;
-	CpaInfo	*cpa = user;
-	OvwRes	ores;
+	gchar		dest[PATH_MAX], *text;
+	const gchar	*name;
+	CpaInfo		*cpa = user;
+	OvwRes		ores;
 
 	if(!fut_cd(src->dir.path, NULL, 0))
 		return 0;
@@ -131,7 +122,7 @@ static int cpa_action(MainInfo *min, DirPane *src, DirPane *dst, DirRow *row, gp
 			case CPA_COPYAS:
 			case CPA_SYMLINKAS:
 				if(strchr(text, G_DIR_SEPARATOR) == NULL)
-					g_snprintf(dest, sizeof dest, "%s/%s", dst->dir.path, text);
+					g_snprintf(dest, sizeof dest, "%s%s%s", dst->dir.path, G_DIR_SEPARATOR_S, text);
 				else
 					stu_strncpy(dest, text, sizeof dest);
 				break;
@@ -139,10 +130,9 @@ static int cpa_action(MainInfo *min, DirPane *src, DirPane *dst, DirRow *row, gp
 			case CPA_SYMLINKCLONE:
 				if(strchr(text, G_DIR_SEPARATOR) != NULL)
 					return 0;
-				g_snprintf(dest, sizeof dest, "%s/%s", src->dir.path, text);
+				g_snprintf(dest, sizeof dest, "%s%s%s", src->dir.path, G_DIR_SEPARATOR_S, text);
 				break;
 		}
-
 		ores = ovw_overwrite_file(min, dest, dp_full_name(src, DP_ROW_INDEX(src, row)));
 		if(ores == OVW_SKIP)
 			return 1;
@@ -178,7 +168,7 @@ static int cpa_action(MainInfo *min, DirPane *src, DirPane *dst, DirRow *row, gp
 
 static void cpa_free(gpointer user)
 {
-	CpaInfo	*cpa = (CpaInfo *) user;
+	CpaInfo	*cpa = user;
 
 	if(cpa->ovw_open)
 	{
@@ -190,7 +180,7 @@ static void cpa_free(gpointer user)
 /* ----------------------------------------------------------------------------------------- */
 
 /* 1998-09-12 -	Fire up either a COPYAS or a CLONE command, depending on the <action> arg. */
-static int copyas_or_clone(MainInfo *min, DirPane *src, DirPane *dst, CpaAction action)
+static gint copyas_or_clone(MainInfo *min, DirPane *src, DirPane *dst, CpaAction action)
 {
 	static CpaInfo	cpa;
 
@@ -202,39 +192,37 @@ static int copyas_or_clone(MainInfo *min, DirPane *src, DirPane *dst, CpaAction
 	cpa.label = gtk_label_new(_("Copy As"));
 	cpa.entry = gtk_entry_new_with_max_length(FILENAME_MAX - 1);
 	gtk_box_pack_start(GTK_BOX(cpa.vbox), cpa.label, FALSE, FALSE, 0);
-	gtk_widget_show(cpa.label);
 	gtk_box_pack_start(GTK_BOX(cpa.vbox), cpa.entry, FALSE, FALSE, 0);
-	gtk_widget_show(cpa.entry);
 
 	if(cpa.action == CPA_COPYAS)
-		return cmd_generic(min, CGF_NOALL, cpa_copy_body, cpa_action, cpa_free, &cpa);
+		return cmd_generic(min, _("Copy As"), CGF_NOALL, cpa_copy_body, cpa_action, cpa_free, &cpa);
 	else if(cpa.action == CPA_CLONE)
-		return cmd_generic(min, CGF_NOALL | CGF_NODST | CGF_SRC, cpa_copy_body, cpa_action, cpa_free, &cpa);
+		return cmd_generic(min, _("Clone"), CGF_NOALL | CGF_NODST | CGF_SRC, cpa_copy_body, cpa_action, cpa_free, &cpa);
 	else if(cpa.action == CPA_SYMLINKAS)
-		return cmd_generic(min, CGF_NOALL, cpa_link_body, cpa_action, cpa_free, &cpa);
+		return cmd_generic(min, _("Symbolic Link As"), CGF_NOALL, cpa_link_body, cpa_action, cpa_free, &cpa);
 	else if(cpa.action == CPA_SYMLINKCLONE)
-		return cmd_generic(min, CGF_NOALL | CGF_NODST | CGF_SRC, cpa_link_body, cpa_action, cpa_free, &cpa);
+		return cmd_generic(min, _("Symbolic Link Clone"), CGF_NOALL | CGF_NODST | CGF_SRC, cpa_link_body, cpa_action, cpa_free, &cpa);
 	return 0;
 }
 
 /* ----------------------------------------------------------------------------------------- */
 
-int cmd_copyas(MainInfo *min, DirPane *src, DirPane *dst, CmdArg *ca)
+gint cmd_copyas(MainInfo *min, DirPane *src, DirPane *dst, const CmdArg *ca)
 {
 	return copyas_or_clone(min, src, dst, CPA_COPYAS);
 }
 
-int cmd_clone(MainInfo *min, DirPane *src, DirPane *dst, CmdArg *ca)
+gint cmd_clone(MainInfo *min, DirPane *src, DirPane *dst, const CmdArg *ca)
 {
 	return copyas_or_clone(min, src, dst, CPA_CLONE);
 }
 
-int cmd_symlinkas(MainInfo *min, DirPane *src, DirPane *dst, CmdArg *ca)
+gint cmd_symlinkas(MainInfo *min, DirPane *src, DirPane *dst, const CmdArg *ca)
 {
 	return copyas_or_clone(min, src, dst, CPA_SYMLINKAS);
 }
 
-int cmd_symlinkclone(MainInfo *min, DirPane *src, DirPane *dst, CmdArg *ca)
+gint cmd_symlinkclone(MainInfo *min, DirPane *src, DirPane *dst, const CmdArg *ca)
 {
 	return copyas_or_clone(min, src, dst, CPA_SYMLINKCLONE);
 }
