@@ -44,7 +44,7 @@ gboolean del_delete_file(MainInfo *min, const gchar *path)
 /* ----------------------------------------------------------------------------------------- */
 
 /* 1999-05-29 -	Delete a directory. */
-gboolean del_delete_directory(MainInfo *min, const gchar *path)
+gboolean del_delete_directory(MainInfo *min, const gchar *path, gboolean progress)
 {
 	gchar		old_dir[PATH_MAX];
 	DIR		*dir;
@@ -62,19 +62,22 @@ gboolean del_delete_directory(MainInfo *min, const gchar *path)
 			{
 				while((de = readdir(dir)) != NULL)
 				{
-					pgs_progress_item_begin(min, de->d_name, 0);
-					if(pgs_progress_item_update(min, 0) == PGS_CANCEL)
+					if(progress)
 					{
-						errno = EINTR;
-						break;
+						pgs_progress_item_begin(min, de->d_name, 0);
+						if(pgs_progress_item_update(min, 0) == PGS_CANCEL)
+						{
+							errno = EINTR;
+							break;
+						}
+						pgs_progress_item_end(min);
 					}
-					pgs_progress_item_end(min);
 					if(!min->cfg.dir_filter(de->d_name))
 						continue;
 					if(lstat(de->d_name, &stat) == 0)
 					{
 						if(S_ISDIR(stat.st_mode))
-							del_delete_directory(min, de->d_name);
+							del_delete_directory(min, de->d_name, progress);
 						else
 							del_delete_file(min, de->d_name);
 					}
@@ -98,13 +101,12 @@ gboolean del_delete_directory(MainInfo *min, const gchar *path)
 /* 1998-09-18 -	A new entrypoint for the delete command. Completely replaces the old one,
 **		which has been removed.
 */
-int cmd_delete(MainInfo *min, DirPane *src, DirPane *dst, CmdArg *ca)
+gint cmd_delete(MainInfo *min, DirPane *src, DirPane *dst, const CmdArg *ca)
 {
 	gchar		old_path[PATH_MAX];
 	const gchar	*dest;
 	gboolean	ok;
 	guint		num = 0;
-	mode_t		mode;
 	OvwRes		ores;
 	GSList		*slist,	 *iter;
 
@@ -131,9 +133,8 @@ int cmd_delete(MainInfo *min, DirPane *src, DirPane *dst, CmdArg *ca)
 			continue;
 		else if(ores == OVW_CANCEL)
 			break;
-		mode = DP_SEL_LSTAT(iter).st_mode;
-		if(S_ISDIR(mode))
-			ok = del_delete_directory(min, dest);
+		if(S_ISDIR(DP_SEL_LSTAT(iter).st_mode))
+			ok = del_delete_directory(min, dest, TRUE);
 		else
 			ok = del_delete_file(min, dest);
 		if(ok)
@@ -144,11 +145,11 @@ int cmd_delete(MainInfo *min, DirPane *src, DirPane *dst, CmdArg *ca)
 		else
 			break;
 	}
-	dp_free_selection(slist);
 	pgs_progress_end(min);
 	ovw_overwrite_end(min);
 	if(num)
-		dp_rescan(src);
+		dp_rescan_post_cmd(src);
+	dp_free_selection(slist);
 
 	fut_cd(old_path, NULL, 0);
 	err_show(min);
