@@ -17,6 +17,7 @@
 #include "dirhistory.h"
 #include "dirpane.h"
 #include "fileutil.h"
+#include "gfam.h"
 #include "mount.h"
 #include "strutil.h"
 
@@ -51,12 +52,10 @@ static const gchar * get_selected_dir(DirPane *dp)
 /* 1999-05-07 -	Rewrote this, now supports a "dir" command argument.
 ** 1999-06-06 -	Now does all work through the dirhistory module.
 */
-int cmd_direnter(MainInfo *min, DirPane *src, DirPane *dst, CmdArg *ca)
+gint cmd_direnter(MainInfo *min, DirPane *src, DirPane *dst, const CmdArg *ca)
 {
 	const gchar	*dir;
-	gchar		buf[PATH_MAX] = "", component[PATH_MAX];
 
-	err_clear(min);
 	if((dir = car_bareword_get(ca, 0)) == NULL)
 		dir = car_keyword_get_value(ca, "dir", NULL);
 	if(dir == NULL)
@@ -64,6 +63,8 @@ int cmd_direnter(MainInfo *min, DirPane *src, DirPane *dst, CmdArg *ca)
 
 	if(dir != NULL)
 	{
+		gchar	buf[PATH_MAX] = "", component[PATH_MAX];
+
 		if((dir[0] == G_DIR_SEPARATOR) || (dir[0] == '~'))	/* Absolute? */
 			stu_strncpy(buf, dir, sizeof buf);
 		else
@@ -75,17 +76,30 @@ int cmd_direnter(MainInfo *min, DirPane *src, DirPane *dst, CmdArg *ca)
 			else
 				g_snprintf(buf, sizeof buf, "%s%c%s", src->dir.path, G_DIR_SEPARATOR, dir);
 		}
+		err_clear(min);
 		if(fut_path_component(buf, component))
 		{
-			mnt_entering(min, component, MNT_RARELY);
-			if(dph_direnter(src, component))
-				dph_set_path_widgets(src);
+			dph_state_save(src);
+			dp_freeze(src);
+			dp_enter_dir(src, component);
+			dp_redisplay(src);
+			/* Annoying hacky code to get GTK+ to resize the pane before we restore
+			** the history state; if not resized, vpos setting will likely fail.
+			*/
+			gtk_widget_queue_resize(GTK_WIDGET(src->list));
+/*			gtk_widget_queue_draw(src->scwin);*/
+			while(gtk_events_pending())
+				gtk_main_iteration();
+			dph_state_restore(src);
+			dp_thaw(src);
+			dp_show_stats(src);
+			fam_monitor(src);
+		}
+		if(errno != 0)
+		{
+			err_set(min, errno, "DirEnter", buf);
+			err_show(min);
 		}
-	}
-	if(errno != 0)
-	{
-		err_set(min, errno, "DirEnter", buf);
-		err_show(min);
 	}
 	return (dir != NULL) && (errno == 0);
 }
