@@ -71,7 +71,7 @@ static void move_page(DirPane *src, gint direction)
 }
 
 /* 1999-05-10 -	Do some focusing. */
-int cmd_dpfocus(MainInfo *min, DirPane *src, DirPane *dst, CmdArg *ca)
+gint cmd_dpfocus(MainInfo *min, DirPane *src, DirPane *dst, const CmdArg *ca)
 {
 	gboolean	sel = car_keyword_get_boolean(ca, "select", FALSE),
 			center = car_keyword_get_boolean(ca, "center", FALSE);
