@@ -39,6 +39,7 @@ typedef struct {
 	gchar	 df_access[DP_DATEFMT_SIZE];	/* Formatter for dates of last access, */
 	gchar	 df_modify[DP_DATEFMT_SIZE];	/*  modification, */
 	gchar	 df_change[DP_DATEFMT_SIZE];	/*  and change. */
+	gchar	 tick[2];
 } OptInfo;
 
 static OptInfo	info_options;
@@ -46,33 +47,36 @@ static CmdCfg	*info_cmc = NULL;
 
 /* ----------------------------------------------------------------------------------------- */
 
-static void build_date(const gchar *label, gint y, time_t *date, const gchar *fmt, GtkWidget *tab)
+static GtkWidget * label_pack(GtkWidget *table, const gchar *text, gint x, gint y, gboolean left)
+{
+	GtkWidget	*algn, *lab;
+
+	algn = gtk_alignment_new(left ? 0.0f : 0.9f, 0.0f, 0.0f, 0.0f);
+	lab  = gtk_label_new(text);
+	gtk_container_add(GTK_CONTAINER(algn), lab);
+	gtk_table_attach(GTK_TABLE(table), algn, x, x + 1, y, y + 1, GTK_FILL,0,4,0);
+	return lab;
+}
+
+static void build_date(const gchar *label, gint y, const time_t *date, const gchar *fmt, GtkWidget *tab)
 {
 	gchar		buf[64];
-	GtkWidget	*lab;
 	struct tm	*tm;
 
-	lab = gtk_label_new(label);
-	gtk_table_attach(GTK_TABLE(tab), lab, 0, 1, y, y + 1, 0,0,5,0);
-
+	label_pack(tab, label, 0, y, FALSE);
 	if((tm = localtime(date)) != NULL)
 	{
 		if(strftime(buf, sizeof buf, fmt, tm) >= 0)
-		{
-			lab = gtk_label_new(buf);
-			gtk_table_attach(GTK_TABLE(tab), lab, 1, 2, y, y + 1, 0,0,5,0);
-		}
+			label_pack(tab, buf, 1, y, TRUE);
 	}
 }
 
-static GtkWidget * build_info(MainInfo *min, DirPane *dp, DirRow *row)
+static GtkWidget * build_info(MainInfo *min, const DirPane *dp, const DirRow *row)
 {
 	gchar		buf[128], sbuf[32], *ptr;
 	guint		y;
 	const gchar	*iname;
-	GtkWidget	*tab, *ico, *lab, *szlab, *sep;
-	GdkPixmap	*ipix;
-	GdkBitmap	*imsk;
+	GtkWidget	*tab, *szlab, *sep, *entry;
 
 	tab = gtk_table_new(17, 2, FALSE);
 
@@ -81,87 +85,79 @@ static GtkWidget * build_info(MainInfo *min, DirPane *dp, DirRow *row)
 	{
 		if((iname = stl_style_property_get_icon(DP_ROW_TYPE(row)->style, SPN_ICON_UNSEL)) != NULL)
 		{
+			GtkWidget	*ico;
+			GdkPixmap	*ipix;
+			GdkBitmap	*imsk;
+
 			ipix = ico_icon_get(min, iname, &imsk);
 			ico = gtk_pixmap_new(ipix, imsk);
 			gtk_table_attach(GTK_TABLE(tab), ico, 0, 1, y, y + 1, 0,0,5,5);
 		}
 	}
-	lab = gtk_label_new(DP_ROW_NAME(row));
-	gtk_table_attach(GTK_TABLE(tab), lab, 1, 2, y, y + 1, GTK_EXPAND,0,5,5);
-	y++;
+	label_pack(tab, DP_ROW_NAME(row), 1, y++, TRUE);
 
 	/* For symbolic links, display name of the target, at least. */
 	if(S_ISLNK(DP_ROW_LSTAT(row).st_mode))
 	{
-		GtkWidget	*scwin, *text;
-
-		lab = gtk_label_new(_("Link To"));
-		gtk_table_attach(GTK_TABLE(tab), lab, 0, 1, y, y + 1, 0,0,5,0);
-		scwin = gtk_scrolled_window_new(NULL, NULL);
-		gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scwin), GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
-		text = gtk_text_new(NULL, NULL);
-		gtk_text_set_editable(GTK_TEXT(text), FALSE);
-		gtk_text_insert(GTK_TEXT(text), NULL, NULL, NULL, DP_ROW_LINKNAME(row), strlen(DP_ROW_LINKNAME(row)));
-		gtk_container_add(GTK_CONTAINER(scwin), text);
-		gtk_table_attach(GTK_TABLE(tab), scwin, 1, 2, y, y + 1,  GTK_EXPAND|GTK_FILL,GTK_EXPAND|GTK_FILL,5,0);
+		label_pack(tab, _("Link To"), 0, y, FALSE);
+		entry = gtk_entry_new();
+		gtk_editable_set_editable(GTK_EDITABLE(entry), FALSE);
+		gtk_entry_set_text(GTK_ENTRY(entry), DP_ROW_LINKNAME(row));
+		gtk_table_attach(GTK_TABLE(tab), entry, 1, 2, y, y + 1,  GTK_EXPAND|GTK_FILL,GTK_EXPAND|GTK_FILL,5,0);
 		y++;
 	}
-
 	sep = gtk_hseparator_new();
 	gtk_table_attach(GTK_TABLE(tab), sep, 0, 2, y, y + 1, GTK_EXPAND | GTK_FILL,0,0,5);
 	y++;
 
-	lab = gtk_label_new(_("Type"));
-	gtk_table_attach(GTK_TABLE(tab), lab, 0, 1, y, y + 1, 0,0,5,0);
-	lab = gtk_label_new(DP_ROW_TYPE(row)->name);
-	gtk_table_attach(GTK_TABLE(tab), lab, 1, 2, y, y + 1, 0,0,5,0);
-	y++;
+	label_pack(tab, _("Type"), 0, y, FALSE);
+	label_pack(tab, DP_ROW_TYPE(row)->name, 1, y++, TRUE);
 
-	lab = gtk_label_new(_("Location"));
-	gtk_table_attach(GTK_TABLE(tab), lab, 0, 1, y, y + 1, 0,0,5,0);
-	lab = gtk_label_new(dp->dir.path);
-	gtk_table_attach(GTK_TABLE(tab), lab, 1, 2, y, y + 1, 0,0,5,0);
-	y++;
+	label_pack(tab, _("Location"), 0, y, FALSE);
+	label_pack(tab, dp->dir.path, 1, y++, TRUE);
 
-	lab = gtk_label_new(_("Size"));
-	gtk_table_attach(GTK_TABLE(tab), lab, 0, 1, y, y + 1, 0,0,5,0);
-	sze_put_size(DP_ROW_LSTAT(row).st_size, sbuf, sizeof sbuf, SZE_AUTO);
+	label_pack(tab, _("Size"), 0, y, FALSE);
+	sze_put_size(sbuf, sizeof sbuf, DP_ROW_LSTAT(row).st_size, SZE_AUTO);
 	if(DP_ROW_LSTAT(row).st_size < 1024)
 		stu_strncpy(buf, sbuf, sizeof buf);
 	else
-		g_snprintf(buf, sizeof buf, _("%s (%u bytes)"), sbuf, (gsize) DP_ROW_LSTAT(row).st_size);
-	szlab = gtk_label_new(buf);
-	gtk_table_attach(GTK_TABLE(tab), szlab, 1, 2, y, y + 1, 0,0,5,0);
-	y++;
+	{
+		gchar	stbuf[64], *st;
 
+		stbuf[sizeof stbuf - 1] = '\0';
+		st = stu_tickify(stbuf + sizeof stbuf - 1, DP_ROW_LSTAT(row).st_size, info_options.tick[0]);
+		g_snprintf(buf, sizeof buf, _("%s (%s bytes)"), sbuf, st);
+	}
+	szlab = label_pack(tab, buf, 1, y++, TRUE);
 	/* Building info for a directory, and recursing enabled? */
 	if(S_ISDIR(DP_ROW_LSTAT(row).st_mode) && info_options.recurse_dirs)
 	{
-		gchar	buf[64], sbuf[64];
-		guint64	bytes = 0;
+		guint64	bytes = 0U;
 		FUCount	fc;
 
 		/* Collect information recursively. */
 		if(fut_dir_size_progress(min, dp_full_name(dp, DP_ROW_INDEX(dp, row)), &bytes, &fc))
 		{
+			gchar	buf[64], sbuf[64], stbuf[64], *st;
+
 			/* Replace contents of previously built size-label. */
-			sze_put_size64(bytes, sbuf, sizeof buf, SZE_AUTO);
-			g_snprintf(buf, sizeof buf, _("%s (%Lu bytes)"), sbuf, bytes);
+			sze_put_size64(sbuf, sizeof buf, bytes, SZE_AUTO);
+			
+			stbuf[sizeof stbuf - 1] = '\0';
+			st = stu_tickify(stbuf + sizeof stbuf - 1, bytes, info_options.tick[0]);
+			g_snprintf(buf, sizeof buf, _("%s (%s bytes)"), sbuf, st);
 			gtk_label_set_text(GTK_LABEL(szlab), buf);
 
 			/* And tell a little something about the contents. */
-			lab = gtk_label_new(_("Contains"));
-			gtk_table_attach(GTK_TABLE(tab), lab, 0, 1, y, y + 1, 0,0,5,0);
-			g_snprintf(buf, sizeof buf, _("%d dirs, %d files, %d symlinks"),
+			label_pack(tab, _("Contains"), 0, y, FALSE);
+			g_snprintf(buf, sizeof buf, _("%u dirs, %u files, %u symlinks"),
 					fc.num_dirs, fc.num_files, fc.num_links);
-			lab = gtk_label_new(buf);
-			gtk_table_attach(GTK_TABLE(tab), lab, 1, 2, y, y + 1, 0,0,5,0);
-			y++;
+			label_pack(tab, buf, 1, y++, TRUE);
 		}
 	}
 
 	/* Is the "Show 'file' Output?" option enabled? Requires "file -n -f -" to work. */
-#if defined HAS_GOOD_FILE
+#if defined HAVE_GOOD_FILE
 	if(info_options.use_file)
 	{
 		gchar	buf[PATH_MAX];
@@ -169,37 +165,28 @@ static GtkWidget * build_info(MainInfo *min, DirPane *dp, DirRow *row)
 		sep = gtk_hseparator_new();
 		gtk_table_attach(GTK_TABLE(tab), sep, 0, 2, y, y + 1, GTK_EXPAND | GTK_FILL,0,0,5);
 		y++;
-		lab = gtk_label_new(_("'File' Info"));
-		gtk_table_attach(GTK_TABLE(tab), lab, 0, 1, y, y + 1, 0,0,5,0);
-		g_snprintf(buf, sizeof buf, "%s/%s", dp->dir.path, DP_ROW_NAME(row));
-		lab = gtk_label_new(fle_file(min, buf));
-		gtk_table_attach(GTK_TABLE(tab), lab, 1, 2, y, y + 1, 0,0,5,0);
-		y++;
+		label_pack(tab, _("'File' Info"), 0, y, FALSE);
+		g_snprintf(buf, sizeof buf, "%s%s%s", dp->dir.path, G_DIR_SEPARATOR_S, DP_ROW_NAME(row));
+		label_pack(tab, fle_file(min, buf), 1, y++, TRUE);
 	}
 #endif
 	sep = gtk_hseparator_new();
 	gtk_table_attach(GTK_TABLE(tab), sep, 0, 2, y, y + 1, GTK_EXPAND | GTK_FILL,0,0,5);
 	y++;
 
-	lab = gtk_label_new(_("Owner"));
-	gtk_table_attach(GTK_TABLE(tab), lab, 0, 1, y, y + 1, 0,0,5,0);
+	label_pack(tab, _("Owner"), 0, y, FALSE);
 	if((ptr = (gchar *) usr_lookup_uname(DP_ROW_LSTAT(row).st_uid)) != NULL)
-		g_snprintf(buf, sizeof buf, "%s (%d)", ptr, (int) DP_ROW_LSTAT(row).st_uid);
+		g_snprintf(buf, sizeof buf, "%s (%d)", ptr, (gint) DP_ROW_LSTAT(row).st_uid);
 	else
-		g_snprintf(buf, sizeof buf, "%d", (int) DP_ROW_LSTAT(row).st_uid);
-	lab = gtk_label_new(buf);
-	gtk_table_attach(GTK_TABLE(tab), lab, 1, 2, y, y + 1, 0,0,5,0);
-	y++;
+		g_snprintf(buf, sizeof buf, "%d", (gint) DP_ROW_LSTAT(row).st_uid);
+	label_pack(tab, buf, 1, y++, TRUE);
 
-	lab = gtk_label_new(_("Group"));
-	gtk_table_attach(GTK_TABLE(tab), lab, 0, 1, y, y + 1, 0,0,5,0);
+	label_pack(tab, _("Group"), 0, y, FALSE);
 	if((ptr = (gchar *) usr_lookup_gname(DP_ROW_LSTAT(row).st_gid)) != NULL)
-		g_snprintf(buf, sizeof buf, "%s (%d)", ptr, (int) DP_ROW_LSTAT(row).st_gid);
+		g_snprintf(buf, sizeof buf, "%s (%d)", ptr, (gint) DP_ROW_LSTAT(row).st_gid);
 	else
-		g_snprintf(buf, sizeof buf, "%d", (int) DP_ROW_LSTAT(row).st_gid);
-	lab = gtk_label_new(buf);
-	gtk_table_attach(GTK_TABLE(tab), lab, 1, 2, y, y + 1, 0,0,5,0);
-	y++;
+		g_snprintf(buf, sizeof buf, "%d", (gint) DP_ROW_LSTAT(row).st_gid);
+	label_pack(tab, buf, 1, y++, TRUE);
 
 	sep = gtk_hseparator_new();
 	gtk_table_attach(GTK_TABLE(tab), sep, 0, 2, y, y + 1, GTK_EXPAND | GTK_FILL,0,0,5);
@@ -220,16 +207,16 @@ static GtkWidget * build_info(MainInfo *min, DirPane *dp, DirRow *row)
 /* ----------------------------------------------------------------------------------------- */
 
 /* 1999-03-06 -	Create window showing all about <row>, which sits in <dp>. */
-static gint info(MainInfo *min, DirPane *dp, DirRow *row)
+static gint information(MainInfo *min, const DirPane *dp, const DirRow *row)
 {
 	GtkWidget	*fr;
 
 	if((fr = build_info(min, dp, row)) != NULL)
 	{
 		gtk_widget_show_all(fr);
-		dlg_dialog_async_new(fr, DP_ROW_NAME(row), _("OK"), NULL, NULL);
+		dlg_dialog_async_new(fr, DP_ROW_NAME(row), _("_OK"), NULL, NULL);
 	}
-	return (fr != NULL) ? TRUE : FALSE;
+	return fr != NULL;
 }
 
 /* ----------------------------------------------------------------------------------------- */
@@ -237,7 +224,7 @@ static gint info(MainInfo *min, DirPane *dp, DirRow *row)
 /* 1998-12-19 -	Show information about files in <src>. Knows about double-click, and handles
 **		it like everybody else.
 */
-int cmd_information(MainInfo *min, DirPane *src, DirPane *dst, CmdArg *ca)
+gint cmd_information(MainInfo *min, DirPane *src, DirPane *dst, const CmdArg *ca)
 {
 	GSList	*slist, *iter;
 
@@ -247,7 +234,7 @@ int cmd_information(MainInfo *min, DirPane *src, DirPane *dst, CmdArg *ca)
 	pgs_progress_begin(min, _("Getting Information..."), PFLG_BUSY_MODE);
 	for(iter = slist; iter != NULL; iter = g_slist_next(iter))
 	{
-		if(!info(min, src, DP_SEL_ROW(iter)))
+		if(!information(min, src, DP_SEL_ROW(iter)))
 			break;
 		dp_unselect(src, DP_SEL_INDEX(src, iter));
 	}
@@ -278,16 +265,18 @@ void cfg_information(MainInfo *min)
 		stu_strncpy(info_options.df_access, "%Y-%m-%d %H:%M.%S", sizeof info_options.df_access);
 		stu_strncpy(info_options.df_modify, "%Y-%m-%d %H:%M.%S", sizeof info_options.df_modify);
 		stu_strncpy(info_options.df_change, "%Y-%m-%d %H:%M.%S", sizeof info_options.df_change);
+		stu_strncpy(info_options.tick, ",", sizeof info_options.tick);
 
 		info_cmc = cmc_config_new("Information", &info_options);
 		cmc_field_add_boolean(info_cmc, "modified", NULL, offsetof(OptInfo, modified));
-#if defined HAS_GOOD_FILE
+#if defined HAVE_GOOD_FILE
 		cmc_field_add_boolean(info_cmc, "use_file", _("Show 'file' Output?"), offsetof(OptInfo, use_file));
 #endif
 		cmc_field_add_boolean(info_cmc, "recurse_dirs", _("Recurse Directories?"), offsetof(OptInfo, recurse_dirs));
 		cmc_field_add_string(info_cmc, "df_access", _("Access Date Format"), offsetof(OptInfo, df_access), sizeof info_options.df_access);
 		cmc_field_add_string(info_cmc, "df_modify", _("Modify Date Format"), offsetof(OptInfo, df_modify), sizeof info_options.df_modify);
 		cmc_field_add_string(info_cmc, "df_change", _("Change Date Format"), offsetof(OptInfo, df_change), sizeof info_options.df_change);
+		cmc_field_add_string(info_cmc, "tick", _("Size Tick Mark"), offsetof(OptInfo, tick), sizeof info_options.tick);
 
 		cmc_config_register(info_cmc);
 	}
