@@ -15,7 +15,7 @@
 #include "errors.h"
 #include "cmdseq.h"
 #include "cmdseq_config.h"
-#include "cfg_gui.h"
+#include "configure.h"
 
 #include "cmd_mkdir.h"
 
@@ -32,6 +32,7 @@ typedef struct {
 typedef struct {			/* Command-specific configuration data. */
 	gboolean	modified;		/* Has the setting(s) been modified? */
 	gboolean	cd_new;			/* CD into new directory after creation? */
+	gboolean	focus_new;		/* Focus the new directory? */
 } OptMkDir;
 
 static OptMkDir	mkdir_options;
@@ -40,19 +41,33 @@ static CmdCfg	*mkdir_cmc = NULL;
 /* ----------------------------------------------------------------------------------------- */
 
 /* 1998-08-27 -	This gets called when user closes down the mkdir dialog. Create the
-**		dir (and, optionally, enter it).
+**		dir (and, optionally, enter or focus it).
 */
 static void make_dir(MkdInfo *mdi)
 {
-	gchar	buf[PATH_MAX];
+	gchar	buf[PATH_MAX], *text = gtk_entry_get_text(GTK_ENTRY(mdi->entry));
 
-	g_snprintf(buf, sizeof buf, "%s%c%s", mdi->src->dir.path, G_DIR_SEPARATOR, gtk_entry_get_text(GTK_ENTRY(mdi->entry)));
+	if(*text == '\0')				/* Avoid attempting to create "". */
+		return;
+	if(strchr(text, G_DIR_SEPARATOR) != NULL)	/* Also avoid clever paths. */
+		return;
+	if(*mdi->src->dir.path == '\0')			/* Don't blow up in strlen() below. */
+		return;
+
+	/* Build destination directory name, taking care to get a single slash between parts. */
+	g_snprintf(buf, sizeof buf, "%s%s%s", mdi->src->dir.path,
+			mdi->src->dir.path[strlen(mdi->src->dir.path) - 1] == G_DIR_SEPARATOR ? "" : G_DIR_SEPARATOR_S,
+			text);
 	if(mkdir(buf, 0755) == 0)
 	{
 		if(mkdir_options.cd_new)
 			csq_execute_format(mdi->src->main, "DirEnter 'dir=%s'", buf);
 		else
-			dp_rescan(mdi->src);
+		{
+			dp_rescan_post_cmd(mdi->src);
+			csq_execute_format(mdi->src->main, "DpGotoRow focus=%s nocase=no 're=%s'", mkdir_options.focus_new ? "yes" : "no",
+				gtk_entry_get_text(GTK_ENTRY(mdi->entry)));
+		}
 	}
 	else
 	{
@@ -66,7 +81,7 @@ static void make_dir(MkdInfo *mdi)
 /* ----------------------------------------------------------------------------------------- */
 
 /* 1998-08-27 -	The mkdir entry point. Simple stuff. */
-int cmd_mkdir(MainInfo *min, DirPane *src, DirPane *dst, CmdArg *ca)
+gint cmd_mkdir(MainInfo *min, DirPane *src, DirPane *dst, const CmdArg *ca)
 {
 	Dialog		*dlg;
 	MkdInfo		mi;
@@ -76,15 +91,13 @@ int cmd_mkdir(MainInfo *min, DirPane *src, DirPane *dst, CmdArg *ca)
 	mi.vbox = gtk_vbox_new(FALSE, 0);
 	wid = gtk_label_new(_("Enter Name of Directory to Create"));
 	gtk_box_pack_start(GTK_BOX(mi.vbox), wid, FALSE, FALSE, 0);
-	gtk_widget_show(wid);
 	mi.entry = gtk_entry_new_with_max_length(MAXNAMLEN - 1);
 	gtk_box_pack_start(GTK_BOX(mi.vbox), mi.entry, FALSE, FALSE, 0);
-	gtk_widget_show(mi.entry);
 	wid = cmc_field_build(mkdir_cmc, "cd_new", &mkdir_options);
 	gtk_box_pack_start(GTK_BOX(mi.vbox), wid, FALSE, FALSE, 0);
-	gtk_widget_show(wid);
-	gtk_widget_show(mi.vbox);
-	dlg = dlg_dialog_sync_new(mi.vbox, _("Make Directory"), _("OK|Cancel"));
+	wid = cmc_field_build(mkdir_cmc, "focus_new", &mkdir_options);
+	gtk_box_pack_start(GTK_BOX(mi.vbox), wid, FALSE, FALSE, 0);
+	dlg = dlg_dialog_sync_new(mi.vbox, _("Make Directory"), NULL);
 	gtk_widget_grab_focus(mi.entry);
 	if(dlg_dialog_sync_wait(dlg) == DLG_POSITIVE)
 		make_dir(&mi);
@@ -103,10 +116,12 @@ void cfg_mkdir(MainInfo *min)
 		/* Initialize default option values. */
 		mkdir_options.modified	= FALSE;
 		mkdir_options.cd_new	= FALSE;
+		mkdir_options.focus_new = FALSE;
 
 		mkdir_cmc = cmc_config_new("MkDir", &mkdir_options);
 		cmc_field_add_boolean(mkdir_cmc, "modified", NULL, offsetof(OptMkDir, modified));
 		cmc_field_add_boolean(mkdir_cmc, "cd_new", _("CD Into New Directory?"), offsetof(OptMkDir, cd_new));
+		cmc_field_add_boolean(mkdir_cmc, "focus_new", _("Focus New Directory?"), offsetof(OptMkDir, focus_new));
 		cmc_config_register(mkdir_cmc);
 	}
 }
