@@ -23,7 +23,7 @@
 /* ----------------------------------------------------------------------------------------- */
 
 /* 1998-09-12 -	Modified: now <to> is the full destination name. */
-int move_file(MainInfo *min, const gchar *from, const gchar *to, struct stat *sstat)
+gint move_file(MainInfo *min, const gchar *from, const gchar *to, const struct stat *sstat)
 {
 	if(copy_file(min, from, to, sstat))
 		unlink(from);
@@ -31,7 +31,7 @@ int move_file(MainInfo *min, const gchar *from, const gchar *to, struct stat *ss
 }
 
 /* 1998-09-12 -	Modified: now <to> is the complete destination name. */
-int move_device(MainInfo *min, const gchar *from, const gchar *to, struct stat *sstat)
+gint move_device(MainInfo *min, const gchar *from, const gchar *to, const struct stat *sstat)
 {
 	if(copy_device(min, from, to, sstat))
 		unlink(from);
@@ -43,7 +43,7 @@ int move_device(MainInfo *min, const gchar *from, const gchar *to, struct stat *
 **		is done so that the new link (created at the destination) is correct.
 ** 1998-09-12 -	Modified; now <to> is the full destination name, rather than just the dir.
 */
-int move_link(MainInfo *min, const gchar *from, const gchar *to)
+gint move_link(MainInfo *min, const gchar *from, const gchar *to)
 {
 	if(copy_link(min, from, to))
 		unlink(from);
@@ -55,7 +55,7 @@ int move_link(MainInfo *min, const gchar *from, const gchar *to)
 **		ineffecient, having to traverse twice.
 ** 1998-09-12 -	Modified: now <to> is the complete destination name, since this is the new deal.
 */
-int move_dir(MainInfo *min, const gchar *from, const gchar *to)
+gint move_dir(MainInfo *min, const gchar *from, const gchar *to)
 {
 	gchar		old_dir[PATH_MAX];
 	DIR		*dir;
@@ -109,10 +109,11 @@ int move_dir(MainInfo *min, const gchar *from, const gchar *to)
 /* 1998-07-11 -	Bug fix: always failed after first file when moving on the same device. The
 **		error reporting was also slightly broken.
 */
-int cmd_move(MainInfo *min, DirPane *src, DirPane *dst, CmdArg *ca)
+gint cmd_move(MainInfo *min, DirPane *src, DirPane *dst, const CmdArg *ca)
 {
-	gchar		old_path[PATH_MAX], dest[PATH_MAX], *name = NULL;
-	guint		count = 0;
+	gchar		old_path[PATH_MAX], dest[PATH_MAX];
+	const gchar	*name = NULL;
+	guint		num = 0U;
 	GSList		*slist, *iter;
 	mode_t		mode;
 	OvwRes		ores;
@@ -136,12 +137,7 @@ int cmd_move(MainInfo *min, DirPane *src, DirPane *dst, CmdArg *ca)
 		if(ores == OVW_SKIP)
 			continue;
 		else if(ores == OVW_CANCEL)
-		{
-			if(errno)
-				err_set(min, errno, "move", dp_full_name(src, DP_SEL_INDEX(src, iter)));
 			break;
-		}
-
 		mode = DP_SEL_LSTAT(iter).st_mode;
 		err_clear(min);
 		if(S_ISLNK(mode))
@@ -149,7 +145,7 @@ int cmd_move(MainInfo *min, DirPane *src, DirPane *dst, CmdArg *ca)
 		else
 		{
 			if(rename(name, dest) == 0)		/* Try a direct rename, very quick. */
-				count++;
+				num++;
 			else if(errno == EXDEV)
 			{
 				err_clear(min);
@@ -164,23 +160,23 @@ int cmd_move(MainInfo *min, DirPane *src, DirPane *dst, CmdArg *ca)
 					move_dir(min, DP_SEL_NAME(iter), dest);
 				else
 					fprintf(stderr, "Can't move '%s' -- don't know how!\n", DP_SEL_NAME(iter));
-				count += (errno == 0);
+				num += (errno == 0);
 			}
 			else
 				break;
 		}
 	}
-	dp_free_selection(slist);
+	if(num)				/* If any files were moved, update panes. */
+	{
+		dp_rescan_post_cmd(src);
+		dp_rescan_post_cmd(dst);
+	}
 	pgs_progress_end(min);
 	ovw_overwrite_end(min);
-	if(count > 0)				/* If any files were moved, update panes. */
-	{
-		dp_rescan(src);
-		dp_rescan(dst);
-		fut_cd(old_path, NULL, 0);
-	}
-	if(errno)
-		err_set(min, errno, CMD_ID, name);
+	fut_cd(old_path, NULL, 0);
+	if(errno && iter != NULL)
+		err_set(min, errno, "move", dp_full_name(src, DP_SEL_INDEX(src, iter)));
+	dp_free_selection(slist);
 	err_show(min);
 
 	return errno == 0;
