@@ -17,13 +17,16 @@
 /* 1999-07-03 -	Go up one level in the source pane. Rewritten to add a little more armor
 **		plating. Still far from bullet-proof, though...
 */
-int cmd_parent(MainInfo *min, DirPane *src, DirPane *dst, CmdArg *ca)
+gint cmd_parent(MainInfo *min, DirPane *src, DirPane *dst, const CmdArg *ca)
 {
 	gchar	*ptr;
 	GString	*comp;
 
 	ptr = src->dir.path + strlen(src->dir.path) - 1;
 
+	if(*ptr == G_DIR_SEPARATOR)
+		ptr--;
+
 	comp = g_string_new("");
 	while(ptr >= src->dir.path)
 	{
