@@ -32,12 +32,11 @@ typedef struct {
 
 /* ----------------------------------------------------------------------------------------- */
 
-static void ren_body(MainInfo *min, DirPane *src, DirRow *row, GtkWindow *win, gpointer user)
+static void ren_body(MainInfo *min, DirPane *src, DirRow *row, gpointer user)
 {
-	gchar	temp[PATH_MAX + 256], *name;
-	RenInfo	*ren = (RenInfo *) user;
-
-	gtk_window_set_title(win, _("Rename"));
+	gchar		temp[PATH_MAX + 256];
+	const gchar	*name;
+	RenInfo		*ren = user;
 
 	name = DP_ROW_NAME(row);
 	g_snprintf(temp, sizeof temp, _("Enter New Name For \"%s\""), name);
@@ -53,11 +52,11 @@ static void ren_body(MainInfo *min, DirPane *src, DirRow *row, GtkWindow *win, g
 	}
 }
 
-static int ren_action(MainInfo *min, DirPane *src, DirPane *dst, DirRow *row, gpointer user)
+static gint ren_action(MainInfo *min, DirPane *src, DirPane *dst, DirRow *row, gpointer user)
 {
-	RenInfo	*ren = (RenInfo *) user;
-	gchar	*old_name, *new_name;
-	OvwRes	ores;
+	RenInfo		*ren = user;
+	const gchar	*old_name, *new_name;
+	OvwRes		ores;
 
 	old_name = DP_ROW_NAME(row);
 	new_name = gtk_entry_get_text(GTK_ENTRY(ren->entry));
@@ -65,7 +64,7 @@ static int ren_action(MainInfo *min, DirPane *src, DirPane *dst, DirRow *row, gp
 	/* Disallow attempts to use rename to move files. */
 	if(strchr(new_name, G_DIR_SEPARATOR))
 	{
-		err_set(min, EISDIR, CMD_ID, old_name);		/* Not quite perfect error message, but simple. */
+		err_set(min, EINVAL, CMD_ID, old_name);		/* Not quite perfect error message, but simple. */
 		return 0;
 	}
 	if(strcmp(old_name, new_name) == 0)			/* Ignore renames to self. */
@@ -79,7 +78,8 @@ static int ren_action(MainInfo *min, DirPane *src, DirPane *dst, DirRow *row, gp
 	else if(ores == OVW_CANCEL)
 		return 0;		/* This tells the generic handler to quit. Nice and simple. */
 
-	if(rename(old_name, new_name))
+	err_clear(min);
+	if(rename(old_name, new_name) != 0)
 		err_set(min, errno, CMD_ID, old_name);
 	else
 		dp_unselect(src, DP_ROW_INDEX(src, row));
@@ -94,18 +94,16 @@ static void ren_free(gpointer user)
 
 /* ----------------------------------------------------------------------------------------- */
 
-int cmd_rename(MainInfo *min, DirPane *src, DirPane *dst, CmdArg *ca)
+gint cmd_rename(MainInfo *min, DirPane *src, DirPane *dst, const CmdArg *ca)
 {
 	static RenInfo	ri;
 
 	ri.ovw_open = FALSE;
 	ri.vbox	    = gtk_vbox_new(FALSE, 0);
-	ri.label    = gtk_label_new(_("Rename"));
+	ri.label    = gtk_label_new("");
 	ri.entry    = gtk_entry_new_with_max_length(MAXNAMLEN - 1);
 	gtk_box_pack_start(GTK_BOX(ri.vbox), ri.label, FALSE, FALSE, 0);
 	gtk_box_pack_start(GTK_BOX(ri.vbox), ri.entry, FALSE, FALSE, 0);
-	gtk_widget_show(ri.label);
-	gtk_widget_show(ri.entry);
 
-	return cmd_generic(min, CGF_NOALL | CGF_SRC | CGF_NODST, ren_body, ren_action, ren_free, &ri);
+	return cmd_generic(min, _("Rename"), CGF_NOALL | CGF_SRC | CGF_NODST, ren_body, ren_action, ren_free, &ri);
 }
