@@ -21,6 +21,7 @@
 #include "dialog.h"
 #include "dirpane.h"
 #include "errors.h"
+#include "fileutil.h"
 #include "overwrite.h"
 #include "strutil.h"
 
@@ -83,7 +84,7 @@ static gint rename_simple(RenREInfo *ri)
 		{
 			for(; base < ptr; base++)
 				g_string_append_c(nn, *base);
-			if((nocase ? stu_strcasencmp(ptr, from, flen) : strncmp(ptr, from, flen)) == 0)
+			if((nocase ? g_strncasecmp(ptr, from, flen) : strncmp(ptr, from, flen)) == 0)
 			{
 				g_string_append(nn, to);
 				base += flen;
@@ -106,7 +107,7 @@ static gint rename_simple(RenREInfo *ri)
 			if(rename(DP_SEL_NAME(iter), nn->str))
 			{
 				err = errno;
-				dp_rescan(ri->src);
+				dp_rescan_post_cmd(ri->src);
 				err_set(ri->min, err, "RenameRE", DP_SEL_NAME(iter));
 			}
 			else
@@ -136,7 +137,7 @@ static gboolean interpolate(GString *str, const gchar *def, const gchar *src, co
 		if(*ptr == '$')
 		{
 			ptr++;
-			if(isdigit((unsigned char) *ptr))
+			if(isdigit((guchar) *ptr))
 			{
 				gint	slot, i;
 
@@ -178,7 +179,7 @@ static gint rename_regexp(RenREInfo *ri)
 		GString		*nn;
 		regmatch_t	match[10];
 
-		ovw_overwrite_begin(ri->min, _("\"%s\" Already Exists - Proeceed With Rename?"), 0U);
+		ovw_overwrite_begin(ri->min, _("\"%s\" Already Exists - Proceed With Rename?"), 0U);
 		nn = g_string_new("");
 		for(iter = ri->selection; iter != NULL; iter = g_slist_next(iter))
 		{
@@ -198,7 +199,7 @@ static gint rename_regexp(RenREInfo *ri)
 						if(rename(DP_SEL_NAME(iter), nn->str))
 						{
 							err = errno;
-							dp_rescan(ri->src);
+							dp_rescan_post_cmd(ri->src);
 							err_set(ri->min, err, "RenameRE", DP_SEL_NAME(iter));
 						}
 						else
@@ -218,7 +219,6 @@ static gint rename_regexp(RenREInfo *ri)
 		regerror(reerr, &fromre, buf + strlen(buf), sizeof buf - strlen(buf) - 1);
 		dlg_dialog_async_new_error(buf);
 	}
-
 	return err;
 }
 
@@ -232,7 +232,7 @@ static void evt_nbook_switchpage(GtkWidget *wid, GtkNotebookPage *page, gint pag
 	gtk_widget_grab_focus(ri->page ? ri->re_from : ri->r_from);
 }
 
-int cmd_renamere(MainInfo *min, DirPane *src, DirPane *dst, CmdArg *ca)
+gint cmd_renamere(MainInfo *min, DirPane *src, DirPane *dst, const CmdArg *ca)
 {
 	GtkWidget		*label, *table;
 	static RenREInfo	ri = { NULL };
@@ -295,7 +295,7 @@ int cmd_renamere(MainInfo *min, DirPane *src, DirPane *dst, CmdArg *ca)
 		gtk_notebook_append_page(GTK_NOTEBOOK(ri.nbook), ri.re_vbox, gtk_label_new(_("Reg Exp")));
 		gtk_widget_show_all(ri.re_vbox);
 
-		ri.dlg = dlg_dialog_sync_new(ri.nbook, "RenameRE", _("OK|Cancel"));
+		ri.dlg = dlg_dialog_sync_new(ri.nbook, "RenameRE", NULL);
 
 		ri.page = 0;
 	}
@@ -304,15 +304,22 @@ int cmd_renamere(MainInfo *min, DirPane *src, DirPane *dst, CmdArg *ca)
 
 	if(dlg_dialog_sync_wait(ri.dlg) == DLG_POSITIVE)
 	{
-		gint	err;
+		gchar	old[PATH_MAX];
+		gint	err = 0;
 
 		err_clear(ri.min);
-		if(ri.page == 0)
-			err = rename_simple(&ri);
+		if(fut_cd(src->dir.path, old, sizeof old))
+		{
+			if(ri.page == 0)
+				err = rename_simple(&ri);
+			else
+				err = rename_regexp(&ri);
+			fut_cd(old, NULL, 0U);
+		}
 		else
-			err = rename_regexp(&ri);
-		if(!err)
-			dp_rescan(ri.src);
+			err_set(ri.min, errno, "CD", src->dir.path);
+		if(!err && !errno)
+			dp_rescan_post_cmd(ri.src);
 		else
 			err_show(ri.min);
 	}
