@@ -260,11 +260,11 @@ static void evt_guess_clicked(GtkWidget *wid, gpointer user)
 	guint		prec = 0;
 
 	head = g_string_new("");
-	for(ptr = dlg->name; *ptr && !isdigit((int) *ptr); ptr++)
+	for(ptr = dlg->name; *ptr && !isdigit((guchar) *ptr); ptr++)
 		g_string_append_c(head, *ptr);
-	if(isdigit((int) *ptr))	/* Did we find a digit? */
+	if(isdigit((guchar) *ptr))	/* Did we find a digit? */
 	{
-		for(; *ptr && isdigit((int) *ptr); prec++, ptr++)
+		for(; *ptr && isdigit((guchar) *ptr); prec++, ptr++)
 			;
 		gtk_entry_set_text(GTK_ENTRY(dlg->tail), ptr);
 	}
@@ -279,10 +279,10 @@ static void evt_guess_clicked(GtkWidget *wid, gpointer user)
 }
 
 /* 2001-10-03 -	Well, here's the actual command entrypoint, like usual. */
-int cmd_renameseq(MainInfo *min, DirPane *src, DirPane *dst, CmdArg *ca)
+gint cmd_renameseq(MainInfo *min, DirPane *src, DirPane *dst, const CmdArg *ca)
 {
 	static RenSeqInfo	rsi = { NULL };
-	const gchar		*bname[] = { N_("Octal"), N_("Decimal"), N_("Hex (a-f)"), N_("Hex (A-F)") };
+	const gchar		*bname[] = { N_("8, Octal"), N_("10, Decimal"), N_("16, Hex (a-f)"), N_("16, Hex (A-F)") };
 	GtkObject		*adj;
 	GtkWidget		*vbox, *table, *label, *bmenu, *bmitem;
 	RenSeqDialog		dlg;
@@ -371,7 +371,7 @@ int cmd_renameseq(MainInfo *min, DirPane *src, DirPane *dst, CmdArg *ca)
 
 	preview_update(&dlg);
 
-	dlg.dlg = dlg_dialog_sync_new(vbox, _("Sequential Rename"), _("OK|Cancel"));
+	dlg.dlg = dlg_dialog_sync_new(vbox, _("Sequential Rename"), NULL);
 	ret = dlg_dialog_sync_wait(dlg.dlg);
 
 	if(ret == DLG_POSITIVE)
@@ -379,7 +379,7 @@ int cmd_renameseq(MainInfo *min, DirPane *src, DirPane *dst, CmdArg *ca)
 		rsi_update(&dlg);
 		err_clear(min);
 		ret = do_renameseq(&rsi, src);
-		dp_rescan(src);
+		dp_rescan_post_cmd(src);
 		if(!ret)
 			err_show(min);
 	}
