@@ -34,6 +34,7 @@
 #include "dpformat.h"
 #include "events.h"
 #include "strutil.h"
+#include "types.h"
 
 #include "cmd_select.h"
 
@@ -73,7 +74,7 @@ static SelParam	old_sp = { NULL, SM_SET_ALL, SM_TYPE_ALL, SM_ACTION_SELECT, DPC_
 /* ----------------------------------------------------------------------------------------- */
 
 /* Select a specific row, or, if the row=ROW keyword is not specified, the one that was last clicked. */
-int cmd_selectrow(MainInfo *min, DirPane *src, DirPane *dst, CmdArg *ca)
+gint cmd_selectrow(MainInfo *min, DirPane *src, DirPane *dst, const CmdArg *ca)
 {
 	GdkEventButton	*evt;
 	const gchar	*num;
@@ -103,7 +104,8 @@ int cmd_selectrow(MainInfo *min, DirPane *src, DirPane *dst, CmdArg *ca)
 		action = car_keyword_get_enum(ca, "action", 0, "select", "unselect", "toggle", NULL);
 		afunc = (action == 0) ? dp_select : (action == 1) ? dp_unselect : dp_toggle;
 		afunc(src, row);
-		gtk_main_iteration();
+		while(gtk_events_pending())
+			gtk_main_iteration();
 	}
 	return 1;
 }
@@ -113,7 +115,7 @@ int cmd_selectrow(MainInfo *min, DirPane *src, DirPane *dst, CmdArg *ca)
 /* 1999-03-04 -	Select all rows in the currently active pane. Rewritten for the new cooler
 **		selection management made possible by GTK+ 1.2.0.
 */
-int cmd_selectall(MainInfo *min, DirPane *src, DirPane *dst, CmdArg *ca)
+gint cmd_selectall(MainInfo *min, DirPane *src, DirPane *dst, const CmdArg *ca)
 {
 	dp_select_all(src);
 	return 1;
@@ -122,7 +124,7 @@ int cmd_selectall(MainInfo *min, DirPane *src, DirPane *dst, CmdArg *ca)
 /* ----------------------------------------------------------------------------------------- */
 
 /* 1999-03-04 -	Unselect all rows of current pane. Rewritten. */
-int cmd_selectnone(MainInfo *min, DirPane *src, DirPane *dst, CmdArg *ca)
+gint cmd_selectnone(MainInfo *min, DirPane *src, DirPane *dst, const CmdArg *ca)
 {
 	dp_unselect_all(src);
 	return 1;
@@ -132,15 +134,56 @@ int cmd_selectnone(MainInfo *min, DirPane *src, DirPane *dst, CmdArg *ca)
 
 /* 1998-06-04 -	Invert the selection set, making all currently selected files become
 **		unselected, and all unselected become selected.
+** 2002-08-25 -	Complete rewrite, now uses bit vector for speed. 40-100X faster.
 */
-int cmd_selecttoggle(MainInfo *min, DirPane *src, DirPane *dst, CmdArg *ca)
+gint cmd_selecttoggle(MainInfo *min, DirPane *src, DirPane *dst, const CmdArg *ca)
 {
-	guint	i;
+	guint	*bm, word, row, mask;
+	gsize	alloc;
+	GSList	*slist;
+
+	/* Compute size of, and allocate selection bit vector. */
+	alloc = (src->dir.num_rows + CHAR_BIT * sizeof *bm - 1) / (CHAR_BIT * sizeof *bm);
+	bm = g_malloc(alloc * sizeof *bm);
+	memset(bm, 0, alloc * sizeof *bm);	/* Assume no selection. */
 
 	gtk_clist_freeze(src->list);
-	for(i = 0; i < src->dir.num_rows; i++)
-		dp_toggle(src, i);
+
+	/* Was there a selection? If so, set bits for selected rows. */
+	if((slist = dp_get_selection(src)) != NULL)
+	{
+		const GSList	*iter;
+
+		for(iter = slist; iter != NULL; iter = g_slist_next(iter))
+		{
+			register guint	n;
+
+			n = DP_ROW_INDEX(src, DP_SEL_ROW(iter));
+			bm[n / (CHAR_BIT * sizeof *bm)] |= 1 << (n % (CHAR_BIT * sizeof *bm));
+		}
+		dp_free_selection(slist);
+	}
+	/* Now invert the selection bitmap. */
+	for(word = 0; word < alloc; word++)
+		bm[word] = ~bm[word];
+	/* Clear selection, and apply the (inverted) bitmap. */
+	dp_unselect_all(src);
+	for(word = row = 0; row < src->dir.num_rows; word++)
+	{
+		if(!bm[word])		/* Test a bunch (typically 32) rows in parallel. */
+		{
+			row += CHAR_BIT * sizeof *bm;
+			continue;
+		}
+		for(mask = 1; mask; mask <<= 1, row++)
+		{
+			if(bm[word] & mask)
+				dp_select(src, row);
+		}
+	}
 	gtk_clist_thaw(src->list);
+	g_free(bm);
+
 	return 1;
 }
 
@@ -274,7 +317,6 @@ static GtkWidget * build_mode_menu(const gchar *def)
 			item = gtk_menu_item_new_with_label(iname);
 			gtk_menu_append(GTK_MENU(menu), item);
 			gtk_object_set_user_data(GTK_OBJECT(item), GUINT_TO_POINTER(minor));
-			gtk_widget_show(item);
 		}
 		g_free(temp);
 	}
@@ -290,9 +332,7 @@ static GtkWidget * build_content_menu(DirPane *dp)
 	guint		i;
 
 	dpf = &min->cfg.dp_format[dp->index];
-
 	menu = gtk_menu_new();
-
 	for(i = 0; i < dpf->num_columns; i++)
 	{
 		if(dpf->format[i].content == DPC_ICON)
@@ -300,7 +340,6 @@ static GtkWidget * build_content_menu(DirPane *dp)
 		item = gtk_menu_item_new_with_label(dpf_get_content_name(dpf->format[i].content));
 		gtk_object_set_user_data(GTK_OBJECT(item), GUINT_TO_POINTER(dpf->format[i].content));
 		gtk_menu_append(GTK_MENU(menu), item);
-		gtk_widget_show(item);
 	}
 	return menu;
 }
@@ -319,28 +358,22 @@ static void init_menus(DirPane *dp, GtkWidget *vbox, GtkWidget *opt_menu[4], gui
 	for(i = 0; i < sizeof mdef / sizeof mdef[0]; i++)
 	{
 		label = gtk_label_new(_(mlabel[i]));
-		gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, i, i + 1);
-		gtk_widget_show(label);
+		gtk_table_attach(GTK_TABLE(table), label, 0, 1, i, i + 1, 0,0,0,0);
 		menu = build_mode_menu(_(mdef[i]));
 		opt_menu[i] = gtk_option_menu_new();
 		gtk_option_menu_set_menu(GTK_OPTION_MENU(opt_menu[i]), menu);
 		gtk_option_menu_set_history(GTK_OPTION_MENU(opt_menu[i]), *history[i]);
-		gtk_table_attach_defaults(GTK_TABLE(table), opt_menu[i], 1, 2, i, i+1);
-		gtk_widget_show(opt_menu[i]);
+		gtk_table_attach(GTK_TABLE(table), opt_menu[i], 1, 2, i, i+1, GTK_EXPAND|GTK_FILL,0,0,0);
 	}
-
 	label = gtk_label_new(_("Content"));
-	gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, i, i + 1);
-	gtk_widget_show(label);
+	gtk_table_attach(GTK_TABLE(table), label, 0, 1, i, i + 1, 0,0,0,0);
 	menu = build_content_menu(dp);
 	opt_menu[3] = gtk_option_menu_new();
 	gtk_option_menu_set_menu(GTK_OPTION_MENU(opt_menu[3]), menu);
 	gtk_option_menu_set_history(GTK_OPTION_MENU(opt_menu[3]), *history[3]);
-	gtk_table_attach_defaults(GTK_TABLE(table), opt_menu[3], 1, 2, i, i + 1);
-	gtk_widget_show(opt_menu[3]);
+	gtk_table_attach(GTK_TABLE(table), opt_menu[3], 1, 2, i, i + 1, GTK_EXPAND|GTK_FILL,0,0,0);
 
 	gtk_box_pack_start(GTK_BOX(vbox), table, FALSE, FALSE, 0);
-	gtk_widget_show(table);
 }
 
 /* 1998-06-04 -	Pop up a dialog window asking the user for a regular expression, and
@@ -349,7 +382,7 @@ static void init_menus(DirPane *dp, GtkWidget *vbox, GtkWidget *opt_menu[4], gui
 **		the GUI uses no external state; all values are extracted from the
 **		widgets after the dialog is closed.
 */
-int cmd_selectre(MainInfo *min, DirPane *src, DirPane *dst, CmdArg *ca)
+gint cmd_selectre(MainInfo *min, DirPane *src, DirPane *dst, const CmdArg *ca)
 {
 	SelParam	sp;
 	const gchar	*str;
@@ -399,19 +432,15 @@ int cmd_selectre(MainInfo *min, DirPane *src, DirPane *dst, CmdArg *ca)
 		glob = gtk_check_button_new_with_label(_("Treat RE as Glob Pattern?"));
 		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(glob), sp.glob);
 		gtk_box_pack_start(GTK_BOX(vbox), glob, FALSE, FALSE, 0);
-		gtk_widget_show(glob);
 		invert = gtk_check_button_new_with_label(_("Invert RE Matching?"));
 		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(invert), sp.invert);
 		gtk_box_pack_start(GTK_BOX(vbox), invert, FALSE, FALSE, 0);
-		gtk_widget_show(invert);
 		full = gtk_check_button_new_with_label(_("Require Match on Full Name?"));
 		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(full), sp.full);
 		gtk_box_pack_start(GTK_BOX(vbox), full, FALSE, FALSE, 0);
-		gtk_widget_show(full);
 		nocase = gtk_check_button_new_with_label(_("Ignore Case?"));
 		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(nocase), sp.nocase);
 		gtk_box_pack_start(GTK_BOX(vbox), nocase, FALSE, FALSE, 0);
-		gtk_widget_show(nocase);
 
 		re = gtk_entry_new();
 		if(sp.re != NULL)
@@ -420,8 +449,7 @@ int cmd_selectre(MainInfo *min, DirPane *src, DirPane *dst, CmdArg *ca)
 			gtk_entry_select_region(GTK_ENTRY(re), 0, -1);
 		}
 		gtk_box_pack_start(GTK_BOX(vbox), re, FALSE, FALSE, 0);
-		gtk_widget_show(re);
-		dlg = dlg_dialog_sync_new(vbox, _("Select Using RE"), _("OK|Cancel"));
+		dlg = dlg_dialog_sync_new(vbox, _("Select Using RE"), NULL);
 		gtk_widget_grab_focus(re);
 		res = dlg_dialog_sync_wait(dlg);
 		if(res == DLG_POSITIVE)		/* If user accepted, collect widget state into 'sp'. */
@@ -466,7 +494,7 @@ int cmd_selectre(MainInfo *min, DirPane *src, DirPane *dst, CmdArg *ca)
 **		after selecting one, and things like that. Assumes the style tree layout is
 **		sensible (i.e. rather deep and with "abstract" parents).
 */
-int cmd_selectext(MainInfo *min, DirPane *src, DirPane *dst, CmdArg *ca)
+gint cmd_selectext(MainInfo *min, DirPane *src, DirPane *dst, const CmdArg *ca)
 {
 	GSList	*slist, *iter;
 	guint	i;
@@ -495,7 +523,7 @@ int cmd_selectext(MainInfo *min, DirPane *src, DirPane *dst, CmdArg *ca)
 **		it to determine which row has the desired suffix. If not, use the focus if
 **		one exists. Else, do nothing.
 */
-int cmd_selectsuffix(MainInfo *min, DirPane *src, DirPane *dst, CmdArg *ca)
+gint cmd_selectsuffix(MainInfo *min, DirPane *src, DirPane *dst, const CmdArg *ca)
 {
 	GdkEventButton	*evt;
 	gint		row = -1;
@@ -539,36 +567,45 @@ int cmd_selectsuffix(MainInfo *min, DirPane *src, DirPane *dst, CmdArg *ca)
 ** NOTE NOTE	This command replaces a command that had the same name, but that I think
 **		wasn't very useful. If you disagree, get in touch. Thanks.
 */
-int cmd_selecttype(MainInfo *min, DirPane *src, DirPane *dst, CmdArg *ca)
+gint cmd_selecttype(MainInfo *min, DirPane *src, DirPane *dst, const CmdArg *ca)
 {
 	GdkEventButton	*evt;
+	const gchar	*tname;
 	gint		row = -1;
+	GList		*types = NULL;
 
-	if((evt = (GdkEventButton *) evt_event_get(GDK_BUTTON_PRESS)) != NULL)
-	{
-		gint	col;
-
-		if(!gtk_clist_get_selection_info(GTK_CLIST(src->list), evt->x, evt->y, &row, &col) || (row > src->dir.num_rows))
-			row = -1;
-	}
+	if((tname = car_bareword_get(ca, 0U)) != NULL)
+		types = typ_type_lookup_glob(min->cfg.type, tname);
 	else
-		row = src->focus_row;
-
-	if(row != -1)
 	{
-		FType	*type;
+		if((evt = (GdkEventButton *) evt_event_get(GDK_BUTTON_PRESS)) != NULL)
+		{
+			gint	col;
+
+			if(!gtk_clist_get_selection_info(GTK_CLIST(src->list), evt->x, evt->y, &row, &col) || (row > src->dir.num_rows))
+				row = -1;
+		}
+		else
+			row = src->focus_row;
+		if(row != -1)
+			types = g_list_append(NULL, (gpointer) DP_ROW_TYPE(&src->dir.row[row]));
+	}
+
+	if(types != NULL)
+	{
+
 		guint	action, i;
 		void	(*afunc)(DirPane *dp, gint row);
 
-		type   = DP_ROW_TYPE(&src->dir.row[row]);
 		action = car_keyword_get_enum(ca, "action", 0, "select", "unselect", "toggle", NULL);
 		afunc  = (action == 0) ? dp_select : (action == 1) ? dp_unselect : dp_toggle;
 
 		for(i = 0; i < src->dir.num_rows; i++)
 		{
-			if(DP_ROW_TYPE(&src->dir.row[i]) == type)
+			if(g_list_find(types, (gpointer) DP_ROW_TYPE(&src->dir.row[i])) != NULL)
 				afunc(src, i);
 		}
+		g_list_free(types);
 	}
 	return 1;
 }
@@ -604,13 +641,13 @@ static gboolean cmp_exec(const DirRow *row, const DirPane *dst, const struct cmp
 	return FALSE;
 }
 
-int cmd_selectcmp(MainInfo *min, DirPane *src, DirPane *dst, CmdArg *ca)
+gint cmd_selectcmp(MainInfo *min, DirPane *src, DirPane *dst, const CmdArg *ca)
 {
 	static struct cmp_rules	rules = { FALSE };
 	gint			i;
 
 	/* Select string comparison function to use. */
-	rules.scmp  = rules.nocase ? stu_strcasecmp : strcmp;
+	rules.scmp  = rules.nocase ? g_strcasecmp : strcmp;
 
 	for(i = 0; i < src->dir.num_rows; i++)
 	{
@@ -626,7 +663,7 @@ int cmd_selectcmp(MainInfo *min, DirPane *src, DirPane *dst, CmdArg *ca)
 /* 1998-09-20 -	The UnselectFirst command. In wrong module?
 ** 1998-12-16 -	This was freeze/thaw bracketed, for no good reason. Didn't work.
 */
-int cmd_unselectfirst(MainInfo *min, DirPane *src, DirPane *dst, CmdArg *ca)
+gint cmd_unselectfirst(MainInfo *min, DirPane *src, DirPane *dst, const CmdArg *ca)
 {
 	GSList	*slist;
 
