@@ -18,12 +18,13 @@
 #include "overwrite.h"
 #include "guiutil.h"
 
-#include "cfg_gui.h"
 #include "cmd_generic.h"
 #include "cmd_split.h"
 
 #define	CMD_ID	"split"
 
+#define	NFORMAT_LIMIT	(MAXNAMLEN + 16)
+
 #define	THRESH_KB	(1.0)
 #define	THRESH_MB	(0.5)
 #define	THRESH_GB	(0.5)
@@ -53,36 +54,32 @@ typedef struct {
 /* ----------------------------------------------------------------------------------------- */
 
 /* 1998-09-01 -	Pretty much rewrote this function, now that the sizeutil module exists. */
-static void spt_body(MainInfo *min, DirPane *src, DirRow *row, GtkWindow *win, gpointer user)
+static void spt_body(MainInfo *min, DirPane *src, DirRow *row, gpointer user)
 {
-	gchar		*name, tmp1[2 * MAXNAMLEN], *tptr = tmp1, siz1[32], siz2[32];
-	SplitInfo	*spi = (SplitInfo *) user;
+	gchar		buf[2 * MAXNAMLEN], siz1[32], siz2[32];
+	SplitInfo	*spi = user;
 
-	gtk_window_set_title(win, _("Split"));
-
-	name = DP_ROW_NAME(row);
-	tptr += g_snprintf(tptr, sizeof tmp1, _("Split \"%s\"\nFile is "), name);
-	sze_put_size(DP_ROW_LSTAT(row).st_size, siz1, sizeof siz1, SZE_BYTES);
-	sze_put_size(DP_ROW_LSTAT(row).st_size, siz2, sizeof siz2, SZE_AUTO);
-	if(strcmp(siz1, siz2))
-		sprintf(tptr, "%s (%s)", siz1, siz2);
+	sze_put_size(siz1, sizeof siz1, DP_ROW_LSTAT(row).st_size, SZE_BYTES);
+	sze_put_size(siz2, sizeof siz2, DP_ROW_LSTAT(row).st_size, SZE_AUTO);
+	if(strcmp(siz1, siz2) != 0)
+		g_snprintf(buf, sizeof buf, _("Split \"%s\".\nFile is %s (%s)."), DP_ROW_NAME(row), siz1, siz2);
 	else
-		strcat(tptr, siz1);
-	gtk_label_set_text(GTK_LABEL(spi->label), tmp1);
-	gtk_entry_set_text(GTK_ENTRY(spi->nformat), name);
+		g_snprintf(buf, sizeof buf, _("Split \"%s\".\nFile is %s."), DP_ROW_NAME(row), siz1);
+	gtk_label_set_text(GTK_LABEL(spi->label), buf);
+	gtk_entry_set_text(GTK_ENTRY(spi->nformat), DP_ROW_NAME(row));
 	gtk_entry_append_text(GTK_ENTRY(spi->nformat), ".%03d");
 }
 
 /* 1998-09-01 -	Construct destination file name for piece number <piece> in a split. */
-static void get_name(SplitInfo *spi, DirPane *dst, gchar *buf, int piece)
+static void build_name(SplitInfo *spi, DirPane *dst, gchar *buf, gint piece)
 {
-	gchar	*ptr = buf;
+	gchar	fmt[sizeof G_DIR_SEPARATOR_S + NFORMAT_LIMIT + 8];
 	gint	base = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spi->nbase)),
 		step = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spi->nstep)),
 		num  = base + step * piece;
 
-	ptr += sprintf(ptr, "%s/", dst->dir.path);
-	sprintf(ptr, gtk_entry_get_text(GTK_ENTRY(spi->nformat)), num);
+	g_snprintf(fmt, sizeof fmt, "%%s%s%s", G_DIR_SEPARATOR_S, gtk_entry_get_text(GTK_ENTRY(spi->nformat)));
+	g_snprintf(buf, PATH_MAX, fmt, dst->dir.path, num);
 }
 
 /* 1998-09-01 -	Split file on row <row> of <src>, producing a bunch of segment files in <dst>.
@@ -92,24 +89,24 @@ static void get_name(SplitInfo *spi, DirPane *dst, gchar *buf, int piece)
 ** 1999-11-13 -	Adapted to use new fut_copy_partial() function, fixed bug with closing output
 **		file on failed write.
 */
-static int size_split(SplitInfo *spi, DirPane *src, DirPane *dst, DirRow *row)
+static gint size_split(SplitInfo *spi, DirPane *src, DirPane *dst, DirRow *row)
 {
 	gchar	inname[PATH_MAX], outname[PATH_MAX];
 	gsize	seg_size, to_go, chunk;
 	gint	ifd, ofd, piece = 0;
 	OvwRes	ores;
 
-	ovw_overwrite_begin(spi->min, _("\"%s\" Exists - Continue With Split?"), 0UL);
+	ovw_overwrite_begin(spi->min, _("\"%s\" Already Exists - Continue With Split?"), 0UL);
 
 	to_go = DP_ROW_LSTAT(row).st_size;
 	if((seg_size = sze_get_size(gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(spi->sssize)->entry)))) > 0)
 	{
-		g_snprintf(inname, sizeof inname, "%s/%s", src->dir.path, DP_ROW_NAME(row));
+		g_snprintf(inname, sizeof inname, "%s%s%s", src->dir.path, G_DIR_SEPARATOR_S, DP_ROW_NAME(row));
 		if((ifd = open(inname, O_RDONLY)) > 0)
 		{
 			while(to_go > 0)
 			{
-				get_name(spi, dst, outname, piece++);
+				build_name(spi, dst, outname, piece++);
 				ores = ovw_overwrite_file(spi->min, outname, dp_full_name(src, DP_ROW_INDEX(src, row)));
 				chunk = (to_go > seg_size) ? seg_size : to_go;
 				if(ores == OVW_CANCEL)
@@ -145,10 +142,10 @@ static int size_split(SplitInfo *spi, DirPane *src, DirPane *dst, DirRow *row)
 	return to_go == 0;
 }
 
-static int spt_action(MainInfo *min, DirPane *src, DirPane *dst, DirRow *row, gpointer user)
+static gint spt_action(MainInfo *min, DirPane *src, DirPane *dst, DirRow *row, gpointer user)
 {
-	SplitInfo	*spi = (SplitInfo *) user;
-	int		ret = 0;
+	SplitInfo	*spi = user;
+	gint		ret = 0;
 
 	if(spi->curr_mode == 0)		/* Fixed-size split? */
 		ret = size_split(spi, src, dst, row);
@@ -173,30 +170,25 @@ static void build_ss(SplitInfo *spi)
 	list = g_list_append(list, _("100431360 bytes (Zip disk)"));
 
 	spi->ssframe = gtk_frame_new(_("Fixed Size Split"));
-	vbox = gtk_vbox_new(FALSE, 0);
-	hbox = gtk_hbox_new(FALSE, 0);
+	vbox  = gtk_vbox_new(FALSE, 0);
+	hbox  = gtk_hbox_new(FALSE, 0);
 	label = gtk_label_new(_("Segment Size"));
 	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
-	gtk_widget_show(label);
 	spi->sssize = gtk_combo_new();
 	gtk_combo_set_popdown_strings(GTK_COMBO(spi->sssize), list);
 	gtk_box_pack_start(GTK_BOX(hbox), spi->sssize, TRUE, TRUE, 0);
-	gtk_widget_show(spi->sssize);
 	gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, 0);
-	gtk_widget_show(hbox);
 
 	gtk_container_add(GTK_CONTAINER(spi->ssframe), vbox);
-	gtk_widget_show(vbox);
 	gtk_box_pack_start(GTK_BOX(spi->vbox), spi->ssframe, TRUE, TRUE, 0);
-	gtk_widget_show(spi->ssframe);
 }
 
 /* ----------------------------------------------------------------------------------------- */
 
 /* 1998-09-01 -	Callback for mode option menu. */
-static gint evt_mode_select(GtkWidget *wid, gpointer data)
+static gint evt_mode_select(GtkWidget *wid, gpointer user)
 {
-	SplitInfo	*spi = (SplitInfo *) data;
+	SplitInfo	*spi = user;
 
 	spi->curr_mode = GPOINTER_TO_UINT(gtk_object_get_user_data(GTK_OBJECT(wid)));
 
@@ -205,7 +197,7 @@ static gint evt_mode_select(GtkWidget *wid, gpointer data)
 
 /* ----------------------------------------------------------------------------------------- */
 
-int cmd_split(MainInfo *min, DirPane *src, DirPane *dst, CmdArg *ca)
+gint cmd_split(MainInfo *min, DirPane *src, DirPane *dst, const CmdArg *ca)
 {
 	static SplitInfo	spi;
 	const gchar		*mode[] = { N_("Fixed size, variable number of parts"),
@@ -222,43 +214,31 @@ int cmd_split(MainInfo *min, DirPane *src, DirPane *dst, CmdArg *ca)
 	hbox = gtk_hbox_new(FALSE, 0);
 	label = gtk_label_new(_("Mode"));
 	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
-	gtk_widget_show(label);
 	spi.mode = gtk_option_menu_new();
 	gtk_box_pack_start(GTK_BOX(spi.vbox), spi.label, FALSE, FALSE, 0);
-	gtk_widget_show(spi.label);
 	hsep = gtk_hseparator_new();
 	gtk_box_pack_start(GTK_BOX(spi.vbox), hsep, FALSE, FALSE, 4);
-	gtk_widget_show(hsep);
 	gtk_option_menu_set_menu(GTK_OPTION_MENU(spi.mode), gui_build_menu(mode, GTK_SIGNAL_FUNC(evt_mode_select), &spi));
 	gtk_box_pack_start(GTK_BOX(hbox), spi.mode, TRUE, TRUE, 0);
-	gtk_widget_show(spi.mode);
 	gtk_box_pack_start(GTK_BOX(spi.vbox), hbox, FALSE, FALSE, 0);
-	gtk_widget_show(hbox);
 	spi.nhbox = gtk_hbox_new(FALSE, 0);
 	label = gtk_label_new(_("Name Format"));
 	gtk_box_pack_start(GTK_BOX(spi.nhbox), label, FALSE, FALSE, 2);
-	gtk_widget_show(label);
-	spi.nformat = gtk_entry_new();
+	spi.nformat = gtk_entry_new_with_max_length(NFORMAT_LIMIT);
 	gtk_box_pack_start(GTK_BOX(spi.nhbox), spi.nformat, TRUE, TRUE, 0);
-	gtk_widget_show(spi.nformat);
 	label = gtk_label_new(_("Base"));
 	gtk_box_pack_start(GTK_BOX(spi.nhbox), label, FALSE, FALSE, 0);
-	gtk_widget_show(label);
 	spi.nbadj   = gtk_adjustment_new(0, 0, 31, 1, 16, 16);
 	spi.nbase   = gtk_spin_button_new(GTK_ADJUSTMENT(spi.nbadj), 0, 0);
 	gtk_box_pack_start(GTK_BOX(spi.nhbox), spi.nbase, FALSE, FALSE, 0);
-	gtk_widget_show(spi.nbase);
 	label = gtk_label_new(_("Step"));
 	gtk_box_pack_start(GTK_BOX(spi.nhbox), label, FALSE, FALSE, 0);
-	gtk_widget_show(label);
 	spi.nsadj = gtk_adjustment_new(1, 1, 31, 1, 16, 16);
 	spi.nstep = gtk_spin_button_new(GTK_ADJUSTMENT(spi.nsadj), 0, 0);
 	gtk_box_pack_start(GTK_BOX(spi.nhbox), spi.nstep, FALSE, FALSE, 0);
-	gtk_widget_show(spi.nstep);
 	gtk_box_pack_start(GTK_BOX(spi.vbox), spi.nhbox, FALSE, FALSE, 2);
-	gtk_widget_show(spi.nhbox);
 
 	build_ss(&spi);
 
-	return cmd_generic(min, CGF_NOALL | CGF_NODIRS, spt_body, spt_action, NULL, &spi);
+	return cmd_generic(min, _("Split"), CGF_NOALL | CGF_NODIRS, spt_body, spt_action, NULL, &spi);
 }
