@@ -15,6 +15,7 @@
 #include "guiutil.h"
 #include "overwrite.h"
 #include "strutil.h"
+
 #include "cmd_delete.h"
 #include "cmd_generic.h"
 #include "cmd_symlink.h"
@@ -45,7 +46,7 @@ static gboolean set_link(MainInfo *min, const gchar *name, const gchar *contents
 		gboolean	ok;
 
 		if(S_ISDIR(stbuf.st_mode))
-			ok = del_delete_directory(min, name);
+			ok = del_delete_directory(min, name, FALSE);
 		else
 			ok = del_delete_file(min, name);
 		if(!ok)
@@ -58,7 +59,7 @@ static gboolean set_link(MainInfo *min, const gchar *name, const gchar *contents
 }
 
 /* 1999-05-29 -	Create (absolute) symbolic links for all selected entries in <src> at <dst>. */
-int cmd_symlink(MainInfo *min, DirPane *src, DirPane *dst, CmdArg *ca)
+gint cmd_symlink(MainInfo *min, DirPane *src, DirPane *dst, const CmdArg *ca)
 {
 	gchar	dest[PATH_MAX];
 	GSList	*slist, *iter;
@@ -99,7 +100,7 @@ int cmd_symlink(MainInfo *min, DirPane *src, DirPane *dst, CmdArg *ca)
 	dp_free_selection(slist);
 
 	if(num)
-		dp_rescan(dst);
+		dp_rescan_post_cmd(dst);
 
 	return errno == 0;
 }
@@ -107,11 +108,10 @@ int cmd_symlink(MainInfo *min, DirPane *src, DirPane *dst, CmdArg *ca)
 /* ----------------------------------------------------------------------------------------- */
 
 /* 1999-05-30 -	Update generic command dialog for a new link. */
-static void link_body(MainInfo *min, DirPane *src, DirRow *row, GtkWindow *win, gpointer user)
+static void link_body(MainInfo *min, DirPane *src, DirRow *row, gpointer user)
 {
 	SleInfo	*sle = user;
 
-	gtk_window_set_title(win, _("Edit Symbolic Link"));
 	gtk_entry_set_text(GTK_ENTRY(sle->name), DP_ROW_NAME(row));
 	gtk_entry_set_text(GTK_ENTRY(sle->contents), DP_ROW_LINKNAME(row));
 	gtk_entry_select_region(GTK_ENTRY(sle->contents), 0, -1);
@@ -119,7 +119,7 @@ static void link_body(MainInfo *min, DirPane *src, DirRow *row, GtkWindow *win,
 }
 
 /* 1999-05-30 -	Change a link. */
-static int link_action(MainInfo *min, DirPane *src, DirPane *dst, DirRow *row, gpointer user)
+static gint link_action(MainInfo *min, DirPane *src, DirPane *dst, DirRow *row, gpointer user)
 {
 	gchar		buf[PATH_MAX];
 	SleInfo		*sle = user;
@@ -185,7 +185,7 @@ static void evt_pick_clicked(GtkWidget *wid, gpointer user)
 /* 1999-05-30 -	Edit all selected symbolic links one at a time, or, if there is no selection,
 **		create a new link letting the user type both source and destination names.
 */
-int cmd_symlinkedit(MainInfo *min, DirPane *src, DirPane *dst, CmdArg *ca)
+gint cmd_symlinkedit(MainInfo *min, DirPane *src, DirPane *dst, const CmdArg *ca)
 {
 	static SleInfo	sle;
 	GtkWidget	*label, *table, *pick;
@@ -197,34 +197,27 @@ int cmd_symlinkedit(MainInfo *min, DirPane *src, DirPane *dst, CmdArg *ca)
 	table = gtk_table_new(2, 3, FALSE);
 	label = gtk_label_new(_("Name"));
 	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 0, 1,  0,0,0,0);
-	gtk_widget_show(label);
 	sle.name = gtk_entry_new_with_max_length(FILENAME_MAX - 1);
 	gtk_table_attach(GTK_TABLE(table), sle.name, 1, 3, 0, 1,  GTK_EXPAND|GTK_FILL,0,0,0);
-	gtk_widget_show(sle.name);
 	label = gtk_label_new(_("Contents"));
 	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 1, 2,  0,0,0,0);
-	gtk_widget_show(label);
 	sle.contents = gtk_entry_new_with_max_length(PATH_MAX - 1);
 	gtk_table_attach(GTK_TABLE(table), sle.contents, 1, 2, 1, 2,  GTK_EXPAND|GTK_FILL,0,0,0);
-	gtk_widget_show(sle.contents);
 	pick = gui_details_button_new(min->gui->window->window);
 	gtk_signal_connect(GTK_OBJECT(pick), "clicked", GTK_SIGNAL_FUNC(evt_pick_clicked), &sle);
 	gtk_table_attach(GTK_TABLE(table), pick, 2, 3, 1, 2,  0,0,0,0);
-	gtk_widget_show(pick);
 	gtk_box_pack_start(GTK_BOX(sle.vbox), table, TRUE, TRUE, 0);
-	gtk_widget_show(table);
-	gtk_widget_show(sle.vbox);
 
 	if(dp_has_selection(src))
 	{
 		gtk_entry_set_editable(GTK_ENTRY(sle.name), FALSE);
-		return cmd_generic(min, CGF_NOALL | CGF_SRC | CGF_LINKSONLY, link_body, link_action, link_free, &sle);
+		return cmd_generic(min, _("Edit Symbolic Link"), CGF_NOALL | CGF_SRC | CGF_LINKSONLY, link_body, link_action, link_free, &sle);
 	}
 	else
 	{
 		Dialog	*dlg;
 
-		dlg = dlg_dialog_sync_new(sle.vbox, _("Create Symbolic Link"), _("OK|Cancel"));
+		dlg = dlg_dialog_sync_new(sle.vbox, _("Create Symbolic Link"), NULL);
 		gtk_widget_grab_focus(sle.name);
 		if(dlg_dialog_sync_wait(dlg) == DLG_POSITIVE)
 		{
@@ -242,7 +235,7 @@ int cmd_symlinkedit(MainInfo *min, DirPane *src, DirPane *dst, CmdArg *ca)
 			if(ores == OVW_PROCEED)
 			{
 				if(set_link(min, buf, contents))
-					dp_rescan(src);
+					dp_rescan_post_cmd(src);
 			}
 			ovw_overwrite_end(min);
 		}
