@@ -40,7 +40,7 @@ static CmdCfg		*viewtext_cmc = NULL;
 /* 1999-03-06 -	A general viewer-trampoline. Will view all currently selected files using
 **		the given viewing function.
 */
-static int view_selection(MainInfo *min, DirPane *dp, int (*viewer)(MainInfo *min, const gchar *full_name))
+static gint view_selection(MainInfo *min, DirPane *dp, gint (*viewer)(MainInfo *min, const gchar *full_name))
 {
 	GSList	*slist, *iter;
 
@@ -61,7 +61,7 @@ static int view_selection(MainInfo *min, DirPane *dp, int (*viewer)(MainInfo *mi
 /* 1998-05-19 -	Just open a window containing some text viewing widgetry.
 ** 1998-09-10 -	The creation of txi_open() above allowed this one to shrink from 50 to 5 lines, sort of.
 */
-static int view_file_text(MainInfo *min, const gchar *full_name)
+static gint view_file_text(MainInfo *min, const gchar *full_name)
 {
 	GtkWidget	*txv;
 
@@ -78,7 +78,7 @@ static int view_file_text(MainInfo *min, const gchar *full_name)
 /* ----------------------------------------------------------------------------------------- */
 
 /* 1998-12-18 -	View file in hex mode. */
-static int view_file_hex(MainInfo *min, const gchar *full_name)
+static gint view_file_hex(MainInfo *min, const gchar *full_name)
 {
 	GtkWidget	*txv;
 
@@ -100,10 +100,10 @@ static int view_file_hex(MainInfo *min, const gchar *full_name)
 **		fall-back code (in case mmap() isn't supported on the file system).
 ** 1999-05-29 -	Changed order of open() and size-check; now produces errno code if file not found.
 */
-static int file_is_text(MainInfo *min, const gchar *full_name)
+static gint file_is_text(MainInfo *min, const gchar *full_name)
 {
 	gchar	*buf;
-	gint	fd, txt = -1, here;
+	gint	fd, txt = -1;
 	gsize	size, i;
 
 	if((fd = open(full_name, O_RDONLY)) >= 0)
@@ -118,8 +118,7 @@ static int file_is_text(MainInfo *min, const gchar *full_name)
 				txt = 1;
 				for(i = 0; txt && (i < size); i++)
 				{
-					here = (int) buf[i];
-					if(!isprint(here) && !isspace(here))
+					if(!isprint((guchar) buf[i]) && !isspace((guchar) buf[i]))
 						txt = 0;
 				}
 			}
@@ -135,9 +134,9 @@ static int file_is_text(MainInfo *min, const gchar *full_name)
 **		rather than using gentoo's type system. This makes it work for all files, even
 **		those of type "Unknown".
 */
-static int view_file_text_or_hex(MainInfo *min, const gchar *full_name)
+static gint view_file_text_or_hex(MainInfo *min, const gchar *full_name)
 {
-	int	res;
+	gint	res;
 
 	if((res = file_is_text(min, full_name)) >= 0)
 	{
@@ -160,7 +159,7 @@ static int view_file_text_or_hex(MainInfo *min, const gchar *full_name)
 ** 1999-05-29 -	Now uses the command argument system to control plain/hex/auto modes, thus
 **		replacing the old ViewTextHex and ViewTextOrHex commands. Neater.
 */
-int cmd_viewtext(MainInfo *min, DirPane *src, DirPane *dst, CmdArg *ca)
+gint cmd_viewtext(MainInfo *min, DirPane *src, DirPane *dst, const CmdArg *ca)
 {
 	guint	mode;
 
