@@ -25,7 +25,7 @@
 
 /* ----------------------------------------------------------------------------------------- */
 
-#define	INP_PATTERN     "\1" "INP%02d" "\2"
+#define	INP_PATTERN     "\1" "INP%02d" "\2"	/* Shouldn't occur in regular text. Really. */
 #define	INP_PAT_LEN	(7)			/* Guaranteed length produced by above. */
 
 #define	DLG_MAX_INPUT	(16)			/* Maximum amount of individual inputs {I}s for a single command. */
@@ -51,11 +51,11 @@ typedef struct {
 
 typedef struct {
 	GtkWidget	*check;			/* Just a GtkCheckButton. */
-	gchar		*active, *inactive;	/* Choice texts. If NULL, the defauls of TRUE and FALSE are used. */
+	const gchar	*active, *inactive;	/* Choice texts. If NULL, the defauls of TRUE and FALSE are used. */
 } CICheckbox;
 
 typedef struct {
-	CIType	type;
+	CIType		type;
 	union {
 	CIString	string;
 	CIMenu		menu;
@@ -73,28 +73,29 @@ typedef struct {
 	gint		row;			/* Current row in table. */
 	gchar		*wintitle;		/* Title to set on window (example: {Iw:"gentoo"}). */
 	gint		str_count;		/* Counts number of string fields (for focus). */
-	guint		str_focused;		/* Gets set as the string is first focused. */
+	gboolean	str_focused;		/* Gets set as the string is first focused. */
 } CmdDlg;
 
+static const gchar	*the_true = "TRUE", *the_false = "FALSE";
+
 /* ----------------------------------------------------------------------------------------- */
 
 /* 1998-09-25 -	Parse a code beginning with 'f' or 'F'.
 ** 1998-10-26 -	Added the 'd' flag to get at the destination pane, rather than the source.
 ** 1999-02-24 -	Rewritten for new argument handling style. Quotes filenames now.
 */
-static void parse_code_file(MainInfo *min, GString *cstr, char *code)
+static void parse_code_file(MainInfo *min, GString *cstr, const gchar *code)
 {
 	DirPane		*dp = min->gui->cur_pane;
 	const gchar	*name;
 	gboolean	unsel = (strchr(code + 1, 'u') != NULL),
 			path  = (strchr(code + 1, 'p') != NULL),
-			dest  = (strchr(code + 1, 'd') != NULL),
-			cnt = 0;
+			dest  = (strchr(code + 1, 'd') != NULL);
+	guint		cnt = 0U;
 	GSList		*slist, *iter;
 
 	if(dest)		/* User requested the destination pane? */
 		dp = dp_mirror(min, dp);
-
 	if((slist = dp_get_selection(dp)) == NULL)
 		return;
 
@@ -117,7 +118,7 @@ static void parse_code_file(MainInfo *min, GString *cstr, char *code)
 /* 1998-09-25 -	Parse a code beginning with 'P', resulting in a path.
 ** 1999-02-24 -	Touched up to comply with the new argument handling style.
 */
-static void parse_code_path(MainInfo *min, GString *cstr, char *code)
+static void parse_code_path(MainInfo *min, GString *cstr, const gchar *code)
 {
 	const gchar	*path = NULL;
 	DirPane		*src = min->gui->cur_pane, *dst = dp_mirror(min, src);
@@ -146,9 +147,9 @@ static void parse_code_path(MainInfo *min, GString *cstr, char *code)
 /* 1998-09-25 -	Parse an environment variable lookup-code, {$...}.
 ** 1999-02-24 -	Touched up for the new argument handling style.
 */
-static void parse_code_env(MainInfo *min, GString *cstr, char *code)
+static void parse_code_env(MainInfo *min, GString *cstr, const gchar *code)
 {
-	char	*val;
+	const gchar	*val;
 
 	if((val = g_getenv(code + 1)) != NULL)
 		g_string_append(cstr, val);
@@ -158,15 +159,15 @@ static void parse_code_env(MainInfo *min, GString *cstr, char *code)
 **		program's PID.
 ** 1999-02-24 -	Upgraded for new style. Not very much to change. :)
 */
-static void parse_code_pid(MainInfo *min, GString *cstr, char *code)
+static void parse_code_pid(MainInfo *min, GString *cstr, const gchar *code)
 {
-	g_string_sprintfa(cstr, "%d", (int) getpid());
+	g_string_sprintfa(cstr, "%d", (gint) getpid());
 }
 
 /* 1998-12-23 -	Parse a {~...} code, used to look after user's home directories.
 ** 1999-02-24 -	Upgraded argument handling.
 */
-static void parse_code_home(MainInfo *min, GString *cstr, char *code)
+static void parse_code_home(MainInfo *min, GString *cstr, const gchar *code)
 {
 	const gchar	*uhome;
 
@@ -182,11 +183,10 @@ static void parse_code_home(MainInfo *min, GString *cstr, char *code)
 */
 static void init_input_dialog(CmdDlg *dlg)
 {
-	dlg->body = gtk_vbox_new(FALSE, 0);
+	dlg->body  = gtk_vbox_new(FALSE, 0);
 	dlg->table = gtk_table_new(DLG_MAX_INPUT, 2, FALSE);
 	gtk_table_set_col_spacing(GTK_TABLE(dlg->table), 0, 5);
 	gtk_box_pack_start(GTK_BOX(dlg->body), dlg->table, TRUE, TRUE, 0);
-	gtk_widget_show(dlg->table);
 }
 
 /* 1999-02-27 -	A little helper routine to pack widgets into the table forming the command input
@@ -207,7 +207,6 @@ static void pack_input_widget(CmdDlg *dlg, GtkWidget *wid, gint pos, gint paddin
 
 		gtk_table_attach(GTK_TABLE(dlg->table), wid, left, right, dlg->row, dlg->row + 1,
 				 GTK_FILL|GTK_EXPAND, 0, 0, padding);
-		gtk_widget_show(wid);
 	}
 }
 
@@ -216,14 +215,14 @@ static void pack_input_widget(CmdDlg *dlg, GtkWidget *wid, gint pos, gint paddin
 **		hack is used instead. Note how a special flag is used to avoid doing this
 **		more than once during the lifetime of the dialog.
 */
-static gint evt_input_string_visible(GtkWidget *wid, gpointer user)
+static gint evt_input_string_realize(GtkWidget *wid, gpointer user)
 {
-	guint	*flag = (guint *) user;
+	gboolean	*flag = user;
 
-	if(*flag == 0)
+	if(!*flag)
 	{
 		gtk_widget_grab_focus(wid);
-		*flag = 1;
+		*flag = TRUE;
 	}
 	return TRUE;
 }
@@ -260,12 +259,12 @@ static gint build_input_string(CmdDlg *dlg, const gchar *def, CmdInp *inp)
 /* 1998-10-07 -	User selected an option in an option menu widget (created by {Im...}).
 **		Store the index in the menu.
 */
-static gint evt_menu_activated(GtkWidget *wid, gpointer data)
+static gint evt_menu_activated(GtkWidget *wid, gpointer user)
 {
 	CmdInp	*inp;
 
 	if((inp = gtk_object_get_user_data(GTK_OBJECT(wid))) != NULL)
-		inp->inp.menu.current = GPOINTER_TO_INT(data);
+		inp->inp.menu.current = GPOINTER_TO_INT(user);
 
 	return TRUE;
 }
@@ -276,44 +275,37 @@ static gint evt_menu_activated(GtkWidget *wid, gpointer data)
 static gint build_input_menu(CmdDlg *dlg, const gchar *def, CmdInp *inp)
 {
 	GtkWidget	*obtn, *menu, *item;
-	gchar		*itext, *colon;
+	const gchar	*itext, *colon;
 	gint		index = 0;
 
 	if(*def != '=')
 		return 0;
-	if((obtn = gtk_option_menu_new()) != NULL)
+	obtn = gtk_option_menu_new();
+	menu = gtk_menu_new();
+	inp->type = CITP_MENU;
+	inp->inp.menu.items = NULL;
+	inp->inp.menu.current = 0;
+	while((def = stu_scan_string(def, &itext)) != NULL)
 	{
-		if((menu = gtk_menu_new()) != NULL)
+		if((colon = strchr(itext, ':')) != NULL)
 		{
-			inp->type = CITP_MENU;
-			inp->inp.menu.items = NULL;
-			inp->inp.menu.current = 0;
-			while((def = stu_scan_string(def, &itext)) != NULL)
-			{
-				if((colon = strchr(itext, ':')) != NULL)
-				{
-					gchar	tmp[2048];
+			gchar	tmp[2048];
 
-					stu_strncpy(tmp, itext, sizeof tmp);
-					tmp[colon - itext] = '\0';
-					item = gtk_menu_item_new_with_label(tmp);
-				}
-				else
-					item = gtk_menu_item_new_with_label(itext);
-
-				gtk_object_set_user_data(GTK_OBJECT(item), (gpointer) inp);
-				inp->inp.menu.items = g_list_append(inp->inp.menu.items, (gpointer) itext);
-				gtk_signal_connect(GTK_OBJECT(item), "activate", GTK_SIGNAL_FUNC(evt_menu_activated), GINT_TO_POINTER(index++));
-				gtk_menu_append(GTK_MENU(menu), item);
-				gtk_widget_show(item);
-			}
-			gtk_option_menu_set_menu(GTK_OPTION_MENU(obtn), menu);
-			pack_input_widget(dlg, obtn, 1, 0);
-			return 1;
+			stu_strncpy(tmp, itext, sizeof tmp);
+			tmp[colon - itext] = '\0';
+			item = gtk_menu_item_new_with_label(tmp);
 		}
-		gtk_widget_destroy(obtn);
+		else
+			item = gtk_menu_item_new_with_label(itext);
+
+		gtk_object_set_user_data(GTK_OBJECT(item), inp);
+		inp->inp.menu.items = g_list_append(inp->inp.menu.items, (gpointer) itext);
+		gtk_signal_connect(GTK_OBJECT(item), "activate", GTK_SIGNAL_FUNC(evt_menu_activated), GINT_TO_POINTER(index++));
+		gtk_menu_append(GTK_MENU(menu), item);
 	}
-	return 0;
+	gtk_option_menu_set_menu(GTK_OPTION_MENU(obtn), menu);
+	pack_input_widget(dlg, obtn, 1, 0);
+	return 1;
 }
 
 /* 1998-11-29 -	Build an input combo box widget, defined by a {Ic...} code.
@@ -322,31 +314,29 @@ static gint build_input_menu(CmdDlg *dlg, const gchar *def, CmdInp *inp)
 static gint build_input_combo(CmdDlg *dlg, const gchar *def, CmdInp *inp)
 {
 	GtkWidget	*combo;
-	gchar		*itext;
+	const gchar	*itext;
 	GList		*item = NULL, *iter;
 
 	if(*def != '=')
 		return 0;
 
-	if((combo = gtk_combo_new()) != NULL)
+	combo = gtk_combo_new();
+	/* Configure the combo to work the way I like it to. */
+	gtk_combo_set_value_in_list(GTK_COMBO(combo), FALSE, TRUE);
+	gtk_combo_set_use_arrows_always(GTK_COMBO(combo), TRUE);
+	while((def = stu_scan_string(def, &itext)) != NULL)
+		item = g_list_append(item, (gpointer) itext);
+	if(item != NULL)
 	{
-		/* Configure the combo to work the way I like it to. */
-		gtk_combo_set_value_in_list(GTK_COMBO(combo), FALSE, TRUE);
-		gtk_combo_set_use_arrows_always(GTK_COMBO(combo), TRUE);
-		while((def = stu_scan_string(def, &itext)) != NULL)
-			item = g_list_append(item, (gpointer) itext);
-		if(item != NULL)
-		{
-			inp->type = CITP_COMBO;
-			gtk_combo_set_popdown_strings(GTK_COMBO(combo), item);
-			for(iter = item; iter != NULL; iter = g_list_next(iter))
-				g_free(iter->data);
-			g_list_free(item);
-			pack_input_widget(dlg, inp->inp.combo.combo = combo, 1, 0);
-			return 1;
-		}
-		gtk_widget_destroy(combo);
+		inp->type = CITP_COMBO;
+		gtk_combo_set_popdown_strings(GTK_COMBO(combo), item);
+		for(iter = item; iter != NULL; iter = g_list_next(iter))
+			g_free(iter->data);
+		g_list_free(item);
+		pack_input_widget(dlg, inp->inp.combo.combo = combo, 1, 0);
+		return 1;
 	}
+	gtk_widget_destroy(combo);
 	return 0;
 }
 
@@ -384,8 +374,20 @@ static gint build_input_checkbox(CmdDlg *dlg, const gchar *label, const gchar *d
 	{
 		inp->type = CITP_CHECKBOX;
 		inp->inp.checkbox.check = gtk_check_button_new_with_label(label);
-		inp->inp.checkbox.active = "TRUE";
-		inp->inp.checkbox.inactive = "FALSE";
+		inp->inp.checkbox.active   = the_true;
+		inp->inp.checkbox.inactive = the_false;
+
+		if(*def == '*')	/* Pre-checked mode? */
+		{
+			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(inp->inp.checkbox.check), TRUE);
+			def++;
+		}
+		if(*def == '=')	/* Non-default result strings requested? */
+		{
+			def = stu_scan_string(++def, (const gchar **) &inp->inp.checkbox.active);
+			if(*def == ',')
+				stu_scan_string(++def, (const gchar **) &inp->inp.checkbox.inactive);
+		}
 		pack_input_widget(dlg, inp->inp.checkbox.check, 1, 0);
 	}
 	return def != NULL;
@@ -426,7 +428,7 @@ static void parse_code_input(MainInfo *min, GString *cstr, gchar *code, CmdDlg *
 		case 's':
 			succ = build_input_string(dlg, def, &dlg->input[dlg->row]);
 			if(dlg->str_count++ == 0)
-				gtk_signal_connect(GTK_OBJECT(dlg->input[dlg->row].inp.string.entry), "realize", GTK_SIGNAL_FUNC(evt_input_string_visible), &dlg->str_focused);
+				gtk_signal_connect(GTK_OBJECT(dlg->input[dlg->row].inp.string.entry), "realize", GTK_SIGNAL_FUNC(evt_input_string_realize), &dlg->str_focused);
 			break;
 		case 'm':
 			succ = build_input_menu(dlg, def, &dlg->input[dlg->row]);
@@ -457,7 +459,6 @@ static void parse_code_input(MainInfo *min, GString *cstr, gchar *code, CmdDlg *
 			{
 				label = gtk_label_new(ltxt);
 				gtk_table_attach(GTK_TABLE(dlg->table), label, 0, 1, dlg->row, dlg->row + 1, GTK_FILL,0,0,0);
-				gtk_widget_show(label);
 			}
 			g_string_sprintfa(cstr, INP_PATTERN, dlg->row++);
 		}
@@ -467,7 +468,7 @@ static void parse_code_input(MainInfo *min, GString *cstr, gchar *code, CmdDlg *
 }
 
 /* 1998-09-25 -	Get a pointer to a string representation of row <row> in dialog <dlg>. */
-static gchar * eval_input(CmdDlg *dlg, gint row)
+static const gchar * eval_input(CmdDlg *dlg, gint row)
 {
 	GList	*data;
 
@@ -482,9 +483,9 @@ static gchar * eval_input(CmdDlg *dlg, gint row)
 				{
 					gchar	*colon;
 
-					if((colon = strchr((char *) data->data, ':')) != NULL)
+					if((colon = strchr(data->data, ':')) != NULL)
 						return colon + 1;
-					return (gchar *) data->data;
+					return data->data;
 				}
 				return NULL;
 			case CITP_COMBO:
@@ -548,9 +549,13 @@ static void destroy_input(CmdDlg *dlg)
 				break;
 			case CITP_COMBO:
 				break;
-			case CITP_LABEL:		/* Just to keep compiler happy. */
+			case CITP_LABEL:
 				break;
 			case CITP_CHECKBOX:
+				if(dlg->input[i].inp.checkbox.active != the_true)
+					g_free((gpointer) dlg->input[i].inp.checkbox.active);
+				if(dlg->input[i].inp.checkbox.inactive != the_false)
+					g_free((gpointer) dlg->input[i].inp.checkbox.inactive);
 				break;
 		}
 	}
@@ -691,14 +696,14 @@ gchar ** cpr_parse(MainInfo *min, const gchar *def)
 	dlg.row = 0;
 	dlg.wintitle = NULL;
 	dlg.str_count = 0;
-	dlg.str_focused = 0;
+	dlg.str_focused = FALSE;
 
 	cstr = interpolate_brace_codes(min, def, &dlg);
 
 	if(dlg.body != NULL)	/* Do we need an input window? */
 	{
-		dlg.dlg = dlg_dialog_sync_new(dlg.body, dlg.wintitle, _("OK|Cancel"));
-		if(dlg_dialog_sync_wait(dlg.dlg) == DLG_POSITIVE)		/* OK? */
+		dlg.dlg = dlg_dialog_sync_new(dlg.body, dlg.wintitle, NULL);
+		if(dlg_dialog_sync_wait(dlg.dlg) == DLG_POSITIVE)	/* OK? */
 			interpolate_input(cstr, &dlg);
 		else
 		{
@@ -708,7 +713,7 @@ gchar ** cpr_parse(MainInfo *min, const gchar *def)
 		destroy_input(&dlg);
 		dlg_dialog_sync_destroy(dlg.dlg);
 	}
-	if(cstr != NULL)			/* Did we get any arguments? */
+	if(cstr != NULL)	/* Did we get any arguments? */
 	{
 		gchar	**argv;
 
