@@ -10,7 +10,6 @@
 
 #include "gentoo.h"
 
-#include <assert.h>
 #include <ctype.h>
 #include <stdlib.h>
 #include <signal.h>
@@ -23,7 +22,6 @@
 #include "dirpane.h"
 #include "dialog.h"
 #include "fileutil.h"
-#include "cfg_gui.h"		/* For the cfg_gui() function ("Configure" built-in). */
 
 #include "children.h"
 #include "cmdparse.h"
@@ -44,19 +42,19 @@ static GMemChunk	*builtin_chunk = NULL;
 */
 typedef struct {
 	Command		cmd;
-	CommandCfg	cfg;
+	CommandCfg	cfg;	/* Another function pointer. */
 } CmdDesc;
 
 /* ----------------------------------------------------------------------------------------- */
 
-void csq_add_builtin(MainInfo *min, const gchar *name, Command cmd, CommandCfg cmd_cfg)
+static void csq_add_builtin(MainInfo *min, const gchar *name, Command cmd, CommandCfg cmd_cfg)
 {
 	CfgInfo	*cfg = &min->cfg;
 
 	if(cfg->commands.builtin == NULL)
 		cfg->commands.builtin = g_hash_table_new(g_str_hash, g_str_equal);
 	if(builtin_chunk == NULL)
-		builtin_chunk = g_mem_chunk_new("CmdDesc", sizeof (CmdDesc), 64, G_ALLOC_ONLY);
+		builtin_chunk = g_mem_chunk_new("CmdDesc", sizeof (CmdDesc), 32, G_ALLOC_ONLY);
 	if((cfg->commands.builtin != NULL) && (builtin_chunk != NULL))
 	{
 		CmdDesc	*cdesc;
@@ -79,7 +77,7 @@ void csq_add_builtin(MainInfo *min, const gchar *name, Command cmd, CommandCfg c
 **		These operations include stripping out whitespace and almost all other
 **		non-alphanumerical letters.
 */
-void csq_cmdseq_filter_name(gchar *dst, const gchar *src)
+static void csq_cmdseq_filter_name(gchar *dst, const gchar *src)
 {
 	gchar	*od = dst;
 
@@ -87,7 +85,7 @@ void csq_cmdseq_filter_name(gchar *dst, const gchar *src)
 	{
 		for(; *src != '\0' && dst - od < CSQ_NAME_SIZE - 1; src++)
 		{
-			if(isalnum((unsigned char) *src) || *src == '_')
+			if(isalnum((guchar) *src) || *src == '_')
 				*dst++ = *src;
 		}
 		*dst = '\0';
@@ -99,11 +97,11 @@ void csq_cmdseq_filter_name(gchar *dst, const gchar *src)
 /* 1998-09-27 -	A callback for g_hash_table; find maximum X in all UnnamedX-keys. OK? */
 static void check_name(gpointer key, gpointer data, gpointer user)
 {
-	gint	here, *iptr = (gint *) user;
+	gint	here, *iptr = user;
 
-	if((strcmp((char *) key, "Unnamed") == 0) && iptr != NULL && *iptr == -1)
+	if((strcmp(key, "Unnamed") == 0) && iptr != NULL && *iptr == -1)
 		*iptr = 0;
-	else if(sscanf((char *) key, "Unnamed_%d", &here) == 1)
+	else if(sscanf(key, "Unnamed_%d", &here) == 1)
 	{
 		if(iptr != NULL && here > *iptr)
 			*iptr = here;
@@ -149,7 +147,7 @@ static const gchar * map_name(const gchar *from, const gchar *to, const gchar *c
 const gchar * csq_cmdseq_map_name(const gchar *name, const gchar *context)
 {
 	/* Ignore NULL and names beginning with a lower case letter (user-defined). */
-	if((name == NULL) || (islower((unsigned char) *name)))
+	if((name == NULL) || (islower((guchar) *name)))
 		return name;
 
 	if(strcmp(name, "FileDefault") == 0)
@@ -191,7 +189,7 @@ CmdSeq * csq_cmdseq_new(const gchar *name, guint32 flags)
 }
 
 /* 1998-09-26 -	Create a "deep" (non-memory-sharing) copy of <src>, and return it. */
-CmdSeq * csq_cmdseq_copy(CmdSeq *src)
+CmdSeq * csq_cmdseq_copy(const CmdSeq *src)
 {
 	CmdSeq	*dst = NULL;
 	CmdRow	*row;
@@ -204,7 +202,7 @@ CmdSeq * csq_cmdseq_copy(CmdSeq *src)
 		dst->rows = NULL;
 		for(iter = src->rows; iter != NULL; iter = g_list_next(iter))
 		{
-			if((row = csq_cmdrow_copy((CmdRow *) iter->data)) != NULL)
+			if((row = csq_cmdrow_copy(iter->data)) != NULL)
 				dst->rows = g_list_append(dst->rows, row);
 			else
 			{
@@ -220,13 +218,13 @@ CmdSeq * csq_cmdseq_copy(CmdSeq *src)
 **		is somewhat more complex than it might seem, since it needs to rehash
 **		the sequence.
 */
-void csq_cmdseq_set_name(GHashTable *hash, CmdSeq *cs, char *name)
+void csq_cmdseq_set_name(GHashTable *hash, CmdSeq *cs, const gchar *name)
 {
 	if(hash != NULL && cs != NULL && name != NULL)
 	{
-		g_hash_table_remove(hash, (gpointer) cs->name);
+		g_hash_table_remove(hash, cs->name);
 		csq_cmdseq_filter_name(cs->name, name);
-		if(cs->name[0] == '\0' || g_hash_table_lookup(hash, (gpointer) cs->name) != NULL)
+		if(cs->name[0] == '\0' || g_hash_table_lookup(hash, cs->name) != NULL)
 			stu_strncpy(cs->name, csq_cmdseq_unique_name(hash), sizeof cs->name);
 		csq_cmdseq_hash(&hash, cs);
 	}
@@ -242,7 +240,7 @@ void csq_cmdseq_hash(GHashTable **hash, CmdSeq *cs)
 		if(*hash == NULL)
 			*hash = g_hash_table_new(g_str_hash, g_str_equal);
 		if(*hash != NULL)
-			g_hash_table_insert(*hash, cs->name, (gpointer) cs);
+			g_hash_table_insert(*hash, cs->name, cs);
 	}
 }
 
@@ -271,7 +269,7 @@ gint csq_cmdseq_row_append(CmdSeq *cs, CmdRow *row)
 {
 	if(cs != NULL && row != NULL)
 	{
-		cs->rows = g_list_append(cs->rows, (gpointer) row);
+		cs->rows = g_list_append(cs->rows, row);
 		return g_list_length(cs->rows) - 1;
 	}
 	return -1;
@@ -289,15 +287,15 @@ gint csq_cmdseq_row_move(CmdSeq *cs, CmdRow *row, gint delta)
 
 	if(cs != NULL && row != NULL && (delta == -1 || delta == 1))
 	{
-		link = g_list_find(cs->rows, (gpointer) row);
+		link = g_list_find(cs->rows, row);
 		pos  = g_list_position(cs->rows, link);
-		cs->rows = g_list_remove(cs->rows, (gpointer) row);
+		cs->rows = g_list_remove(cs->rows, row);
 		np = pos + delta;
 		if(np < 0)
 			np = g_list_length(cs->rows);
 		else if(np > (gint) g_list_length(cs->rows))
 			np = 0;
-		cs->rows = g_list_insert(cs->rows, (gpointer) row, np);
+		cs->rows = g_list_insert(cs->rows, row, np);
 		return np;
 	}
 	return -1;
@@ -313,8 +311,8 @@ gint csq_cmdseq_row_delete(CmdSeq *cs, CmdRow *row)
 
 	if(cs != NULL && row != NULL)
 	{
-		pos = g_list_index(cs->rows, (gpointer) row);
-		cs->rows = g_list_remove(cs->rows, (gpointer) row);
+		pos = g_list_index(cs->rows, row);
+		cs->rows = g_list_remove(cs->rows, row);
 		if(pos >= (gint) g_list_length(cs->rows))
 			return pos - 1;
 		return pos;
@@ -327,7 +325,7 @@ gint csq_cmdseq_row_delete(CmdSeq *cs, CmdRow *row)
 /* 1998-09-25 -	Execute a built-in command. Currently very simple.
 ** 1999-05-06 -	Rewritten, now handles command arguments. Very nice.
 */
-static int execute_builtin(MainInfo *min, const char *cmd)
+static gint execute_builtin(MainInfo *min, const gchar *cmd)
 {
 	gchar	msg[CSQ_NAME_SIZE + 64];
 	gchar	*spc, **argv;
@@ -337,32 +335,32 @@ static int execute_builtin(MainInfo *min, const char *cmd)
 	{
 		if((argv = cpr_parse(min, cmd)) != NULL)
 		{
-			if((cdesc = (CmdDesc *) g_hash_table_lookup(min->cfg.commands.builtin, argv[0])) != NULL)
+			if((cdesc = g_hash_table_lookup(min->cfg.commands.builtin, argv[0])) != NULL)
 			{
 				CmdArg	*ca;
 				DirPane	*src = min->gui->cur_pane;
 				gint	ret;
 
-				ca = car_create(argv);
+				ca  = car_create(argv);
 				ret = cdesc->cmd(min, src, dp_mirror(min, src), ca);
 				car_destroy(ca);
 				return ret;
 			}
-			g_snprintf(msg, sizeof msg, "Unable to execute unknown\ncommand \"%s\".", cmd);
+			g_snprintf(msg, sizeof msg, _("Unable to execute unknown\ncommand \"%s\"."), cmd);
 			dlg_dialog_async_new_error(msg);
 		}
 		return 1;
 	}
 	else
 	{
-		if((cdesc = (CmdDesc *) g_hash_table_lookup(min->cfg.commands.builtin, cmd)) != NULL)
+		if((cdesc = g_hash_table_lookup(min->cfg.commands.builtin, cmd)) != NULL)
 		{
 			DirPane	*src = min->gui->cur_pane;
 
 			return cdesc->cmd(min, src, dp_mirror(min, src), NULL);
 		}
 	}
-	g_snprintf(msg, sizeof msg, "Unable to execute unknown\ncommand \"%s\".", cmd);
+	g_snprintf(msg, sizeof msg, _("Unable to execute unknown\ncommand \"%s\"."), cmd);
 	dlg_dialog_async_new_error(msg);
 
 	return 0;
@@ -375,14 +373,15 @@ void csq_handle_ba_flags(MainInfo *min, guint32 flags)
 {
 	DirPane	*src = min->gui->cur_pane, *dst = dp_mirror(min, src);
 
-	if(flags & CBAF_RESCAN_SOURCE)
-		dp_rescan(src);
-	if(flags & CBAF_RESCAN_DEST)
-		dp_rescan(dst);
 	if(flags & CBAF_CD_SOURCE)
 		fut_cd(src->dir.path, NULL, 0);
 	else if(flags & CBAF_CD_DEST)
 		fut_cd(dst->dir.path, NULL, 0);
+
+	if(flags & CBAF_RESCAN_SOURCE)
+		dp_rescan_post_cmd(src);
+	if(flags & CBAF_RESCAN_DEST)
+		dp_rescan_post_cmd(dst);
 }
 
 /* ----------------------------------------------------------------------------------------- */
@@ -406,16 +405,15 @@ void csq_handle_ba_flags(MainInfo *min, guint32 flags)
 ** 1998-09-25 -	Moved into the new cmdseq module, and adapted accordingly.
 ** 1999-05-29 -	Added pre-fork() protection through new call in fileutil module. Very nice.
 */
-static int fork_and_execute(MainInfo *min, CmdRow *row, gchar **argv)
+static gint fork_and_execute(MainInfo *min, CmdRow *row, gchar **argv)
 {
 	pid_t	child;
-	int	estat, fd_out[2], fd_err[2];
+	gint	estat, fd_out[2], fd_err[2];
 	CX_Ext	*ext = &row->extra.external;
 
 	if(!fut_locate_executable(g_getenv("PATH"), argv[0]))
 	{
 		err_set(min, errno, "Execute", argv[0]);
-		err_show(min);
 		return 0;
 	}
 
@@ -431,8 +429,10 @@ static int fork_and_execute(MainInfo *min, CmdRow *row, gchar **argv)
 	{
 		if(child == 0)		/* In child process? */
 		{
-			cgr_set_child_pipes(min, row, fd_out, fd_err);
-			estat = execvp(argv[0], argv);
+			if(cgr_set_child_pipes(min, row, fd_out, fd_err))
+			{
+				estat = execvp(argv[0], argv);
+			}
 			usleep(50000);				/* Someone, sue me. */
 			_exit(CHD_EXIT_FAILURE);
 		}
@@ -448,7 +448,7 @@ static int fork_and_execute(MainInfo *min, CmdRow *row, gchar **argv)
 }
 
 /* 1998-09-25 -	Execute an external command. */
-static int execute_external(MainInfo *min, CmdRow *row)
+static gint execute_external(MainInfo *min, CmdRow *row)
 {
 	gchar	**argv;
 
@@ -461,7 +461,7 @@ static int execute_external(MainInfo *min, CmdRow *row)
 }
 
 /* 1998-09-25 -	Execute a single row, returning success or failure. */
-static int execute_row(MainInfo *min, CmdRow *row)
+static gint execute_row(MainInfo *min, CmdRow *row)
 {
 	switch(row->type)
 	{
@@ -480,7 +480,7 @@ static int execute_row(MainInfo *min, CmdRow *row)
 /* 1998-09-25 -	Figure out whether running <row> results in a block (i.e. no further rows are
 **		executed until <row> has completed, or not.
 */
-static int row_is_blocking(MainInfo *min, CmdRow *row)
+static gint row_is_blocking(MainInfo *min, CmdRow *row)
 {
 	switch(row->type)
 	{
@@ -497,11 +497,10 @@ static int row_is_blocking(MainInfo *min, CmdRow *row)
 /* 1998-09-25 -	Execute a command sequence. This, of course, is sort of the most important
 **		functionality in this module, the core that makes the rest worth having.
 */
-static int cmdseq_execute(MainInfo *min, CmdSeq *cs)
+static gint cmdseq_execute(MainInfo *min, CmdSeq *cs)
 {
 	GList	*iter;
-	CmdRow	*row;
-	int	ok = 0, block, i;
+	gint	ok = 0, block, i;
 
 	if(cs == NULL)
 		return 0;
@@ -509,9 +508,8 @@ static int cmdseq_execute(MainInfo *min, CmdSeq *cs)
 	err_clear(min);
 	for(iter = cs->rows, i = 0; iter != NULL; iter = g_list_next(iter), i++)
 	{
-		row = (CmdRow *) iter->data;
-		block = row_is_blocking(min, row);
-		if((ok = execute_row(min, row)) != 0)
+		block = row_is_blocking(min, iter->data);
+		if((ok = execute_row(min, iter->data)) != 0)
 		{
 			if(block)
 			{
@@ -574,7 +572,7 @@ void csq_continue(MainInfo *min)
 		for(iter = g_list_nth(cs->rows, index); iter != NULL; iter = g_list_next(iter))
 		{
 			chd_set_running(cs, ++index);
-			row = (CmdRow *) iter->data;
+			row = iter->data;
 			if(row_is_blocking(min, row))
 			{
 				if(execute_row(min, row))
@@ -645,7 +643,7 @@ CmdRow * csq_cmdrow_new(CRType type, const gchar *def, guint32 flags)
 /* 1998-09-26 -	Create and return a verbatim copy of <src>. The copy is "deep"; i.e. there is
 **		zero sharing of memory between the original and the copy.
 */
-CmdRow * csq_cmdrow_copy(CmdRow *src)
+CmdRow * csq_cmdrow_copy(const CmdRow *src)
 {
 	CmdRow	*dst;
 
@@ -653,7 +651,7 @@ CmdRow * csq_cmdrow_copy(CmdRow *src)
 	{
 		dst->type  = src->type;
 		dst->flags = src->flags;
-		memcpy(&dst->extra, &src->extra, sizeof dst->extra);
+		dst->extra = src->extra;
 		if((dst->def = g_string_new(src->def->str)) != NULL)
 			return dst;
 		g_free(dst);
@@ -700,16 +698,15 @@ void csq_init_commands(MainInfo *min)
 	min->cfg.commands.builtin = NULL;
 
 	csq_add_builtin(min, "DirEnter",	cmd_direnter,		NULL);
-/*	csq_add_builtin(min, "DirForward",	cmd_dirforward,		NULL);*/
 	csq_add_builtin(min, "DirFromOther", 	cmd_fromother,		NULL);
 	csq_add_builtin(min, "DirToOther", 	cmd_toother,		NULL);
 	csq_add_builtin(min, "DirParent",	cmd_parent,		NULL);
-/*	csq_add_builtin(min, "DirBackward",	cmd_dirbackward,	NULL);*/
-	csq_add_builtin(min, "DirRescan",	cmd_rescan,		NULL);
+	csq_add_builtin(min, "DirRescan",	cmd_dirrescan,		NULL);
 	csq_add_builtin(min, "DirSwap",		cmd_swap,		NULL);
 
 	csq_add_builtin(min, "DpHide",		cmd_dphide,		NULL);
 	csq_add_builtin(min, "DpRecenter",	cmd_dprecenter,		NULL);
+	csq_add_builtin(min, "DpReorient",	cmd_dpreorient,		NULL);
 	csq_add_builtin(min, "DpFocus",		cmd_dpfocus,		cfg_dpfocus);
 	csq_add_builtin(min, "DpGotoRow",	cmd_dpgotorow,		NULL);
 	csq_add_builtin(min, "DpFocusPath",	cmd_dpfocuspath,	NULL);
