@@ -8,6 +8,7 @@
 #include "gentoo.h"
 
 #include <stdio.h>
+#include <string.h>
 
 #include "odemilbutton.h"
 
@@ -102,8 +103,8 @@ void cmc_config_base_save(CmdCfg *cmc, FILE *out)
 {
 	if((cmc != NULL) && (out != NULL))
 	{
-		GList	*iter;
-		CField	*fld;
+		const GList	*iter;
+		CField		*fld;
 
 		xml_put_node_open(out, cmc->name);
 		for(iter = cmc->fields; iter != NULL; iter = g_list_next(iter))
@@ -140,9 +141,8 @@ void cmc_config_base_load(CmdCfg *cmc, const XmlNode *node)
 {
 	if((cmc != NULL) && (node != NULL))
 	{
-		GList	*iter;
-		CField	*fld;
-		gint	ok = 0;
+		const GList	*iter;
+		CField		*fld;
 
 		for(iter = cmc->fields; iter != NULL; iter = g_list_next(iter))
 		{
@@ -153,19 +153,19 @@ void cmc_config_base_load(CmdCfg *cmc, const XmlNode *node)
 			switch(fld->type)
 			{
 				case CFT_INTEGER:
-					ok = xml_get_integer(node, fld->name, (gint *) ((gchar *) cmc->base + fld->offset));
+					xml_get_integer(node, fld->name, (gint *) ((gchar *) cmc->base + fld->offset));
 					break;
 				case CFT_BOOLEAN:
-					ok = xml_get_boolean(node, fld->name, (gboolean *) ((gchar *) cmc->base + fld->offset));
+					xml_get_boolean(node, fld->name, (gboolean *) ((gchar *) cmc->base + fld->offset));
 					break;
 				case CFT_SIZE:
-					ok = xml_get_uinteger(node, fld->name, (gsize *) ((gchar *) cmc->base + fld->offset));
+					xml_get_uinteger(node, fld->name, (gsize *) ((gchar *) cmc->base + fld->offset));
 					break;
 				case CFT_STRING:
-					ok = xml_get_text_copy(node, fld->name, (gchar *) cmc->base + fld->offset, fld->field.string.size);
+					xml_get_text_copy(node, fld->name, (gchar *) cmc->base + fld->offset, fld->field.string.size);
 					break;
 				case CFT_COLOR:
-					ok = xml_get_color(node, fld->name, (GdkColor *) ((gchar *) cmc->base + fld->offset));
+					xml_get_color(node, fld->name, (GdkColor *) ((gchar *) cmc->base + fld->offset));
 					break;
 			}
 		}
@@ -188,7 +188,7 @@ void cmc_config_register(CmdCfg *cmc)
 /* 1999-04-05 -	Call a user-defined function for each registered command config descriptor. */
 void cmc_config_registered_foreach(void (*func)(CmdCfg *cmc, gpointer user), gpointer user)
 {
-	GList	*iter;
+	const GList	*iter;
 
 	for(iter = cmdcfg_list; iter != NULL; iter = g_list_next(iter))
 		func(iter->data, user);
@@ -210,7 +210,7 @@ void cmc_config_unregister(CmdCfg *cmc)
 		if((link = g_list_find(cmdcfg_list, cmc)) != NULL)
 		{
 			cmdcfg_list = g_list_remove_link(cmdcfg_list, link);
-			g_list_free(link);
+			g_list_free_1(link);
 		}
 	}
 }
@@ -276,8 +276,8 @@ static void field_add(CmdCfg *cmc, CField *fld)
 {
 	if((cmc != NULL) && (fld != NULL))
 	{
-		if((cmc->fields == NULL) && strcmp(fld->name, "modified"))
-			fprintf(stderr, "CMDCFG: First field should be boolean named 'modified' (%s)\n", cmc->name);
+		if((cmc->fields == NULL) && strcmp(fld->name, "modified") != 0)
+			g_error("CMDCFG: First field should be boolean named 'modified' (%s)", cmc->name);
 
 		cmc->fields = g_list_insert_sorted(cmc->fields, fld, cmp_field);
 	}
@@ -339,7 +339,7 @@ static CField * field_find(CmdCfg *cmc, const gchar *name)
 {
 	if((cmc != NULL) && (name != NULL))
 	{
-		GList	*iter;
+		const GList	*iter;
 
 		for(iter = cmc->fields; iter != NULL; iter = g_list_next(iter))
 		{
@@ -452,9 +452,9 @@ static gint evt_size_changed(GtkObject *obj, gpointer user)
 
 		value *= fld->field.size.step;
 		if(fld->field.size.step >= 512)
-			sze_put_size(value, buf, sizeof buf, SZE_KB);
+			sze_put_size(buf, sizeof buf, value, SZE_KB);
 		else
-			sze_put_size(value, buf, sizeof buf, SZE_BYTES);
+			sze_put_size(buf, sizeof buf, value, SZE_BYTES);
 		gtk_label_set_text(GTK_LABEL(label), buf);
 
 		*(gsize *) ((gchar *) instance + fld->offset) = value;
@@ -495,6 +495,17 @@ static void evt_color_clicked(GtkWidget *wid, gpointer wuser, guint index, gpoin
 	}
 }
 
+/* 2002-08-01 -	Build a neatly right-aligned label, and stuff it into <table>. */
+static void label_build(const gchar *text, GtkWidget *table, gint x, gint y)
+{
+	GtkWidget	*algn, *label;
+
+	algn  = gtk_alignment_new(0.9f, 0.0f, 0.0f, 0.0f);
+	label = gtk_label_new(text);
+	gtk_container_add(GTK_CONTAINER(algn), label);
+	gtk_table_attach(GTK_TABLE(table), algn, x, x + 1, y, y +1,  GTK_FILL,0,0,0);
+}
+
 /* 1999-04-05 -	Build config widgetry for given <fld>. This function is a bit schizofrenic, since it
 **		does one of two things: if <table> is non-NULL, the widgetry is packed into it, which
 **		looks neat. If it is NULL, a new table is created (height==1), packed, and returned.
@@ -503,7 +514,7 @@ static GtkWidget * field_build(CmdCfg *cmc, CField *fld, gpointer instance, GtkW
 {
 	if((cmc != NULL) && (fld != NULL) && (instance != NULL))
 	{
-		GtkWidget	*label, *wid, *hbox;
+		GtkWidget	*wid, *hbox, *label;
 		GtkAdjustment	*adj;
 		GtkObject	*obj;
 
@@ -512,20 +523,16 @@ static GtkWidget * field_build(CmdCfg *cmc, CField *fld, gpointer instance, GtkW
 			table = gtk_table_new(1, 2, FALSE);
 			row = 0;
 		}
-
 		switch(fld->type)
 		{
 			case CFT_INTEGER:
-				label = gtk_label_new(fld->desc);
-				gtk_table_attach(GTK_TABLE(table), label, 0, 1, row, row + 1,  0,0,0,0);
-				gtk_widget_show(label);
+				label_build(fld->desc, table, 0, row);
 				wid = gtk_spin_button_new(NULL, 1, 0);
 				set_object_data(GTK_OBJECT(wid), fld, instance);
 				adj = gtk_spin_button_get_adjustment(GTK_SPIN_BUTTON(wid));
 				gtk_adjustment_clamp_page(adj, fld->field.integer.min, fld->field.integer.max);
 				gtk_adjustment_set_value(adj, *(gint *) ((gchar *) instance + fld->offset));
 				gtk_table_attach(GTK_TABLE(table), wid, 1, 2, row, row + 1,  GTK_EXPAND|GTK_FILL,0,5,0);
-				gtk_widget_show(wid);
 				break;
 			case CFT_BOOLEAN:
 				wid = gtk_check_button_new_with_label(fld->desc);
@@ -533,12 +540,9 @@ static GtkWidget * field_build(CmdCfg *cmc, CField *fld, gpointer instance, GtkW
 				gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(wid), *(gboolean *) ((gchar *) instance + fld->offset));
 				gtk_signal_connect(GTK_OBJECT(wid), "clicked", GTK_SIGNAL_FUNC(evt_boolean_clicked), cmc);
 				gtk_table_attach(GTK_TABLE(table), wid, 0, 2, row, row + 1,  GTK_EXPAND|GTK_FILL,0,5,0);
-				gtk_widget_show(wid);
 				break;
 			case CFT_SIZE:
-				label = gtk_label_new(fld->desc);
-				gtk_table_attach(GTK_TABLE(table), label, 0, 1, row, row + 1,  GTK_SHRINK,0,0,0);
-				gtk_widget_show(label);
+				label_build(fld->desc, table, 0, row);
 				hbox = gtk_hbox_new(FALSE, 0);
 				obj = gtk_adjustment_new((gfloat) *(gint *) ((gchar *) instance + fld->offset) / fld->field.size.step,
 							 fld->field.size.min / fld->field.size.step,
@@ -550,37 +554,28 @@ static GtkWidget * field_build(CmdCfg *cmc, CField *fld, gpointer instance, GtkW
 				gtk_range_set_update_policy(GTK_RANGE(wid), GTK_UPDATE_DELAYED);
 				gtk_scale_set_draw_value(GTK_SCALE(wid), FALSE);
 				gtk_box_pack_start(GTK_BOX(hbox), wid, TRUE, TRUE, 0);
-				gtk_widget_show(wid);
 				label = gtk_label_new("");
 				gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 5);
-				gtk_widget_show(label);
 				gtk_object_set_data(obj, "label", label);
 				gtk_signal_connect(obj, "value_changed", GTK_SIGNAL_FUNC(evt_size_changed), cmc);
 				gtk_adjustment_value_changed(GTK_ADJUSTMENT(obj));
 				gtk_table_attach(GTK_TABLE(table), hbox, 1, 2, row, row + 1,  GTK_EXPAND|GTK_FILL,0,5,0);
-				gtk_widget_show(hbox);
 				break;
 			case CFT_STRING:
-				label = gtk_label_new(fld->desc);
-				gtk_table_attach(GTK_TABLE(table), label, 0, 1, row, row + 1,  0,0,0,0);
-				gtk_widget_show(label);
+				label_build(fld->desc, table, 0, row);
 				wid = gtk_entry_new_with_max_length(fld->field.string.size - 1);
 				set_object_data(GTK_OBJECT(wid), fld, instance);
 				gtk_entry_set_text(GTK_ENTRY(wid), (gchar *) instance + fld->offset);
 				gtk_signal_connect(GTK_OBJECT(wid), "changed", GTK_SIGNAL_FUNC(evt_string_changed), cmc);
 				gtk_table_attach(GTK_TABLE(table), wid, 1, 2, row, row + 1,  GTK_EXPAND|GTK_FILL,0,5,0);
-				gtk_widget_show(wid);
 				break;
 			case CFT_COLOR:
-				label = gtk_label_new(fld->desc);
-				gtk_table_attach(GTK_TABLE(table), label, 0, 1, row, row + 1,  0,0,0,0);
-				gtk_widget_show(label);
+				label_build(fld->desc, table, 0, row);
 				wid = od_emil_button_new();
 				set_object_data(GTK_OBJECT(wid), fld, instance);
 				od_emil_button_set_colors(OD_EMIL_BUTTON(wid), OD_EMIL_BUTTON_PRIMARY, NULL, (GdkColor *) ((gchar *) instance + fld->offset));
 				gtk_signal_connect(GTK_OBJECT(wid), "activate", GTK_SIGNAL_FUNC(evt_color_clicked), cmc);
 				gtk_table_attach(GTK_TABLE(table), wid, 1, 2, row, row + 1, GTK_EXPAND|GTK_FILL,0,5,0);
-				gtk_widget_show(wid);
 				break;
 		}
 		return table;
@@ -627,9 +622,9 @@ gpointer cmc_instance_new(CmdCfg *cmc)
 {
 	if(cmc != NULL)
 	{
-		GList	*iter;
-		gsize	size = 0, fsize;
-		CField	*fld;
+		const GList	*iter;
+		gsize		size = 0, fsize;
+		CField		*fld;
 
 		for(iter = cmc->fields; iter != NULL; iter = g_list_next(iter))
 		{
@@ -689,8 +684,8 @@ void cmc_instance_copy(CmdCfg *cmc, gpointer dst, gpointer src)
 {
 	if((cmc != NULL) && (dst != NULL) && (src != NULL))
 	{
-		GList	*iter;
-		CField	*fld;
+		const GList	*iter;
+		CField		*fld;
 
 		for(iter = cmc->fields; iter != NULL; iter = g_list_next(iter))
 		{
@@ -741,7 +736,7 @@ GtkWidget * cmc_instance_build(CmdCfg *cmc, gpointer instance)
 
 	if((cmc != NULL) && (instance != NULL))
 	{
-		GList		*iter;
+		const GList	*iter;
 		gint		row = 0;
 
 		table = gtk_table_new(g_list_length(cmc->fields) - 1, 2, FALSE);
