@@ -12,6 +12,7 @@
 #include "cmd_copyas.h"
 #include "cmd_delete.h"
 #include "cmd_direnter.h"
+#include "cmd_dirrescan.h"
 #include "cmd_dpfocus.h"
 #include "cmd_dpmisc.h"
 #include "cmd_file.h"
@@ -27,7 +28,6 @@
 #include "cmd_rename.h"
 #include "cmd_renamere.h"
 #include "cmd_renameseq.h"
-#include "cmd_rescan.h"
 #include "cmd_run.h"
 #include "cmd_select.h"
 #include "cmd_split.h"
