@@ -342,7 +342,7 @@ gboolean ctrl_key_has_cmdseq(CtrlInfo *ctrl, CtrlKey *key, const gchar *cmdseq)
 static void key_install(gpointer key, gpointer value, gpointer user)
 {
 	if(((CtrlKey *) value)->cmdseq != NULL)
-		kbd_context_entry_add((KbdContext *) user, ((CtrlKey *) value)->keyname, KET_CMDSEQ, ((CtrlKey *) value)->cmdseq->str);
+		kbd_context_entry_add(user, ((CtrlKey *) value)->keyname, KET_CMDSEQ, ((CtrlKey *) value)->cmdseq->str);
 }
 
 /* 1999-03-16 -	Install all <ctrl>'s key mappings as command-type keyboard commands in <ctx>.
