@@ -16,6 +16,8 @@
 ** 1999-09-05 -	Cut away the fancy browser-esque history system implemented for 0.11.8, since it wasn't
 **		complete, and I didn't like it much. Going back to a simple combo as before.
 ** 1999-11-13 -	Prettied up handling of vertical position remembering somewhat, exported it separately.
+** 2002-08-09 -	DHEntries can't be keyed on inode, since that can be fooled by deleting a directory,
+**		and then creating a new (differently named) one. The filesystem can re-use the inode.
 */
 
 #include "gentoo.h"
@@ -24,46 +26,42 @@
 #include <sys/time.h>
 #include <unistd.h>
 
-#include "odemilbutton.h"
-
 #include "cmdseq.h"
 #include "dirpane.h"
 #include "errors.h"
 #include "fileutil.h"
 #include "strutil.h"
+#include "xmlutil.h"
 
 #include "dirhistory.h"
 
 /* ----------------------------------------------------------------------------------------- */
 
-#define	HISTORY_SIZE	(16)	/* Maximum number of items in history. Should be dynamic. */
+#define	HISTORY_SIZE	(16)	/* Maximum number of items in history. Should this be dynamic? */
 
 /* ----------------------------------------------------------------------------------------- */
 
-typedef struct {				/* A history key. Just a (device,inode) pair. */
+typedef struct {			/* A history key. A (device,inode) pair. */
 	dev_t	dev;
 	ino_t	inode;
+	guint	hash;
 } HKey;
 
 /* This is used to store the selected files in a pane, and nothing else. */
 struct DHSel {
-	GHashTable	*hash;			/* It's a hash of HKeys, as above. For fast 'apply' operations. */
+	GHashTable	*hash;		/* It's a hash of HKeys, as above. For fast 'apply' operations. */
 };
 
-/* Info about a single remembered directory. The 'hist' field of dirpanes stores a linked
-** list of these.
-*/
+/* Info about a single remembered directory. The 'hist' field of dirpanes stores a list of these. */
 typedef struct {
-	gchar		path[PATH_MAX];		/* The path to this directory. Must be first! */
-	HKey		key;			/* The directory's unique identifier. */
-	DHSel		*sel;			/* The selection last time we were here. */
-	gfloat		vpos;			/* Vertical position. */
-	gint		focus_row;		/* Row that had the keyboard-controlled focus. */
-	GTimeVal	a_time;
+	gchar	path[PATH_MAX];		/* The path to this directory. Must be first! */
+	gchar	realpath[PATH_MAX];	/* Canonical path, used as key since <dev,inode> isn't enough. */
+	DHSel	*sel;			/* The selection last time we were here. */
+	gfloat	vpos;			/* Vertical position. */
+	gint	focus_row;		/* Row that had the keyboard-controlled focus, or -1. */
 } DHEntry;
 
 struct DirHistory {
-	DHEntry	*current;		/* Description of current directory. */
 	GList	*history;		/* Visited directories. Recent are close to head. */
 };
 
@@ -75,7 +73,7 @@ static GMemChunk	*the_chunk = NULL;
 
 /* ----------------------------------------------------------------------------------------- */
 
-static DHSel *	dirsel_set(DHSel *sel, DirPane *dp);
+static DHSel *	dirsel_set(DHSel *sel, const DirPane *dp);
 
 /* ----------------------------------------------------------------------------------------- */
 
@@ -85,7 +83,6 @@ DirHistory * dph_dirhistory_new(void)
 	DirHistory	*dh;
 
 	dh = g_malloc(sizeof *dh);
-	dh->current = NULL;
 	dh->history = NULL;
 
 	return dh;
@@ -94,7 +91,7 @@ DirHistory * dph_dirhistory_new(void)
 /* ----------------------------------------------------------------------------------------- */
 
 /* 1998-12-23 -	Set a history key according to information in <st>. */
-static void hkey_set(HKey *hk, struct stat *st)
+static void hkey_set(HKey *hk, const struct stat *st)
 {
 	if(hk != NULL && st != NULL)
 	{
@@ -103,19 +100,6 @@ static void hkey_set(HKey *hk, struct stat *st)
 	}
 }
 
-/* 1999-06-05 -	Set a key from a name. Returns FALSE on failure, TRUE on success. */
-static gboolean hkey_set_from_name(HKey *hk, const gchar *path)
-{
-	struct stat	stbuf;
-
-	if(stat(path, &stbuf) == 0)
-	{
-		hkey_set(hk, &stbuf);
-		return TRUE;
-	}
-	return FALSE;
-}
-
 /* 1998-12-23 -	Create a new history key, initialized with information from <st>. */
 static HKey * hkey_new(struct stat *st)
 {
@@ -134,7 +118,7 @@ static HKey * hkey_new(struct stat *st)
 /* 1998-12-23 -	Compare two history keys for equality. */
 static gint hkey_equal(gconstpointer a, gconstpointer b)
 {
-	HKey	*ha = (HKey *) a, *hb = (HKey *) b;
+	const HKey	*ha = a, *hb = b;
 
 	return (ha->dev == hb->dev) && (ha->inode == hb->inode);
 }
@@ -142,7 +126,7 @@ static gint hkey_equal(gconstpointer a, gconstpointer b)
 /* 1998-12-23 -	Compute hash value from a history key. Nothing fancy. */
 static guint hkey_hash(gconstpointer a)
 {
-	HKey	*ha = (HKey *) a;
+	const HKey	*ha = a;
 
 	return (guint) ha->dev ^ (guint) ha->inode;
 }
@@ -172,7 +156,7 @@ static DHEntry * dhentry_new(void)
 
 	data = g_malloc(sizeof *data);
 	data->path[0] = '\0';
-	data->key.dev = data->key.inode = 0;
+	data->realpath[0] = '\0';
 	data->sel = NULL;
 	data->vpos = 0.0f;
 	data->focus_row = -1;
@@ -180,8 +164,18 @@ static DHEntry * dhentry_new(void)
 	return data;
 }
 
+static void dhentry_path_set(DHEntry *dhe, const gchar *path)
+{
+	g_snprintf(dhe->path, sizeof dhe->path, "%s", path);
+}
+
+static void dhentry_realpath_set(DHEntry *dhe, const gchar *realpath)
+{
+	g_snprintf(dhe->realpath, sizeof dhe->realpath, "%s", realpath);
+}
+
 /* 1999-06-09 -	Update <dhe> to mimic state of <dp>. */
-static void dhentry_update(DHEntry *dhe, DirPane *dp)
+static void dhentry_update(DHEntry *dhe, const DirPane *dp)
 {
 	dhe->sel  = dirsel_set(dhe->sel, dp);
 	dhe->vpos = dph_vpos_get(dp);
@@ -189,11 +183,10 @@ static void dhentry_update(DHEntry *dhe, DirPane *dp)
 		dhe->focus_row = dp->focus_row;
 	else
 		dhe->focus_row = -1;
-	g_get_current_time(&dhe->a_time);
 }
 
 /* 1999-06-09 -	Apply state conserved in <dhe> to the rows of <dp>. */
-static void dhentry_apply(DHEntry *dhe, DirPane *dp)
+static void dhentry_apply(const DHEntry *dhe, DirPane *dp)
 {
 	dph_dirsel_apply(dp, dhe->sel);
 	dph_vpos_set(dp, dhe->vpos);
@@ -205,7 +198,6 @@ static void dhentry_apply(DHEntry *dhe, DirPane *dp)
 			row = 0;
 		dp_focus(dp, row);
 	}
-	g_get_current_time(&dhe->a_time);
 }
 
 /* 1999-06-06 -	Destroy a directory history entry. */
@@ -247,14 +239,14 @@ static void dirsel_clear(DHSel *sel)
 **		new selection (or NULL if <dp> had no selected items). If <sel> is NULL on
 **		entry, a new selection will be created and returned.
 */
-static DHSel * dirsel_set(DHSel *sel, DirPane *dp)
+static DHSel * dirsel_set(DHSel *sel, const DirPane *dp)
 {
 	GSList	*slist;
 
 	if((slist = dp_get_selection_full(dp)) != NULL)
 	{
-		HKey	*key;
-		GSList	*iter;
+		HKey		*key;
+		const GSList	*iter;
 
 		if(sel == NULL)
 			sel = dirsel_new();
@@ -287,7 +279,7 @@ DHSel * dph_dirsel_new(DirPane *dp)
 **		and it need not have the same contents. This is not terribly efficient,
 **		but I think it'll be OK.
 */
-void dph_dirsel_apply(DirPane *dp, DHSel *sel)
+void dph_dirsel_apply(DirPane *dp, const DHSel *sel)
 {
 	HKey	key;
 	guint	i;
@@ -324,7 +316,7 @@ void dph_dirsel_destroy(DHSel *sel)
 /* 1999-11-13 -	Return a floating-point number that somehow encodes the vertical position of
 **		the given pane. The only use for that number is to pass it to dph_vpos_set().
 */
-gfloat dph_vpos_get(DirPane *dp)
+gfloat dph_vpos_get(const DirPane *dp)
 {
 	GtkAdjustment	*adj;
 
@@ -346,133 +338,176 @@ void dph_vpos_set(DirPane *dp, gfloat vpos)
 
 /* ----------------------------------------------------------------------------------------- */
 
-#if 0
-/* Here's a handy history display routine, for debugging. */
-static void hist_dump(DirHistory *dh)
-{
-	GList	*iter;
-
-	printf("(");
-	for(iter = dh->backward; iter != NULL; iter = g_list_next(iter))
-	{
-		printf("%s", ((DHEntry *) iter->data)->path);
-		if(g_list_next(iter))
-			putchar(' ');
-	}
-	printf(") '%s' (", ((DHEntry *) dh->current)->path);
-	for(iter = dh->forward; iter != NULL; iter = g_list_next(iter))
-	{
-		printf("%s", ((DHEntry *) iter->data)->path);
-		if(g_list_next(iter))
-			putchar(' ');
-	}
-	printf(") [");
-	for(iter = dh->cache; iter != NULL; iter = g_list_next(iter))
-	{
-		printf("%s", ((DHEntry *) iter->data)->path);
-		if(g_list_next(iter))
-			putchar(' ');
-	}
-	printf("]\n");
-}
-#endif
-
-/* 1999-06-06 -	Enter the directory specified by <entry>, and apply any historic information
-**		it may contain.
-** 2000-11-30 -	Did tricky resize-queuing + event-gobbling before setting vpos. Better.
+/* 2002-06-13 -	A new way to look at history handling. For the, like, third time or so. Anyway,
+**		this should just save the state of <dp>. This means the pane should already be
+**		displaying a directory.
 */
-static void enter_and_apply(DirPane *dp, DHEntry *entry)
+void dph_state_save(DirPane *dp)
 {
-	dp_freeze(dp);
-	dp_enter_dir(dp, entry->path);
-	dp_redisplay(dp);
-	dp_thaw(dp);
-	gtk_widget_queue_resize(GTK_WIDGET(dp->list));
-	while(gtk_events_pending())
-		gtk_main_iteration();
-	dhentry_apply(entry, dp);
-	dp->hist->current = entry;
+	gchar	rp[PATH_MAX];
+
+	if(dp->dir.num_rows == 0)		/* No directory shown? Then give up. */
+		return;
+	if(realpath(dp->dir.path, rp) == NULL)
+		return;
+	if((dp->hist->history == NULL) || strcmp(rp, ((DHEntry *) dp->hist->history->data)->realpath))
+	{
+		dp->hist->history = g_list_prepend(dp->hist->history, dhentry_new());
+		dhentry_path_set(dp->hist->history->data, dp->dir.path);
+		dhentry_realpath_set(dp->hist->history->data, rp);
+	}
+	dhentry_update(dp->hist->history->data, dp);
 }
 
-/* 1999-06-05 -	Enter <new_path> in <dp>, and update the history data accordingly. If the <new_path>
-**		isn't stat()-able, return FALSE while setting errno to something sensible.
-** 1999-09-05 -	Rewritten to deal with a simple, single, history list instead.
+/* 2002-06-13 -	Other half of the third-generation history interface. This restores any saved
+**		state for the directory currently shown by <dp>.
 */
-gboolean dph_direnter(DirPane *dp, const gchar *new_path)
+void dph_state_restore(DirPane *dp)
 {
-	DHEntry	*entry = NULL;
-	HKey	nkey;
 	GList	*iter;
+	gchar	rp[PATH_MAX];
+	DHEntry	*entry;
 
-	/* First make (reasonably) sure we can really enter the directory. */
-	if(access(new_path, R_OK) != 0 || !hkey_set_from_name(&nkey, new_path))
-		return FALSE;
-
-	/* Save current pane state in the current history entry. */
-	if(dp->hist->current != NULL)
-		dhentry_update(dp->hist->current, dp);
-
-	/* Find the history entry that describes the target directory, if there is one. */
-	for(iter = dp->hist->history; iter != NULL; iter = g_list_next(iter))
+	/* Search for key matching current dir in history. */
+	if(realpath(dp->dir.path, rp) == NULL)
+		return;
+	for(iter = dp->hist->history, entry = NULL; iter != NULL; iter = g_list_next(iter))
 	{
 		entry = iter->data;
-		if(hkey_equal(&nkey, &entry->key))
+		if(strcmp(rp, entry->realpath) == 0)
+			break;
+	}
+	if(iter != NULL)
+	{
+		dhentry_apply(entry, dp);
+		if(g_list_previous(iter) != NULL)
 		{
 			dp->hist->history = g_list_remove_link(dp->hist->history, iter);
 			g_list_free_1(iter);
-			break;
+			dp->hist->history = g_list_prepend(dp->hist->history, entry);
 		}
 	}
-
-	/* No old entry found? Then construct a new one. */
-	if(iter == NULL)
-		entry = dhentry_new();
-
-	stu_strncpy(entry->path, new_path, sizeof entry->path);
-	entry->key = nkey;
-
-	dp->hist->history = g_list_prepend(dp->hist->history, entry);
-
-	/* Has the list grown too long? Then we prune the end off. */
-	if(g_list_length(dp->hist->history) > HISTORY_SIZE)
+	else
 	{
-		GList	*tail, *next;
-
-		tail = g_list_nth(dp->hist->history, HISTORY_SIZE);
-		for(; tail != NULL; tail = next)
+		dp->hist->history = g_list_prepend(dp->hist->history, dhentry_new());
+		dhentry_path_set(dp->hist->history->data, dp->dir.path);
+		dhentry_realpath_set(dp->hist->history->data, rp);
+		/* Prune history list, if it has grown too long. */
+		if(g_list_length(dp->hist->history) > HISTORY_SIZE)
 		{
-			next = g_list_next(tail);
-			dp->hist->history = g_list_remove_link(dp->hist->history, tail);
-			dhentry_destroy(tail->data);
-			g_list_free_1(tail);
+			GList	*tail, *next;
+
+			tail = g_list_nth(dp->hist->history, HISTORY_SIZE);
+			for(; tail != NULL; tail = next)
+			{
+				next = g_list_next(tail);
+				dp->hist->history = g_list_remove_link(dp->hist->history, tail);
+				dhentry_destroy(tail->data);
+				g_list_free_1(tail);
+			}
 		}
 	}
-	enter_and_apply(dp, entry);
-
-	return TRUE;
-}
-
-/* 1999-09-05 -	Rescan the directory currently in <dp>. Implemented lazily. */
-gboolean dph_dirrescan(DirPane *dp)
-{
-	if(dph_direnter(dp, dp->dir.path))
-	{
-		dph_set_path_widgets(dp);
-		return TRUE;
-	}
-	return FALSE;
+	/* Update path history widgets. Relies on <path> member being first in DHEntry. */
+	gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(dp->path)->entry), dp->hist->history->data);
+	gtk_editable_changed(GTK_EDITABLE(GTK_COMBO(dp->path)->entry));
+	gtk_combo_set_popdown_strings(GTK_COMBO(dp->path), dp->hist->history);
 }
 
 /* ----------------------------------------------------------------------------------------- */
 
-/* 1999-06-25 -	Update path-related widgets for <dp>. This is a little bit besides the core "point" of this module
-**		perhaps, but since the data is so incredibly opaque, this has to go here. Big deal.
-*/
-void dph_set_path_widgets(DirPane *dp)
+/* 2002-02-15 -	Return first path from history for <dp>. */
+const gchar * dph_history_get_first(const DirPane *dp)
 {
-	DirHistory	*hist = dp->hist;
+	if(dp->hist->history != NULL)
+		return dp->hist->history->data;
+	return NULL;
+}
 
-	gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(dp->path)->entry), hist->current->path);
-	gtk_editable_changed(GTK_EDITABLE(GTK_COMBO(dp->path)->entry));
-	gtk_combo_set_popdown_strings(GTK_COMBO(dp->path), hist->history);	/* Since path is first in DHEntry, this works. */
+/* ----------------------------------------------------------------------------------------- */
+
+static const gchar * history_filename(MainInfo *min)
+{
+	static gchar	buffer[PATH_MAX];
+
+	if(fut_path_component("~/.gentoo-history", buffer))
+		return buffer;
+	return NULL;
+}
+
+/* 2002-02-15 -	Save contents of history lists for <num> panes, arrayed at <dp>. Weird inter-
+**		face, good code. In a blatant case of code reuse, we use our trusted XML module.
+**		We do not, however, reuse the main config file, but keep the histories separate.
+*/
+void dph_history_save(MainInfo *min, const DirPane *dp, gsize num)
+{
+	FILE	*out;
+	gsize	i;
+	GList	*iter;
+
+	if((out = xml_put_open(history_filename(min))) == NULL)
+		return;
+
+	xml_put_node_open(out, "DirHistory");
+	xml_put_node_open(out, "Panes");
+	for(i = 0; i < num; i++)
+	{
+		xml_put_node_open(out, "Pane");
+		xml_put_uinteger(out, "index", i);
+		xml_put_node_open(out, "History");
+		for(iter = dp[i].hist->history; iter != NULL; iter = g_list_next(iter))
+			xml_put_text(out, "dir", iter->data);
+		xml_put_node_close(out, "History");
+		xml_put_node_close(out, "Pane");
+	}
+	xml_put_node_close(out, "Panes");
+	xml_put_node_close(out, "DirHistory");
+
+	xml_put_close(out);
+}
+
+/* ----------------------------------------------------------------------------------------- */
+
+static void load_dir(const XmlNode *node, gpointer user)
+{
+	DirPane		*dp = user;
+	const gchar	*text;
+	DHEntry		*ent;
+
+	if(!xml_get_text(node, "dir", &text))
+		return;
+	ent = dhentry_new();
+	stu_strncpy(ent->path, text, sizeof ent->path);
+	if(realpath(ent->path, ent->realpath) != NULL)
+		dp->hist->history = g_list_append(dp->hist->history, ent);
+	else
+		dhentry_destroy(ent);
+}
+
+static void load_pane(const XmlNode *node, gpointer user)
+{
+	MainInfo	*min = user;
+	guint		index;
+
+	if(!xml_get_uinteger(node, "index", &index))
+		return;
+	if((node = xml_tree_search(node, "History")) == NULL)
+		return;
+	xml_node_visit_children(node, load_dir, min->gui->pane + index);
+}
+
+void dph_history_load(MainInfo *min, DirPane *dp, gsize num)
+{
+	XmlNode		*tree;
+	const XmlNode	*node;
+
+	if((tree = xml_tree_load(history_filename(min))) == NULL)
+		return;
+
+	if((node = xml_tree_search(tree, "DirHistory")) == NULL)
+		return;
+	if((node = xml_tree_search(node, "Panes")) == NULL)
+		return;
+
+	xml_node_visit_children(node, load_pane, min);
+	xml_tree_destroy(tree);
 }
