@@ -31,27 +31,26 @@ typedef struct {
 ** in rising content ID order, but we typically ignore that and search it sequentially
 ** every time. Just keep those searches out from the inner loops, and things should be fine.
 */
-static ContentInfo	content_info[] = {
-	{ DPC_NAME,		"Name",				"name",		N_("Name") },
-	{ DPC_SIZE,		"Size",				"size",		N_("Size") },
-	{ DPC_IQSIZE,		"Smart Size",			"smartsize",	N_("IQSize") },
-	{ DPC_BLOCKS,		"Blocks",			"blocks",	N_("Blocks") },
-	{ DPC_BLOCKSIZE,	"Block Size",			"blocksize",	N_("BSize") },
-	{ DPC_MODENUM,		"Mode, numerical",		"modenum",	N_("Mode") },
-	{ DPC_MODESTR,		"Mode, string",			"modestr",	N_("Mode") },
-	{ DPC_NLINK,		"Number of links",		"nlink",	N_("Nlink") },
-	{ DPC_UIDNUM,		"Owner ID",			"uid",		N_("Uid") },
-	{ DPC_UIDSTR,		"Owner Name",			"uname",	N_("Uname") },
-	{ DPC_GIDNUM,		"Group ID",			"gid",		N_("Gid") },
-	{ DPC_GIDSTR,		"Group Name",			"gname",	N_("Gname") },
-	{ DPC_DEVICE,		"Device Number",		"device",	N_("Device") },
-	{ DPC_DEVMAJ,		"Device Number, major",		"devmaj",	N_("DevMaj") },
-	{ DPC_DEVMIN,		"Device Number, minor",		"devmin",	N_("DevMin") },
-	{ DPC_ATIME,		"Date of last Access",		"atime",	N_("Accessed") },
-	{ DPC_MTIME,		"Date of last Modification",	"mtime",	N_("Modified") },
-	{ DPC_CTIME,		"Date of Creation",		"ctime",	N_("Created") },
-	{ DPC_TYPE,		"Type Name",			"type",		N_("Type") },
-	{ DPC_ICON,		"Icon",				"icon",		N_("I") }
+static ContentInfo content_info[] = {
+	{ DPC_NAME,		N_("Name"),			"name",		N_("Name") },
+	{ DPC_SIZE,		N_("Size"),			"size",		N_("Size") },
+	{ DPC_BLOCKS,		N_("Blocks"),			"blocks",	N_("Blocks") },
+	{ DPC_BLOCKSIZE,	N_("Block Size"),		"blocksize",	N_("BSize") },
+	{ DPC_MODENUM,		N_("Mode, numerical"),		"modenum",	N_("Mode") },
+	{ DPC_MODESTR,		N_("Mode, string"),		"modestr",	N_("Mode") },
+	{ DPC_NLINK,		N_("Number of links"),		"nlink",	N_("Nlink") },
+	{ DPC_UIDNUM,		N_("Owner ID"),			"uid",		N_("Uid") },
+	{ DPC_UIDSTR,		N_("Owner Name"),		"uname",	N_("Uname") },
+	{ DPC_GIDNUM,		N_("Group ID"),			"gid",		N_("Gid") },
+	{ DPC_GIDSTR,		N_("Group Name"),		"gname",	N_("Gname") },
+	{ DPC_DEVICE,		N_("Device Number"),		"device",	N_("Device") },
+	{ DPC_DEVMAJ,		N_("Device Number, major"),	"devmaj",	N_("DevMaj") },
+	{ DPC_DEVMIN,		N_("Device Number, minor"),	"devmin",	N_("DevMin") },
+	{ DPC_ATIME,		N_("Date of last Access"),	"atime",	N_("Accessed") },
+	{ DPC_MTIME,		N_("Date of last Modification"),"mtime",	N_("Modified") },
+	{ DPC_CTIME,		N_("Date of Creation"),		"ctime",	N_("Created") },
+	{ DPC_TYPE,		N_("Type Name"),		"type",		N_("Type") },
+	{ DPC_ICON,		N_("Icon"),			"icon",		N_("I") }
 	};
 
 /* ----------------------------------------------------------------------------------------- */
@@ -73,7 +72,7 @@ const gchar * dpf_get_content_name(DPContent content)
 	for(i = 0; i < sizeof content_info / sizeof content_info[0]; i++)
 	{
 		if(content_info[i].id == content)
-			return content_info[i].name_full;
+			return _(content_info[i].name_full);
 	}
 	return NULL;
 }
@@ -85,12 +84,27 @@ DPContent dpf_get_content_from_name(const gchar *name)
 
 	for(i = 0; i < sizeof content_info / sizeof content_info[0]; i++)
 	{
-		if(strcmp(content_info[i].name_full, name) == 0)
-			return (DPContent) content_info[i].id;
+		if(strcmp(content_info[i].name_full, name) == 0)	/* Compatibility: try it in English first. */
+			return content_info[i].id;
+		else if(strcmp(_(content_info[i].name_full), name) == 0)
+			return content_info[i].id;
 	}
 	return DPC_NUM_TYPES;
 }
 
+/* 2002-06-16 -	Get mnemonic (short, single-word, non-translated) name for content. */
+const gchar * dpf_get_content_mnemonic(DPContent content)
+{
+	guint i;
+
+	for(i = 0; i < sizeof content_info / sizeof *content_info; i++)
+	{
+		if(content_info[i].id == content)
+			return content_info[i].name_short;
+	}
+	return NULL;
+}
+
 /* 1999-11-14 -	Map a short, single word mnemonic content name to an integer content identifier.
 **		Returns an illegal identifier if the name is unknown.
 */
@@ -130,7 +144,7 @@ GtkWidget * dpf_get_content_menu(GtkSignalFunc sigfunc, gpointer user)
 	menu = gtk_menu_new();
 	for(i = 0; i < sizeof content_info / sizeof content_info[0]; i++)
 	{
-		item = gtk_menu_item_new_with_label(content_info[i].name_full);
+		item = gtk_menu_item_new_with_label(_(content_info[i].name_full));
 		gtk_object_set_user_data(GTK_OBJECT(item), GUINT_TO_POINTER(content_info[i].id));
 		gtk_signal_connect(GTK_OBJECT(item), "activate", GTK_SIGNAL_FUNC(sigfunc), user);
 		gtk_menu_append(GTK_MENU(menu), item);
@@ -144,7 +158,7 @@ void dpf_set_default_format(DpCFmt *fmt, DPContent content)
 {
 	stu_strncpy(fmt->title, dpf_get_content_title(content), sizeof fmt->title);
 	fmt->content = content;
-	fmt->just = GTK_JUSTIFY_LEFT;
+	fmt->just  = GTK_JUSTIFY_LEFT;
 	fmt->width = 50;
 
 	switch(fmt->content)
@@ -154,13 +168,14 @@ void dpf_set_default_format(DpCFmt *fmt, DPContent content)
 			fmt->width = 250;
 			break;
 		case DPC_SIZE:
-			stu_strncpy(fmt->extra.size.format, "%lu", sizeof fmt->extra.size.format);
+			fmt->extra.size.type = DPSIZE_BYTES;
+			fmt->extra.size.digits = 0;
+			fmt->extra.size.ticks  = 0;
+			fmt->extra.size.tick   = ',';
+			g_snprintf(fmt->extra.size.dformat, sizeof fmt->extra.size.dformat, "%%#.%uf", fmt->extra.size.digits);
 			fmt->extra.size.dir_show_fs_size = TRUE;
 			fmt->just = GTK_JUSTIFY_RIGHT;
 			break;
-		case DPC_IQSIZE:
-			fmt->extra.iqsize.dir_show_fs_size = TRUE;
-			break;
 		case DPC_BLOCKS:
 			stu_strncpy(fmt->extra.blocks.format, "%lu", sizeof fmt->extra.blocks.format);
 			break;
@@ -260,6 +275,12 @@ void dpf_init_defaults(CfgInfo *cfg)
 	cfg->dp_colors.color[DPCOL_FOCUS_SELECTED].green   = 0x0000;
 	cfg->dp_colors.color[DPCOL_FOCUS_SELECTED].blue	   = 0x9E79;
 	cfg->dp_colors.color[DPCOL_FOCUS_SELECTED].pixel   = 0UL;
+
+	cfg->dp_paning.orientation = DPORIENT_HORIZ;
+	cfg->dp_paning.mode	   = DPSPLIT_FREE;
+	cfg->dp_paning.value	   = 0.0;
+
+	cfg->dp_history.save = TRUE;	/* Slightly out of place, but hey. */
 }
 
 /* ----------------------------------------------------------------------------------------- */
@@ -285,33 +306,57 @@ static void format_typed_name(gchar *out, const gchar *name, mode_t mode)
 
 /* ----------------------------------------------------------------------------------------- */
 
-/* 1999-01-08 -	Put a size (regular or "IQSize") at <out>. Assumes <out> can hold 64 bytes.
-** 1999-03-27 -	Rewritten for new flags handling. Cleaner & shorter, too!
+/* 2002-03-27 -	A new "size" content formatter. This one is a lot more intelligent and feature-rich
+**		than the old one, and even incorporates the "IQ Size" content type, too. Way cleaner.
 */
-static void put_size(gchar *out, const gchar *fmt, DirRow *row, gboolean iq, gboolean dir_fs)
+static void put_size(gchar **out, const DC_Size *dcs, const DirRow *row)
 {
-	gsize	size = DP_ROW_LSTAT(row).st_size;
+	gchar	*uptr, *unit = NULL;
 
-	out[0] = '\0';
-
-	if(DP_ROW_FLAGS_CHK(row, DPRF_HAS_SIZE))		/* Does the row have a "real" size? */
+	if(!(S_ISDIR(DP_ROW_LSTAT(row).st_mode) && dcs->dir_show_fs_size) && !DP_ROW_FLAGS_CHK(row, DPRF_HAS_SIZE))
+		return;
+	if(dcs->type == DPSIZE_NATIVE)			/* Native gets it done right away, in a single line. */
+		g_snprintf(*out, 64, "%Lu", (guint64) DP_ROW_LSTAT(row).st_size);
+	else
 	{
-		if(iq)
-			sze_put_size(size, out, 64, SZE_AUTO);
-		else
-			g_snprintf(out, 64, fmt, size);
-	}
-	else if(S_ISDIR(DP_ROW_LSTAT(row).st_mode) && dir_fs)	/* If no, do we show directories' fs sizes? */
-	{
-		gint	len;
+		gdouble	size = DP_ROW_LSTAT(row).st_size;
 
-		out[0] = '(';
-		if(iq)
-			len = sze_put_size(size, out + 1, 63, SZE_AUTO);
+		if(dcs->type == DPSIZE_KILOBYTES)
+			size /= 1024.0;
+		else if(dcs->type == DPSIZE_MEGABYTES)
+			size /= 1024.0 * 1024.0;
+		else if(dcs->type == DPSIZE_GIGABYTES)
+			size /= 1024.0 * 1024.0 * 1024.0;
+		else if(dcs->type == DPSIZE_SMART)	/* Let's do this inline, for speed. */
+		{
+			if(size < (1 << 10))
+				unit = _("B");
+			else if(size < (1 << 20))
+			{
+				size /= 1024.0;
+				unit = _("KB");
+			}
+			else if(size < (1 << 30))
+			{
+				size /= 1024.0 * 1024.0;
+				unit = _("MB");
+			}
+			else
+			{
+				size /= 1024.0 * 1024.0 * 1024.0;
+				unit = _("GB");
+			}
+		}
+		if(dcs->digits && (size - (gint) size) != 0.0)
+			uptr = *out + 32 + sprintf(*out + 32, dcs->dformat, size - (gint) size);
 		else
-			len = g_snprintf(out + 1, 63, fmt, size);
-		out[len + 1] = ')';
-		out[len + 2] = '\0';
+			*(uptr = *out + 33) = '\0';
+		if(dcs->ticks)
+			*out = stu_tickify(*out + 32 + 1, size, dcs->tick);
+		else
+			*out = stu_tickify(*out + 32 + 1, size, '\0');
+		if(unit)
+			strcpy(uptr, unit);
 	}
 }
 
@@ -323,136 +368,104 @@ static void put_size(gchar *out, const gchar *fmt, DirRow *row, gboolean iq, gbo
 **		the formatting strings used for date columns, for example.
 ** 1998-06-24 -	By direct logical extension, I made all numerical columns' formatting configurable.
 ** 1998-08-10 -	Made this routine a bit more complex, and added it's friend the post-insertion
-**		dpf_post_format_column() function (below). If a call to that function is necessary _after_
-**		the row being formatted has been added, this function will return 1. Otherwise 0 is returned.
+**		dpf_post_format_column() function (below).
+** 2002-04-28 -	Added one level of indirection to the <out> argument, for new size formatting.
 */
-gint dpf_format_column(MainInfo *min, DirPane *dp, DirRow *row, guint column, gchar *out)
+void dpf_format_column(MainInfo *min, const DirPane *dp, const DirRow *row, guint column, gchar **out)
 {
-	DPFormat	*fmt;
-	DpCFmt		*col_fmt;
+	const DPFormat	*fmt;
+	const DpCFmt	*col_fmt;
+	const gchar	*name;
 	struct tm	*tm;
-	const char	*name;
 
 	fmt = &min->cfg.dp_format[dp->index];
-	*out = '\0';
-	if(column < fmt->num_columns)
+	if(column >= fmt->num_columns)
+		return;
+
+	**out = '\0';
+	col_fmt = &fmt->format[column];
+
+	switch(col_fmt->content)
 	{
-		col_fmt = &fmt->format[column];
+		case DPC_NAME:
+			if(col_fmt->extra.name.show_type)
+				format_typed_name(*out, DP_ROW_NAME(row), DP_ROW_LSTAT(row).st_mode);
+			else
+				sprintf(*out, "%s", DP_ROW_NAME(row));
+			if(S_ISLNK(DP_ROW_LSTAT(row).st_mode) && col_fmt->extra.name.show_linkname)
+				sprintf(strchr(*out, '\0'), " -> %s", DP_ROW_LINKNAME(row));
+			break;
+		case DPC_SIZE:
+			put_size(out, &col_fmt->extra.size, row);
+			break;
+		case DPC_BLOCKS:
+			sprintf(*out, col_fmt->extra.blocks.format, DP_ROW_LSTAT(row).st_blocks);
+			break;
+		case DPC_BLOCKSIZE:
+			sprintf(*out, col_fmt->extra.blocksize.format, DP_ROW_LSTAT(row).st_blocks * dp->dir.fs.stat.f_bsize);
+			break;
+		case DPC_MODENUM:
+			sprintf(*out, col_fmt->extra.mode.format, DP_ROW_LSTAT(row).st_mode);
+			break;
+		case DPC_MODESTR:
+			stu_mode_to_text(*out, 32, DP_ROW_LSTAT(row).st_mode);
+			break;
+		case DPC_NLINK:
+			sprintf(*out, col_fmt->extra.nlink.format, DP_ROW_LSTAT(row).st_nlink);
+			break;
 
-		switch(col_fmt->content)
-		{
-			case DPC_NAME:
-				if(col_fmt->extra.name.show_type)
-					format_typed_name(out, DP_ROW_NAME(row), DP_ROW_LSTAT(row).st_mode);
-				else
-					sprintf(out, "%s", DP_ROW_NAME(row));
-				if(S_ISLNK(DP_ROW_LSTAT(row).st_mode) && col_fmt->extra.name.show_linkname)
-					sprintf(strchr(out, '\0'), " -> %s", DP_ROW_LINKNAME(row));
-				break;
-			case DPC_SIZE:
-/*
-** Here's some (gruesomely inefficient) code to insert a separatar character into the size numbers.
-** Wrote it mainly as a proof-of-concept, and to get an idea of what it would look like. I like the
-** look, but not the implementation. This stuff should be taken care of by I18N code (%'ld).
-** Personally, I would prefer a half-space as the group separator. Unfortunately, that's not correct
-** in any locale I know of, and I really don't know how to get GTK+ to render such a thing. :(
-*/
-/*
+		case DPC_UIDNUM:
+			sprintf(*out, col_fmt->extra.uidnum.format, DP_ROW_LSTAT(row).st_uid);
+			break;
+		case DPC_UIDSTR:
+			if((name = usr_lookup_uname(DP_ROW_LSTAT(row).st_uid)) != NULL)
+				strcpy(*out, name);
+			else
+				sprintf(*out, "%d", (gint) DP_ROW_LSTAT(row).st_uid);
+			break;
+		case DPC_GIDNUM:
+			sprintf(*out, col_fmt->extra.gidnum.format, DP_ROW_LSTAT(row).st_gid);
+			break;
+		case DPC_GIDSTR:
+			if((name = usr_lookup_gname(DP_ROW_LSTAT(row).st_gid)) != NULL)
+				strcpy(*out, name);
+			else
+				sprintf(*out, "%d", (gint) DP_ROW_LSTAT(row).st_gid);
+			break;
 
-				{
-					gchar	tmp[32];
-					GString	*str;
-					gint	i, j;
+		case DPC_DEVICE:
+			sprintf(*out, col_fmt->extra.device.format, DP_ROW_LSTAT(row).st_rdev);
+			break;
+		case DPC_DEVMAJ:
+			sprintf(*out, col_fmt->extra.devmaj.format, DP_ROW_LSTAT(row).st_rdev >> 8);
+				break;
+		case DPC_DEVMIN:
+			sprintf(*out, col_fmt->extra.devmin.format, DP_ROW_LSTAT(row).st_rdev & 0xFF);
+			break;
 
-					put_size(tmp, col_fmt->extra.size.format, row, FALSE, col_fmt->extra.size.dir_show_fs_size);
-					str = g_string_new("");
-					for(i = strlen(tmp) - 1, j = 0; i >= 0; i--, j++)
-					{
-						if(j && !(j % 3))
-							g_string_prepend_c(str, '.');
-						g_string_prepend_c(str, tmp[i]);
-					}
-					strcpy(out, str->str);
-					g_string_free(str, TRUE);
-					break;
-				}
-*/
-				put_size(out, col_fmt->extra.size.format, row, FALSE, col_fmt->extra.size.dir_show_fs_size);
-				break;
-			case DPC_IQSIZE:
-				put_size(out, NULL, row, TRUE, col_fmt->extra.iqsize.dir_show_fs_size);
-				break;
-			case DPC_BLOCKS:
-				sprintf(out, col_fmt->extra.blocks.format, DP_ROW_LSTAT(row).st_blocks);
-				break;
-			case DPC_BLOCKSIZE:
-				sprintf(out, col_fmt->extra.blocksize.format, DP_ROW_LSTAT(row).st_blocks * dp->dir.fs.stat.f_bsize);
-				break;
-			case DPC_MODENUM:
-				sprintf(out, col_fmt->extra.mode.format, DP_ROW_LSTAT(row).st_mode);
-				break;
-			case DPC_MODESTR:
-				stu_mode_to_text(out, 32, DP_ROW_LSTAT(row).st_mode);
-				break;
-			case DPC_NLINK:
-				sprintf(out, col_fmt->extra.nlink.format, DP_ROW_LSTAT(row).st_nlink);
-				break;
-			case DPC_UIDNUM:
-				sprintf(out, col_fmt->extra.uidnum.format, DP_ROW_LSTAT(row).st_uid);
-				break;
-			case DPC_UIDSTR:
-				if((name = usr_lookup_uname(DP_ROW_LSTAT(row).st_uid)) != NULL)
-					strcpy(out, name);
-				else
-					sprintf(out, "%d", (int) DP_ROW_LSTAT(row).st_uid);
-				break;
-			case DPC_GIDNUM:
-				sprintf(out, col_fmt->extra.gidnum.format, DP_ROW_LSTAT(row).st_gid);
-				break;
-			case DPC_GIDSTR:
-				if((name = usr_lookup_gname(DP_ROW_LSTAT(row).st_gid)) != NULL)
-					strcpy(out, name);
-				else
-					sprintf(out, "%d", (int) DP_ROW_LSTAT(row).st_gid);
-				break;
-
-			case DPC_DEVICE:
-				sprintf(out, col_fmt->extra.device.format, DP_ROW_LSTAT(row).st_rdev);
-				break;
-			case DPC_DEVMAJ:
-				sprintf(out, col_fmt->extra.devmaj.format, DP_ROW_LSTAT(row).st_rdev >> 8);
-				break;
-			case DPC_DEVMIN:
-				sprintf(out, col_fmt->extra.devmin.format, DP_ROW_LSTAT(row).st_rdev & 0xFF);
-				break;
-
-			case DPC_ATIME:
-				if((tm = localtime(&DP_ROW_LSTAT(row).st_atime)) != NULL)
-					strftime(out, 32, col_fmt->extra.a_time.format, tm);
-				break;
-			case DPC_MTIME:
-				if((tm = localtime(&DP_ROW_LSTAT(row).st_mtime)) != NULL)
-					strftime(out, 32, col_fmt->extra.m_time.format, tm);
-				break;
-			case DPC_CTIME:
-				if((tm = localtime(&DP_ROW_LSTAT(row).st_ctime)) != NULL)
-					strftime(out, 32, col_fmt->extra.c_time.format, tm);
-				break;
-			case DPC_TYPE:
-				if(DP_ROW_TYPE(row) != NULL)
-					sprintf(out, "%s", DP_ROW_TYPE(row)->name);
-				else
-					sprintf(out, "<NULL>");
-				break;
-			case DPC_ICON:
-				out[0] = '\0';
-				return 1;
-			case DPC_NUM_TYPES:
-				fprintf(stderr, "**DPFORMAT: Mysterious!\n");
-				break;
-		}
+		case DPC_ATIME:
+			if((tm = localtime(&DP_ROW_LSTAT(row).st_atime)) != NULL)
+				strftime(*out, 32, col_fmt->extra.a_time.format, tm);
+			break;
+		case DPC_MTIME:
+			if((tm = localtime(&DP_ROW_LSTAT(row).st_mtime)) != NULL)
+				strftime(*out, 32, col_fmt->extra.m_time.format, tm);
+			break;
+		case DPC_CTIME:
+			if((tm = localtime(&DP_ROW_LSTAT(row).st_ctime)) != NULL)
+				strftime(*out, 32, col_fmt->extra.c_time.format, tm);
+			break;
+		case DPC_TYPE:
+			if(DP_ROW_TYPE(row) != NULL)
+				sprintf(*out, "%s", DP_ROW_TYPE(row)->name);
+			else
+				sprintf(*out, "(none)");
+			break;
+		case DPC_ICON:
+			break;
+		case DPC_NUM_TYPES:
+			break;
 	}
-	return 0;
 }
 
 /* 1998-08-11 -	Format the given row, once it has actually been added to the dirpane. Modified
@@ -477,10 +490,9 @@ void dpf_post_format_row(MainInfo *min, DirPane *dp, gint row)
 	stl = DP_ROW_TYPE(&dp->dir.row[row])->style;
 	if(stl == NULL)
 	{
-		fprintf(stderr, "DPFORMAT: No style for '%s'\n", DP_ROW_NAME(&dp->dir.row[row]));
+		g_warning("DpFormat: No style for \"%s\"", DP_ROW_NAME(&dp->dir.row[row]));
 		return;
 	}
-
 	iname = stl_style_property_get_icon(stl, SPN_ICON_UNSEL);
 	for(col = 0; col < min->cfg.dp_format[dp->index].num_columns; col++)
 	{
@@ -499,10 +511,12 @@ void dpf_post_format_row(MainInfo *min, DirPane *dp, gint row)
 /* 1999-05-10 -	Set colors of <row> in <dp> to the correct style's colors. */
 void dpf_format_colors_row(DirPane *dp, gint row)
 {
-	Style	*stl;
+	Style			*stl;
+	register GdkColor	*col;
 
 	stl = DP_ROW_TYPE(&dp->dir.row[row])->style;
 
-	gtk_clist_set_background(GTK_CLIST(dp->list), row, (GdkColor *) stl_style_property_get_color(stl, SPN_COL_UNSEL_BG));
+	if((col = (GdkColor *) stl_style_property_get_color(stl, SPN_COL_UNSEL_BG)) != NULL)
+		gtk_clist_set_background(GTK_CLIST(dp->list), row, col);
 	gtk_clist_set_foreground(GTK_CLIST(dp->list), row, (GdkColor *) stl_style_property_get_color(stl, SPN_COL_UNSEL_FG));
 }
