@@ -9,7 +9,6 @@
 #include "gentoo.h"
 
 #include <ctype.h>
-#include <errno.h>
 #include <fcntl.h>
 #include <regex.h>
 #include <stdio.h>
@@ -30,9 +29,9 @@
 #define	MIN_COPY_CHUNK	(1 << 17)
 
 /* Path components are relative to this if they're not absolute. */
-static char	base_path[PATH_MAX];
+static gchar	base_path[PATH_MAX];
 /* The length of the base. Saves some calls to strlen() in the middle of everything. :) */
-static int	base_len;
+static gint	base_len;
 
 /* ----------------------------------------------------------------------------------------- */
 
@@ -90,18 +89,6 @@ gboolean fut_fsize(gint fd, gsize *size)
 	return ret ? TRUE : FALSE;
 }
 
-/* 1999-01-29 -	Development debugging routine that helps find fd leaks. */
-void fut_check_fd(const gchar *label)
-{
-	gint	fd;
-
-	if((fd = open(".", O_RDONLY)) >= 0)
-	{
-		fprintf(stderr, "%s fd: %d\n", label, fd);
-		close(fd);
-	}
-}
-
 /* ----------------------------------------------------------------------------------------- */
 
 /* 1998-05-17 -	Return number of entries in given directory.
@@ -155,7 +142,6 @@ gint fut_dir_length(DirPane *dp, const gchar *path, gsize *auxsize, guint *num_l
 		}
 		closedir(dir);
 	}
-
 	if(auxsize != NULL)
 		*auxsize = nl;
 	if(num_links != NULL)
@@ -240,21 +226,19 @@ static gint do_dir_size(MainInfo *min, const gchar *path, guint64 *bytes, FUCoun
 
 	if(fc != NULL)		/* If there is a fc present, wipe it clean. */
 	{
-		fc->num_dirs  = 0;
-		fc->num_files = 0;
-		fc->num_links = 0;
-		fc->num_cdevs = 0;
-		fc->num_bdevs = 0;
-		fc->num_fifos = 0;
-		fc->num_socks = 0;
+		fc->num_dirs  = 0U;
+		fc->num_files = 0U;
+		fc->num_links = 0U;
+		fc->num_cdevs = 0U;
+		fc->num_bdevs = 0U;
+		fc->num_fifos = 0U;
+		fc->num_socks = 0U;
 
-		fc->num_total = 0;
+		fc->num_total = 0U;
 
-		fc->num_bytes  = 0;
-		fc->num_blocks = 0;
+		fc->num_bytes  = 0ULL;
+		fc->num_blocks = 0ULL;
 	}
-	if(bytes != NULL)
-		*bytes = 0;
 	if((ret = recurse_dir_size(min, path, bytes, fc, with_progress)) != 0 && fc != NULL)
 		fc->num_total = fc->num_dirs + fc->num_files + fc->num_links + fc->num_cdevs +
 				fc->num_bdevs + fc->num_fifos + fc->num_socks;
@@ -311,7 +295,7 @@ static void eval_env(gchar *component)
 				}
 				else
 				{
-					for(vptr = vname; *ptr && (isalnum((unsigned char) *ptr) || *ptr == '_') &&
+					for(vptr = vname; *ptr && (isalnum((guchar) *ptr) || *ptr == '_') &&
 						(gsize) (vptr - vname) < sizeof vname - 1;)
 					{
 						*vptr++ = *ptr++;
@@ -348,7 +332,7 @@ static void eval_home(gchar *component)
 			*uptr++ = *ptr++;
 		}
 		*uptr = '\0';
-		if((dir = (char *) usr_lookup_uhome(uname)) != NULL)
+		if((dir = usr_lookup_uhome(uname)) != NULL)
 			g_string_assign(temp, dir);
 		else
 			g_string_assign(temp, "");
@@ -403,7 +387,7 @@ gint fut_path_component(const gchar *paths, gchar *component)
 		src = NULL;			/* Makes us stop next time. */
 	comp_len = dst - component;
 
-	if(use_env)				/* Do we need to look up environmet variables? */
+	if(use_env)				/* Do we need to look up environment variables? */
 		eval_env(component);
 	if(component[0] == '~')			/* Do we need to look up a home directory? */
 		eval_home(component);
@@ -419,7 +403,6 @@ gint fut_path_component(const gchar *paths, gchar *component)
 			}
 		}
 	}
-
 	return dst > component;
 }
 
@@ -439,6 +422,7 @@ const gchar * fut_locate(const gchar *paths, const gchar *name)
 {
 	static gchar	buf[PATH_MAX];
 	const gchar	*ptr = paths;
+	gsize		len;
 
 	if(*name == G_DIR_SEPARATOR)		/* Absolute name? */
 	{
@@ -449,8 +433,8 @@ const gchar * fut_locate(const gchar *paths, const gchar *name)
 
 	while(fut_path_component(ptr, buf))	/* No, iterate paths. */
 	{
-		stu_strncat(buf, G_DIR_SEPARATOR_S,  sizeof buf);
-		stu_strncat(buf, name, sizeof buf);
+		len = strlen(buf);
+		g_snprintf(buf + len, sizeof buf - len, "%s%s", G_DIR_SEPARATOR_S, name);
 		if(fut_exists(buf))
 			return buf;
 		ptr = NULL;
@@ -475,7 +459,7 @@ static gboolean kill_path(gpointer key, gpointer data, gpointer user)
 ** 1998-12-23 -	Now ignores directories already visited. Uses a string comparison rather than a
 **		file-system level check, so you can still fool it. It's a bit harder, though.
 */
-GList * fut_scan_path(const gchar *paths, gint (*filter)(const gchar *path, const gchar *name), gboolean with_path)
+GList * fut_scan_path(const gchar *paths, gint (*filter)(const gchar *path, const gchar *name))
 {
 	gchar		buf[PATH_MAX], *item;
 	const gchar	*ptr = paths;
@@ -492,7 +476,10 @@ GList * fut_scan_path(const gchar *paths, gint (*filter)(const gchar *path, cons
 		return NULL;
 
 	if((set = g_hash_table_new(g_str_hash, g_str_equal)) == NULL)
+	{
+		g_string_free(temp, TRUE);
 		return NULL;
+	}
 
 	for(ptr = paths; fut_path_component(ptr, buf); ptr = NULL)
 	{
@@ -505,17 +492,8 @@ GList * fut_scan_path(const gchar *paths, gint (*filter)(const gchar *path, cons
 			{
 				if(filter(buf, de->d_name))
 				{
-					if(with_path)
-					{
-						g_string_assign(temp, buf);
-						g_string_append_c(temp, G_DIR_SEPARATOR);
-						g_string_append(temp, de->d_name);
-						item = g_strdup(temp->str);
-					}
-					else
-						item = g_strdup(de->d_name);
-					if(item != NULL)
-						list = g_list_insert_sorted(list, (gpointer) item, (GCompareFunc) strcmp);
+					item = g_strdup(de->d_name);
+					list = g_list_insert_sorted(list, item, (GCompareFunc) strcmp);
 				}
 			}
 			closedir(dir);
@@ -524,6 +502,7 @@ GList * fut_scan_path(const gchar *paths, gint (*filter)(const gchar *path, cons
 	g_hash_table_foreach_remove(set, kill_path, NULL);
 	g_hash_table_destroy(set);
 	g_string_free(temp, TRUE);
+
 	return list;
 }
 
@@ -554,10 +533,11 @@ void fut_free_path(GList *list)
 ** 2000-07-16 -	Bug fix: the interruption check was done after the write(), so small files
 **		that fit into the chunk_size could not be interrupted. Oops.
 */
-gsize fut_copy(gint in, gint out, gsize chunk_size)
+off_t fut_copy(gint in, gint out, gsize chunk_size)
 {
 	gpointer	buf;
-	gssize		put = 0, got, wrote;
+	gssize		got, wrote;
+	off_t		put = 0U;
 
 	if((in < 0) || (out < 0))
 		return 0;
@@ -567,7 +547,7 @@ gsize fut_copy(gint in, gint out, gsize chunk_size)
 	buf = g_malloc(chunk_size);
 	while((got = read(in, buf, chunk_size)) > 0)
 	{
-		if(!pgs_progress_item_update(NULL, (guint32) put))
+		if(!pgs_progress_item_update(NULL, put))
 		{
 			errno = EINTR;
 			break;
@@ -583,10 +563,11 @@ gsize fut_copy(gint in, gint out, gsize chunk_size)
 }
 
 /* 1999-11-13 -	Copy just <length> bytes from <in> to <out>. */
-gsize fut_copy_partial(gint in, gint out, gsize chunk_size, gsize length)
+off_t fut_copy_partial(gint in, gint out, gsize chunk_size, gsize length)
 {
 	gpointer	buf;
-	gssize		put = 0, got, block, wrote;
+	gssize		got, wrote, block;
+	off_t		put = 0U;
 
 	chunk_size = MAX(MIN_COPY_CHUNK, MIN(chunk_size, length));
 
