@@ -25,19 +25,18 @@ gint		fut_dir_size(MainInfo *min, const gchar *path, guint64 *bytes, FUCount *fc
 gint		fut_dir_size_progress(MainInfo *min, const gchar *path, guint64 *bytes, FUCount *fc);
 
 const gchar *	fut_locate(const gchar *paths, const gchar *name);
-GList *		fut_scan_path(const gchar *paths, gint (*filter)(const gchar *path, const gchar *name), gboolean with_path);
+GList *		fut_scan_path(const gchar *paths, gint (*filter)(const gchar *path, const gchar *name));
 void		fut_free_path(GList *list);
 gint		fut_path_component(const gchar *paths, gchar *component);
 
-gsize		fut_copy(gint in, gint out, gsize chunk_size);
-gsize		fut_copy_partial(gint in, gint out, gsize chunk_size, gsize length);
+off_t		fut_copy(gint in, gint out, gsize chunk_size);
+off_t		fut_copy_partial(gint in, gint out, gsize chunk_size, gsize length);
 
 gboolean	fut_cd(const gchar *new_dir, gchar *old, gsize old_max);
 
 gboolean	fut_exists(const gchar *name);
 gboolean	fut_size(const gchar *name, gsize *size);
 gboolean	fut_fsize(gint fd, gsize *size);
-void		fut_check_fd(const gchar *label);
 
 gboolean	fut_can_read(const struct stat *stat, uid_t uid, gid_t gid);
 gboolean	fut_can_read_named(const gchar *name);
