@@ -12,23 +12,25 @@
 #include <sys/wait.h>
 
 #include "errors.h"
+#include "configure.h"
 #include "dirpane.h"
 #include "dialog.h"
 #include "userinfo.h"
 #include "file.h"
 #include "fileutil.h"
+#include "gfam.h"
 #include "xmlutil.h"
 #include "strutil.h"
 #include "types.h"
 #include "styles.h"
 #include "sizeutil.h"
 #include "buttons.h"
+#include "buttonlayout.h"
 #include "queue.h"
 #include "cmdseq.h"
 #include "children.h"
 #include "dpformat.h"
 #include "options.h"
-#include "shortcuts.h"
 #include "mount.h"
 #include "keyboard.h"
 #include "controls.h"
@@ -38,7 +40,6 @@
 
 #include "odlabel.h"
 
-#include "cfg_gui.h"
 #include "cfg_module.h"
 #include "cfg_windows.h"
 
@@ -59,7 +60,7 @@ static gboolean dir_filter(const gchar *name)
 /* ----------------------------------------------------------------------------------------- */
 
 /* 1998-10-11 -	This is called as GTK+ quits. Very handy. */
-static gint quit_handler(gpointer data)
+static gint quit_handler(gpointer user)
 {
 	chd_kill_children();
 
@@ -78,16 +79,12 @@ static gboolean evt_main_delete(GtkWidget *wid, GdkEventAny *evt, gpointer user)
 	return TRUE;
 }
 
-/* 1998-10-21 -	This gets called as the user closes the window. Shut down GTK+ main loop. */
-static void evt_main_destroy(GtkWidget *wid, gpointer data)
-{
-	gtk_main_quit();
-}
-
 /* 1998-10-26 -	Initialize some fields in the dp structure. */
 static void init_pane(DirPane *dp, gint index)
 {
 	dp->index = index;
+	dp->complete.list = NULL;
+	dp->complete.iter = NULL;
 	dp->dir.row = NULL;
 	dp->dir.auxbuf = NULL;
 	dp->hist = dph_dirhistory_new();
@@ -122,7 +119,7 @@ static void evt_pane_sizealloc(GtkWidget *wid, GtkAllocation *alloc, gpointer us
 		cfg_modified_set(min);
 }
 
-/* 2001-08-04 -	Track release of GtkHPaned handle, and react according to current mode. */
+/* 2001-08-04 -	Track release of GtkPaned handle, and react according to current mode. */
 static gboolean evt_paned_buttonrelease(GtkWidget *wid, GdkEventButton *evt, gpointer user)
 {
 	if(evt->window == GTK_PANED(wid)->handle)
@@ -141,31 +138,24 @@ static gboolean evt_paned_buttonrelease(GtkWidget *wid, GdkEventButton *evt, gpo
 */
 static GtkWidget * build_dirpanes(MainInfo *min)
 {
-	GtkWidget	*box, *left, *right;
+	GtkWidget	*pane;
 	GuiInfo		*gui;
 
 	gui = min->gui;
 
-	box = gtk_hbox_new(FALSE, 0);
-	gui->hpane = gtk_hpaned_new();
-	gtk_signal_connect(GTK_OBJECT(gui->hpane), "button_release_event", GTK_SIGNAL_FUNC(evt_paned_buttonrelease), min);
+	gui->panes = min->cfg.dp_paning.orientation == DPORIENT_HORIZ ? gtk_hpaned_new() : gtk_vpaned_new();
+	gtk_signal_connect(GTK_OBJECT(gui->panes), "button_release_event", GTK_SIGNAL_FUNC(evt_paned_buttonrelease), min);
 	init_pane(&gui->pane[0], 0);
-	if((left = dp_build(min, &min->cfg.dp_format[0], &gui->pane[0])) != NULL)
-	{
-		init_pane(&gui->pane[1], 1);
-		if((right = dp_build(min, &min->cfg.dp_format[1], &gui->pane[1])) != NULL)
-		{
-			gtk_paned_add1(GTK_PANED(gui->hpane), left);
-			gtk_paned_add2(GTK_PANED(gui->hpane), right);
-			gtk_paned_set_handle_size(GTK_PANED(gui->hpane), 10);
-			gtk_paned_set_gutter_size(GTK_PANED(gui->hpane), 10);
-			gtk_box_pack_start(GTK_BOX(box), gui->hpane,  TRUE, TRUE, 0);
-			gtk_widget_show(gui->hpane);
-			gtk_widget_show(box);
-			return box;
-		}
-	}
-	return NULL;
+	pane = dp_build(min, &min->cfg.dp_format[0], &gui->pane[0]);
+	gtk_paned_add1(GTK_PANED(gui->panes), pane);
+	init_pane(&gui->pane[1], 1);
+	pane = dp_build(min, &min->cfg.dp_format[1], &gui->pane[1]);
+	gtk_paned_add2(GTK_PANED(gui->panes), pane);
+	gtk_paned_set_handle_size(GTK_PANED(gui->panes), 10);
+	gtk_paned_set_gutter_size(GTK_PANED(gui->panes), 10);
+	gtk_widget_show(gui->panes);
+
+	return gui->panes;
 }
 
 /* 1998-05-19 -	Create the top widgetry, returning something the caller can just add to a box. Sets the
@@ -191,15 +181,17 @@ static GtkWidget * build_top(GtkWidget **label)
 	return top;
 }
 
-/* 1998-12-25 -	Rewritten in a lot less lines thanks to the new shortcuts module. */
+/* 2002-05-31 -	Rewritten yet again, now using the first weak version of a specialized button layout module. */
 static GtkWidget * build_bottom(MainInfo *min)
 {
-	GtkWidget	*hbox, *bbank;
+	GtkWidget	*hbox, *sb, *cb;
 
-	hbox = gtk_hbox_new(FALSE, 0);
-	bbank = btn_buttonsheet_build(min, &min->cfg.buttons, NULL, FALSE, NULL, min);
-	btn_buttonsheet_built_add_keys(min, GTK_CONTAINER(bbank), NULL);
-	shc_shortcutbank_pack(min, min->cfg.shortcutbank, hbox, bbank);
+	if((sb = btn_buttonsheet_build(min, &min->cfg.buttons, "Shortcuts", FALSE, NULL, NULL)) != NULL)
+		btn_buttonsheet_built_add_keys(min, GTK_CONTAINER(sb), NULL);
+	if((cb = btn_buttonsheet_build(min, &min->cfg.buttons, NULL, FALSE, NULL, min)) != NULL)
+		btn_buttonsheet_built_add_keys(min, GTK_CONTAINER(cb), NULL);
+	if((hbox = btl_buttonlayout_pack(min->cfg.buttonlayout, cb, sb)) != NULL)
+		gtk_widget_show_all(hbox);
 
 	return hbox;
 }
@@ -209,16 +201,21 @@ void rebuild_middle(MainInfo *min)
 {
 	GtkWidget	*left, *right;
 
-	gtk_container_remove(GTK_CONTAINER(min->gui->hpane), min->gui->pane[0].vbox);
-	gtk_container_remove(GTK_CONTAINER(min->gui->hpane), min->gui->pane[1].vbox);
+	gtk_container_remove(GTK_CONTAINER(min->gui->panes), min->gui->pane[0].vbox);
+	gtk_container_remove(GTK_CONTAINER(min->gui->panes), min->gui->pane[1].vbox);
+	gtk_widget_destroy(min->gui->panes);
+	min->gui->panes = min->cfg.dp_paning.orientation == DPORIENT_HORIZ ? gtk_hpaned_new() : gtk_vpaned_new();
+	gtk_box_pack_start(GTK_BOX(min->gui->vbox), min->gui->panes, TRUE, TRUE, 0);
+	gtk_box_reorder_child(GTK_BOX(min->gui->vbox), min->gui->panes, 1);
 	if((left = dp_build(min, &min->cfg.dp_format[0], &min->gui->pane[0])) != NULL)
 	{
 		min->gui->sig_left_sizealloc = gtk_signal_connect(GTK_OBJECT(left), "size_allocate", GTK_SIGNAL_FUNC(evt_pane_sizealloc), min);
 		gtk_signal_handler_block(GTK_OBJECT(left), min->gui->sig_left_sizealloc);
-		gtk_paned_add1(GTK_PANED(min->gui->hpane), left);
+		gtk_paned_add1(GTK_PANED(min->gui->panes), left);
 	}	
 	if((right = dp_build(min, &min->cfg.dp_format[1], &min->gui->pane[1])) != NULL)
-		gtk_paned_add2(GTK_PANED(min->gui->hpane), right);
+		gtk_paned_add2(GTK_PANED(min->gui->panes), right);
+	gtk_widget_show(min->gui->panes);
 }
 
 /* 1998-07-14 -	Rebuild the bottom part of the GUI, whose main responsibility is to contain
@@ -232,16 +229,33 @@ void rebuild_bottom(MainInfo *min)
 	gtk_widget_show(min->gui->bottom);
 }
 
-/* 1999-03-26 -	Set icon for when gentoo gets iconified. Neato. */
-static void set_icon(GtkWidget *win)
+/* 1999-03-26 -	Set icon for when gentoo gets iconified. Neato.
+** 2002-08-10 -	It seems gdk_window_set_icon_name() has problems when the locale gets set to
+**		something more interesting than "C". So we work around that.
+*/
+static void window_set_icon(GtkWidget *win)
 {
 	GdkPixmap	*pmap;
 	GdkBitmap	*bmap;
 
 	if((pmap = gdk_pixmap_create_from_xpm_d(win->window, &bmap, NULL, icon_iconify_xpm)) != NULL)
 	{
+		const gchar	*op;
+		gchar		old[256];
+
 		gdk_window_set_icon(win->window, NULL, pmap, bmap);
-		gdk_window_set_icon_name(win->window, "gentoo");
+#if defined ENABLE_NLS
+		if((op = setlocale(LC_ALL, NULL)) != NULL)
+		{
+			g_snprintf(old, sizeof old, "%s", op);
+			setlocale(LC_ALL, "C");
+		}
+#endif
+		gdk_window_set_icon_name(win->window, PACKAGE);
+#if defined ENABLE_NLS
+		if(op != NULL)
+			setlocale(LC_ALL, old);
+#endif
 		gdk_window_set_group(win->window, win->window);
 	}
 }
@@ -267,52 +281,45 @@ static void evt_main_sizealloc(GtkWidget *wid, GtkAllocation *alloc, gpointer us
 			break;
 	}
 	if(pos > 0)
-		gtk_paned_set_position(GTK_PANED(min->gui->hpane), pos);
+		gtk_paned_set_position(GTK_PANED(min->gui->panes), pos);
 }
 
 static GtkWidget * build_gui(MainInfo *min)
 {
-	gchar		title[64];
 	GtkWidget	*top, *panes, *bottom;
 
 	min->gui = g_malloc(sizeof *min->gui);
 	min->gui->window = NULL;
 	min->gui->window = win_window_open(min->cfg.wininfo, WIN_MAIN);
-	g_snprintf(title, sizeof title, "gentoo v" VERSION "%s", geteuid() == 0 ? " [root]" : "");
-	gtk_window_set_title(GTK_WINDOW(min->gui->window), title);
-	gtk_object_set_user_data(GTK_OBJECT(min->gui->window), (gpointer) min);
+	gtk_object_set_user_data(GTK_OBJECT(min->gui->window), min);
 	gtk_signal_connect(GTK_OBJECT(min->gui->window), "size_allocate", GTK_SIGNAL_FUNC(evt_main_sizealloc), min);
-	min->gui->pane[0].main = min;
-	min->gui->pane[0].list = NULL;
-	min->gui->pane[1].main = min;
-	min->gui->pane[1].list = NULL;
-	min->gui->pane[0].flags = min->gui->pane[1].flags = 0UL;
+	min->gui->pane[0].main = min->gui->pane[1].main = min;
+	min->gui->pane[0].list = min->gui->pane[1].list = NULL;
 	min->gui->vbox = gtk_vbox_new(FALSE, 0);
 	min->gui->main_tt = gtk_tooltips_new();
 	min->gui->kbd_ctx = kbd_context_new(min);
 	min->gui->sig_left_sizealloc = 0U;
-	if((top = build_top(&min->gui->top)) != NULL)
+
+	top = build_top(&min->gui->top);
+	if((panes = build_dirpanes(min)) != NULL)
 	{
-		if((panes = build_dirpanes(min)) != NULL)
-		{
-			bottom = build_bottom(min);
-			gtk_box_pack_start(GTK_BOX(min->gui->vbox), top,    FALSE, FALSE, 0);
-			gtk_box_pack_start(GTK_BOX(min->gui->vbox), panes,  TRUE,  TRUE,  0);
-			gtk_box_pack_start(GTK_BOX(min->gui->vbox), bottom, FALSE, FALSE, 0);
-			gtk_widget_show(bottom);
-			gtk_widget_show(min->gui->vbox);
-			min->gui->middle = panes;
-			min->gui->bottom = bottom;
-			min->gui->cur_pane = NULL;
-			gtk_tooltips_enable(min->gui->main_tt);
-			kbd_context_attach(min->gui->kbd_ctx, GTK_WINDOW(min->gui->window));
-			ctrl_keys_install(min->cfg.ctrlinfo, min->gui->kbd_ctx);
-			return min->gui->vbox;
-			gtk_widget_destroy(panes);
-		}
-		gtk_widget_destroy(top);
+		bottom = build_bottom(min);
+		gtk_box_pack_start(GTK_BOX(min->gui->vbox), top,    FALSE, FALSE, 0);
+		gtk_box_pack_start(GTK_BOX(min->gui->vbox), panes,  TRUE,  TRUE,  0);
+		gtk_box_pack_start(GTK_BOX(min->gui->vbox), bottom, FALSE, FALSE, 0);
+		gtk_widget_show(bottom);
+		gtk_widget_show(min->gui->vbox);
+		min->gui->middle = panes;
+		min->gui->bottom = bottom;
+		min->gui->cur_pane = &min->gui->pane[0];
+		gtk_tooltips_enable(min->gui->main_tt);
+		kbd_context_attach(min->gui->kbd_ctx, GTK_WINDOW(min->gui->window));
+		ctrl_keys_install(min->cfg.ctrlinfo, min->gui->kbd_ctx);
+		return min->gui->vbox;
 	}
+	gtk_widget_destroy(top);
 	gtk_widget_destroy(min->gui->vbox);
+
 	return NULL;
 }
 
@@ -364,6 +371,8 @@ static void evt_key_press(GtkWidget *wid, GdkEventKey *evt, gpointer user)
 {
 	MainInfo	*min = user;
 
+	printf("Someone pressed %d ('%c'), state=%04X\n", evt->keyval, evt->keyval, evt->state);
+
 	if(evt->keyval == GDK_v)
 	{
 		GtkAdjustment	*adj;
@@ -382,7 +391,7 @@ static void evt_key_press(GtkWidget *wid, GdkEventKey *evt, gpointer user)
 		Dialog	*dlg;
 		gint	ret;
 
-		dlg = dlg_dialog_sync_new(NULL, "Hello", "A|B");
+		dlg = dlg_dialog_sync_new(NULL, "Hello", "_A|_B");
 		ret = dlg_dialog_sync_wait(dlg);
 		printf("You answered %d\n", ret);
 		dlg_dialog_sync_destroy(dlg);
@@ -394,34 +403,92 @@ static void evt_key_press(GtkWidget *wid, GdkEventKey *evt, gpointer user)
 			min->cfg.dp_paning.mode = DPSPLIT_FREE;
 		printf("Pane split mode: %d\n", min->cfg.dp_paning.mode);
 	}
+	else if(evt->keyval == GDK_0)
+		gtk_widget_hide(min->gui->pane[0].scwin);
+	else if(evt->keyval == GDK_plus)
+		gtk_widget_show(min->gui->pane[0].scwin);
 }
 #endif
 
+/* 2002-02-19 -	Resolve what the initial directory is going to be, and go there. */
+static void enter_initial_dirs(MainInfo *min, const gchar *dirleft, const gchar *dirright)
+{
+	const gchar	*dir;
+	const gchar	*odir[2];
+	const gchar	*cmd[] = { "ActivateLeft", "ActivateRight" };
+	gint		i;
+
+	odir[0] = dirleft;
+	odir[1] = dirright;
+	for(i = sizeof cmd / sizeof *cmd - 1; i >= 0; i--)
+	{
+		if((dir = odir[i]) == NULL)
+		{
+			dir = min->cfg.dp_format[i].def_path;
+			if(!dir[0])
+			{
+				dir = dph_history_get_first(&min->gui->pane[i]);
+				if(dir == NULL)
+					dir = ".";
+			}
+		}
+		if(dir && *dir)
+		{
+			csq_execute(min, cmd[i]);
+			csq_execute_format(min, "DirEnter 'dir=%s'", dir);
+		}
+	}
+}
+
 int main(int argc, char *argv[])
 {
-	gchar		welcome[128], *dir[2];
-	guint32		arg_flags, cfg_flags = 0U;
+	enum { OPT_VERSION = 1 << 0, OPT_ROOTOK = 1 << 1, OPT_NORC = 1 << 2, OPT_NOGTKRC = 1 << 3, OPT_NOFAM = 1 << 4 };
+	OptDesc	od[] = { { '\0', "version",  OPT_ARG_NONE, NULL, OPT_VERSION,	N_("Report the version to standard output, and exit.") },
+			 { '\0', "root-ok",  OPT_ARG_NONE, NULL, OPT_ROOTOK,	N_("Allows gentoo to be run by the root user. Could be dangerous.") },
+			 { '\0', "no-rc",    OPT_ARG_NONE, NULL, OPT_NORC,	N_("Do not load the ~/.gentoorc configuration file; instead, use default values.") },
+			 { '\0', "no-gtkrc", OPT_ARG_NONE, NULL, OPT_NOGTKRC,	N_("Do not load the ~/.gentoogtkrc GTK+ configuration file; instead, use system defaults.") },
+#if defined HAVE_FAM
+			 { '\0', "no-fam",   OPT_ARG_NONE, NULL, OPT_NOFAM,	N_("Do not use FAM to automatically detect changes in viewed directories.") },
+#endif
+			 { '1',  "left",     OPT_ARG_REQ,  NULL, 0U, N_("Use ARG as path for the left directory pane. Overrides default (and history).") },
+			 { '2',  "right",    OPT_ARG_REQ,  NULL, 0U, N_("Use ARG as path for the right directory pane. Overrides default (and history).") }
+			};
+	guint32		opt_flags, cfg_flags = 0U;
 	GtkWidget	*box;
 	static MainInfo	min;
 
-#if defined ENABLE_NLS
-	setlocale(LC_ALL, "");
-	bindtextdomain(GENTOO_PACKAGE, GENTOO_LOCALEDIR);
-	textdomain(GENTOO_PACKAGE);
-#endif
-	/* Parse options. Might exit, if --version or root-problems occur. */
-	arg_flags = opt_extract(&argc, argv);
-
 	memset(&min, 0, sizeof min);
 	min.gui = NULL;
 
+#if defined ENABLE_NLS	
+	setlocale(LC_ALL, "");
+	bindtextdomain(PACKAGE, LOCALEDIR);
+	textdomain(PACKAGE);
+	for(opt_flags = 0; opt_flags < sizeof od / sizeof *od; opt_flags++)
+		od[opt_flags].help = _(od[opt_flags].help);
+#endif
+	opt_flags = opt_parse(&argc, argv, od, sizeof od / sizeof od[0]);
+	if(argc < 1)
+		return argc < -1 ? EXIT_FAILURE : EXIT_SUCCESS;
+	if(opt_flags & OPT_VERSION)
+	{
+		puts(VERSION);
+		return EXIT_SUCCESS;
+	}
+	if(geteuid() == 0 && !(opt_flags & OPT_ROOTOK))
+	{
+		fprintf(stderr, _("%s: To run as root, envoke with the '--root-ok' option\n"), argv[0]);
+		return EXIT_FAILURE;
+	}
+
 	/* Must be run early, before anyone changes the directory. */
 	fut_initialize();
 
 	gtk_init(&argc, &argv);
 
-	gtk_quit_add(1, quit_handler, (gpointer) &min);
+	gtk_quit_add(1, quit_handler, &min);
 
+	dp_initialize();
 	dpf_initialize();
 	sze_initialize();
 	mnu_initialize();
@@ -430,9 +497,9 @@ int main(int argc, char *argv[])
 	dpf_init_defaults(&min.cfg);
 	csq_init_commands(&min);
 	mnt_init_defaults(&min.cfg);
-	min.cfg.shortcutbank = shc_shortcutbank_new_default();
 	min.cfg.menus = mnu_menuinfo_new_default(&min);
 	btn_buttoninfo_new_default(&min, &min.cfg.buttons);
+	min.cfg.buttonlayout = btl_buttonlayout_new();
 	init_paths(&min.cfg);
 	min.cfg.style = stl_styleinfo_default();
 	typ_init(&min.cfg);
@@ -443,24 +510,28 @@ int main(int argc, char *argv[])
 	chd_initialize(&min);
 	min.que = que_initialize();
 
-	if(!(arg_flags & OPTF_NO_RC))
-		cfg_flags = cfg_load_config(&min);
-
 	min.ico = ico_initialize(&min);
 
+	if(!(opt_flags & OPT_NORC))
+		cfg_flags = cfg_load_config(&min);
 	if(!usr_init())
-		g_error("Couldn't initialize userinfo module");
-	if((min.cfg.mount.mode != MNT_NEVER) && (!mnt_init(&min)))
-		g_warning("Couldn't initialize mount data - automount won't work");
-
-	if(!(arg_flags & OPTF_NO_GTKRC))
+		g_warning(_("Couldn't initialize userinfo module - username resolving won't work"));
+	if(!(opt_flags & OPT_NOGTKRC))
 		load_gtk_rc(&min);
 
+	if((min.cfg.mount.mode != MNT_NEVER) && (!mnt_init(&min)))
+		g_warning(_("Couldn't initialize mount data - automounting won't work"));
+
+	if(!(opt_flags & OPT_NOFAM))
+		fam_initialize(&min);
+
 	if((box = build_gui(&min)) != NULL)
 	{
+		gchar	welcome[128];
+
 		dlg_group_set(min.gui->window->window);
+		dlg_position_set(min.cfg.dialogs.pos);
 		min.gui->win_del_evt = gtk_signal_connect(GTK_OBJECT(min.gui->window), "delete_event", GTK_SIGNAL_FUNC(evt_main_delete), &min);
-		gtk_signal_connect(GTK_OBJECT(min.gui->window), "destroy", GTK_SIGNAL_FUNC(evt_main_destroy), NULL);
 #if 0
 		gtk_signal_connect(GTK_OBJECT(min.gui->window), "key_press_event", GTK_SIGNAL_FUNC(evt_key_press), &min);
 #endif
@@ -468,24 +539,21 @@ int main(int argc, char *argv[])
 		init_cmd_options(&min.cfg);		
 
 		gtk_container_add(GTK_CONTAINER(min.gui->window), box);
-		dir[0] = (argc > 1) ? argv[1] : min.cfg.dp_format[0].def_path;
-		dir[1] = (argc > 2) ? argv[2] : min.cfg.dp_format[1].def_path;
+
+		dph_history_load(&min, min.gui->pane, sizeof min.gui->pane / sizeof *min.gui->pane);
 
 		gtk_window_set_policy(GTK_WINDOW(min.gui->window), TRUE, TRUE, FALSE);
 
 		csq_execute(&min, "DpRecenter");
 
-		set_icon(min.gui->window);
+		window_set_icon(min.gui->window);
 		win_window_show(min.gui->window);
 
 		min.gui->sig_left_sizealloc = gtk_signal_connect(GTK_OBJECT(min.gui->pane[0].vbox), "size_allocate",
 								GTK_SIGNAL_FUNC(evt_pane_sizealloc), &min);
 		gtk_signal_handler_block(GTK_OBJECT(min.gui->pane[0].vbox), min.gui->sig_left_sizealloc);
 
-		csq_execute(&min, "ActivateRight");
-		csq_execute_format(&min, "DirEnter 'dir=%s'", dir[1]);
-		csq_execute(&min, "ActivateLeft");
-		csq_execute_format(&min, "DirEnter 'dir=%s'", dir[0]);
+		enter_initial_dirs(&min, opt_arg_get("left", od, sizeof od / sizeof *od), opt_arg_get("right", od, sizeof od / sizeof *od));
 
 		g_snprintf(welcome, sizeof welcome, _("gentoo v%s by Emil Brink <emil@obsession.se>"), VERSION);
 		gtk_label_set_text(GTK_LABEL(min.gui->top), welcome);
@@ -494,6 +562,7 @@ int main(int argc, char *argv[])
 			csq_execute(&min, "About");
 		gtk_main();
 	}
+	fam_shutdown(&min);
 	mnt_shutdown(&min);
 
 	return EXIT_SUCCESS;
