@@ -13,12 +13,11 @@
 
 #include "config.h"
 
-#if !defined __alpha && !defined __NetBSD__
+#if !(defined __osf__ && defined __alpha__) && !defined __NetBSD__ && !defined __FreeBSD__ && !defined __sgi
 #define __EXTENSIONS__
 #define _POSIX_C_SOURCE	3	/* This is for Solaris. */
 #define	POSIX_C_SOURCE	3
-
-#endif				/* !__alpha */
+#endif				/* !__osf__ && !__alpha */
 
 #if defined __osf__ && defined __alpha__		/* On Tru64, this should bring in mknod(). */
 #define	_XOPEN_SOURCE_EXTENDED
@@ -27,7 +26,9 @@
 
 #define	_BSD_SOURCE					/* For MAXNAMLEN on Linux. */
 
+#include <dirent.h>
 #include <errno.h>
+#include <glob.h>
 #include <limits.h>
 #include <stdio.h>
 #include <string.h>
@@ -35,22 +36,31 @@
 #include <sys/mman.h>
 #include <sys/stat.h>
 #include <sys/time.h>
-#include <dirent.h>
-#include <unistd.h>
 #include <regex.h>
+#include <time.h>
+#include <unistd.h>
 
-#if defined __linux
-#include <linux/limits.h>
-#include <sys/vfs.h>
-#elif defined __OpenBSD__ || defined __FreeBSD__ || defined __NetBSD__
+#if defined HAVE_STRUCT_STATVFS
+#include <sys/statvfs.h>
+#elif defined HAVE_STRUCT_STATFS
+#include <sys/statfs.h>
+#endif
+
+/* The various BSDs seem to have a lot in common. Let's try and use that,
+** and save some typing in various tests below. This should probably all
+** be replaced by proper Autoconfed stuff at some point.
+*/
+#if defined __OpenBSD__ || defined __FreeBSD__ || defined __NetBSD__
+#define GENTOO_ON_BSD
+#endif
+
+/* BSD-specific mounting stuff. */
+#if defined GENTOO_ON_BSD
 #include <sys/param.h>
 #include <sys/mount.h>
-#else
-#include <sys/statvfs.h>
 #endif
 
 #define	GTK_DISABLE_COMPAT_H		/* No GTK+ 1.0.6-compatibility required. */
-
 #include <gtk/gtk.h>
 
 /* Not all systems' <mman.h> seem to define this. */
@@ -58,32 +68,35 @@
 #define	MAP_FAILED	((void *) -1)
 #endif
 
+#if !defined MAXNAMLEN			/* Still no MAXNAMLEN? Then make one up. */
+#define	MAXNAMLEN	255
+#endif
+
 /* ----------------------------------------------------------------------------------------- */
 
 #define	RCNAME	".gentoorc"
 
 /* ----------------------------------------------------------------------------------------- */
 
-/* Internationalization is still disabled. I haven't gotten around to
-** integrating it with the still new autoconf build process. Help is
-** welcome.
+/* Sometimes, this does good stuff. I hope. That's about all you can do with GNU tools,
+** since actually understanding them requires way more time than I'm willing to spend.
 */
-#undef	ENABLE_NLS
+#include "gnu-gettext.h"
 
 #if defined ENABLE_NLS
+#include <locale.h>
 /* These two are used to mark translatable strings in the source code.
 ** _() is used in contexts where C allows a function call, N_() where
 ** it does not. Strings marked with N_() still need to be passed to _()
 ** before being shown to the user.
 */
-#include <libintl.h>
 #define	_(str)	gettext(str)
 #define	N_(str)	(str)
 #else
 /* Provide transparent definitions if NLS is disabled. */
-#define	_(str)	(str)
-#define	N_(str)	(str)
-#endif
+#define	_(str)	str
+#define	N_(str)	str
+#endif		/* ENABLE_NLS */
 
 /* ----------------------------------------------------------------------------------------- */
 
@@ -95,6 +108,7 @@ typedef struct CmdSeq		CmdSeq;
 ** that are needed below. This is a good point to include them, since MainInfo has been
 ** introduced (above).
 */
+#include "buttonlayout.h"
 #include "keyboard.h"
 #include "controls.h"
 #include "iconutil.h"
@@ -148,7 +162,7 @@ typedef struct {			/* Used to map stat() info onto names. */
 #define	DP_DATEFMT_SIZE	(32)
 #define	DP_MAX_COLUMNS	(32)
 
-typedef enum {	DPC_NAME, DPC_SIZE, DPC_IQSIZE, DPC_BLOCKS, DPC_BLOCKSIZE, DPC_MODENUM, DPC_MODESTR,
+typedef enum {	DPC_NAME, DPC_SIZE, DPC_BLOCKS, DPC_BLOCKSIZE, DPC_MODENUM, DPC_MODESTR,
 		DPC_NLINK, DPC_UIDNUM, DPC_UIDSTR, DPC_GIDNUM, DPC_GIDSTR, DPC_DEVICE, DPC_DEVMAJ, DPC_DEVMIN,
 		DPC_ATIME, DPC_MTIME, DPC_CTIME,
 		DPC_TYPE, DPC_ICON,
@@ -159,14 +173,17 @@ typedef struct {
 	gint	show_linkname : 1;		/* Append " -> destination" on symbolic links? */
 } DC_Name;
 
-typedef struct {
-	gchar	format[DP_FORMAT_SIZE];		/* Formatting string for size. */
-	gint	dir_show_fs_size : 1;		/* Show filesystem size for directories? */
-} DC_Size;
+typedef enum {  DPSIZE_NATIVE = 0, DPSIZE_BYTES, DPSIZE_KILOBYTES, DPSIZE_MEGABYTES, DPSIZE_GIGABYTES,
+		DPSIZE_SMART  = 32 } DPSizeType;
 
 typedef struct {
-	gint	dir_show_fs_size : 1;		/* The same as for regular size, of course. */
-} DC_IQSz;
+	DPSizeType	type;
+	gint		ticks : 1;
+	gchar		tick;
+	gint		digits;
+	gchar		dformat[DP_FORMAT_SIZE];	/* Hidden from user, not saved, built by config code. */
+	gint		dir_show_fs_size : 1;		/* Show filesystem size for directories? */
+} DC_Size;
 
 typedef struct {
 	gchar	format[DP_FORMAT_SIZE];		/* General numerical formatter (for size, mode, uid, etc). */
@@ -180,7 +197,6 @@ typedef union {
 	DC_Name	name;
 	DC_Size	size;
 	DC_Fmt	blocks, blocksize;
-	DC_IQSz	iqsize;
 	DC_Fmt	mode, nlink, uidnum, gidnum, device, devmaj, devmin;
 	DC_Time	a_time, m_time, c_time;		/* Accessed, modified & changed, right? */
 } DpCExtra;
@@ -198,9 +214,9 @@ typedef enum { DPS_DIRS_FIRST = 0, DPS_DIRS_LAST, DPS_DIRS_MIXED } SortMode;
 typedef struct {
 	DPContent	content;		/* The content type we wish to sort on. */
 	SortMode	mode;			/* Controls placement of directories. */
-	gint		invert : 1;		/* If set, we sort backwards (Z-A). */
-	gint		nocase : 1;		/* Set for case-insensitive string comparisons. Lame! */
-	gint		numerical : 1;		/* Special numerical mode (way cool)? */
+	gboolean	invert;			/* If set, we sort backwards (Z-A). */
+	gboolean	nocase;			/* Set for case-insensitive string comparisons. Lame! */
+	gboolean	numerical;		/* Special numerical mode (way cool)? */
 } DPSort;
 
 typedef enum { SBP_IGNORE = 0, SBP_LEFT, SBP_RIGHT } SBarPos;
@@ -227,14 +243,20 @@ typedef struct {
 	GdkColor	color[DPCOL_NUM];
 } DPColors;
 
-typedef enum { DPSPLIT_FREE = 0, DPSPLIT_RATIO, DPSPLIT_ABS_LEFT, DPSPLIT_ABS_RIGHT } DpSplit;
+typedef enum { DPORIENT_HORIZ = 0, DPORIENT_VERT } DpOrient;
+typedef enum { DPSPLIT_FREE = 0,   DPSPLIT_RATIO, DPSPLIT_ABS_LEFT, DPSPLIT_ABS_RIGHT } DpSplit;
 
 /* Allow user to control how gentoo should allocate window space between the panes. */
 typedef struct {
-	DpSplit	mode;
-	gdouble	value;		/* Parameter interpreted depending on mode. */
+	DpOrient	orientation;
+	DpSplit		mode;
+	gdouble		value;		/* Parameter interpreted depending on mode. */
 } DPPaning;
 
+typedef struct {
+	gboolean	save;			/* Save history lists on exit? */
+} DPHistory;
+
 /* ----------------------------------------------------------------------------------------- */
 
 /* General command row flags. */
@@ -285,38 +307,10 @@ typedef struct {			/* Information about commands lives here. */
 #define	BTN_LABEL_SIZE		(32)
 #define	BTN_TOOLTIP_SIZE	(64)	/* Arbitrary, as always. */
 
-typedef enum { SHP_CURRENT = 0, SHP_OTHER, SHP_LEFT, SHP_RIGHT } ShcPane;
-
-typedef struct {			/* A shortcut is a button for rapid access to a path. Very Opus. */
-	gchar	label[BTN_LABEL_SIZE];		/* The label to display on the button, of course. */
-	gchar	path[PATH_MAX];			/* Path associated with this shortcut button. */
-	ShcPane	pane;				/* Which pane should the shortcut be read into? */
-	gint	show_tooltip : 1;		/* Should tooltip (with the path) be shown? */
-} Shortcut;
-
-typedef enum { SSM_NONE = 0, SSM_SIMPLE, SSM_PANED } ShbSep;
-
-typedef struct {			/* Main "container" datatype for the shortcuts. Keeps it tidy. */
-	GList	*shortcut;			/* Just a simple glib list of Shortcuts (above). */
-	ShbSep	sep_mode;			/* Separator mode. */
-	gint	visible : 1;			/* Should the shortcuts be visible at all? */
-	gint	right : 1;			/* To the right of the buttons? */
-} ShortcutBank;
-
 typedef struct {
 	GList	*sheets;			/* List of button sheets. */
 } ButtonInfo;
 
-/*typedef struct {
-	gchar	*text;
-	gchar	*command;
-} MenuItem;
-
-typedef struct {
-	MenuItem	*menuitem;
-} MenuRoot;
-*/
-
 typedef struct {			/* Options for overwrite-confirmation dialog module. */
 	gint	show_info : 1;			/* Show info (sizes & dates) for conflicting files? */
 	gchar	datefmt[DP_DATEFMT_SIZE];	/* How to format dates? */
@@ -341,11 +335,9 @@ typedef struct {				/* Miscellanous paths. */
 
 /* This controls how often we do mounts, if ever. The levels are:
 ** MNT_NEVER	Disable all mounting.
-** MNT_RARELY	Do mounting only "rarely". Currently means that mounting is done
-**		when a dir is double-clicked, and when a shortcut is accessed.
-** MNT_OFTEN	Does a mount attempt on every directory scan. Might be slow.
+** MNT_ALWAYS	Does a mount attempt on every directory scan. Might be slow.
 */
-typedef enum { MNT_NEVER = 0, MNT_RARELY, MNT_OFTEN }	MntMode;
+typedef enum { MNT_NEVER = 0, MNT_ALWAYS }	MntMode;
 
 typedef struct {			/* Configuration info for automounting. */
 	MntMode	mode;				/* Global mounting mode. [MNT_NEVER] */
@@ -356,8 +348,13 @@ typedef struct {			/* Configuration info for automounting. */
 	gchar	cmd_umount[MAXNAMLEN];		/* This command is used for unmounting. ["/bin/umount"] */
 } MountInfo;
 
+typedef struct {			/* Configuration information for dialogs. */
+	GtkWindowPosition	pos;		/* How should dialogs be positioned? */
+} DialogInfo;
+
 typedef struct {
 	gboolean	dialog;			/* Use dialog for errors? */
+	gboolean	beep;			/* Do a gdk_beep() on error? */
 } ErrInfo;
 
 #define	CFLG_CHANGED	(1<<0)		/* Config has changed. Set on "OK", cleared by cfg_save_all(). */
@@ -369,14 +366,16 @@ typedef struct {			/* Holds all configuration info. */
 	DPFormat	dp_format[2];
 	DPColors	dp_colors;		/* Global color information used to render dirpanes. */
 	DPPaning	dp_paning;		/* Controls how the paned view acts. */
+	DPHistory	dp_history;		/* History list option. */
 	OptOverwrite	opt_overwrite;
-	ShortcutBank	*shortcutbank;
 	MenuInfo	*menus;
 	ButtonInfo	buttons;
+	ButtonLayout	*buttonlayout;
 	CmdInfo		commands;
 	PathInfo	path;
 	MountInfo	mount;
 	WinInfo		*wininfo;
+	DialogInfo	dialogs;
 	CtrlInfo	*ctrlinfo;
 	ErrInfo		errors;
 	gboolean	(*dir_filter)(const gchar *name);
@@ -393,6 +392,14 @@ struct MainInfo {				/* gentoo's single most central data structure. */
 
 /* -- Directory content data structures -- */
 
+/* This is used for tab-completion of paths. Easy. */
+typedef struct {
+	glob_t	buf;		/* Result of a glob() call from original Tab press. */
+	GList	*list;		/* List items referencing into the above. */
+	GList	*iter;		/* Iterator for current position in list. */
+	guint	change_sig;	/* Signal for change handler on path entry widget. */
+} PathComplete;
+
 /* These macros are used to access fields of a DirRow structure. */
 #define	DP_ROW_NAME(r)		((r)->dr_name)
 #define	DP_ROW_LSTAT(r)		((r)->dr_lstat)
@@ -427,12 +434,12 @@ struct MainInfo {				/* gentoo's single most central data structure. */
 #define	DPRF_HAS_SIZE		(1<<0)		/* Set when a row has a "real" size (not set for directories unless GetSize:d). */
 
 typedef struct {			/* Representation of a single line in a directory (a file, typically). */
-	gchar		*dr_name;		/* Name, points into big block in DirContents. */
+	const gchar	*dr_name;		/* Name, points into big block in DirContents. */
 	struct stat	dr_lstat;		/* From a call to lstat(). */
-	gchar		*dr_linkname;		/* Contents of symbolic link (if S_ISLNK() true on dr_lstat). Else NULL. */
+	const gchar	*dr_linkname;		/* Contents of symbolic link (if S_ISLNK() true on dr_lstat). Else NULL. */
 	struct stat	*dr_stat;		/* If link, this holds link target stats. Else, it points at dr_lstat. */
 	guint32		dr_flags;		/* Misc. flags. */
-	FType		*dr_type;		/* Type information. */
+	const FType	*dr_type;		/* Type information. */
 } DirRow;
 
 typedef struct {
@@ -442,16 +449,16 @@ typedef struct {
 
 typedef struct {			/* Some trivial file system information. Updated on rescan. */
 	gboolean	valid;			/* Set if the structure's contents are valid. */
-#if defined __linux || defined __OpenBSD__ || defined __FreeBSD__ || defined __NetBSD__
-	struct statfs	stat;			/* Holds information about filesystem. */
-#else
-	struct statvfs	stat;
+#if defined HAVE_STRUCT_STATVFS
+	struct statvfs	stat;			/* Holds information about filesystem. */
+#elif defined HAVE_STRUCT_STATFS || defined GENTOO_ON_BSD
+	struct statfs	stat;
 #endif
 } FsInfo;
 
 typedef struct {
 	gchar		path[PATH_MAX];		/* Contents apply to this path. */
-	guint		num_rows;		/* Number of valid lines. */
+	guint		num_rows;		/* Number of valid rows. */
 	guint		tot_dirs, tot_files;	/* Total number of entries in directory. */
 	guint64		tot_bytes, tot_blocks;	/* Sum of all sizes (in bytes & blocks). */
 	DirRow		*row;			/* A bunch of rows. */
@@ -466,7 +473,8 @@ struct DirPane {
 	MainInfo	*main;		/* Handy to have around. */
 	guint		index;		/* Index of *this* pane, in the grand scheme of things. */
 	GtkWidget	*vbox;
-	GtkWidget	*path;		/* As of gentoo 0.11.9, this is again a combo, not an entry. */
+	GtkWidget	*path;		/* Combo widget showing path, and path history. */
+	PathComplete	complete;	/* Information for completing the path. */
 	GtkWidget	*hide;		/* A toggle button showing (and controlling) hide status. */
 	GtkWidget	*scwin;		/* Scrolling window for clist to live in. */
 	GtkCList	*list;		/* The actual clist that shows all stuff. */
@@ -475,7 +483,6 @@ struct DirPane {
 	GtkWidget	*menu_action;	/* The menu containing the intersection of all available actions on the selection. */
 	DirContents	dir;		/* Contents of directory (list of names etc). */
 	DirHistory	*hist;		/* Historic data about previously visited directories. */
-	guint32		flags;		/* Various fun flags. */
 	gint		last_row;	/* The last clicked row. */
 	gint		last_row2;	/* The second last clicked row. */
 	gint		dbclk_row;	/* The row that was double clicked, or -1. */
@@ -492,7 +499,7 @@ struct GuiInfo {
 	KbdContext	*kbd_ctx;		/* Keyboard context, for shortcuts. */
 	GtkWidget	*vbox;			/* Vbox that contains entire GUI. */
 	GtkWidget	*top;			/* A label showing status and stuff. Handy to focus. */
-	GtkWidget	*hpane;			/* Horizontal pane holding panes. */
+	GtkWidget	*panes;			/* GtkPaned widget holding DirPanes. */
 	GtkWidget	*middle;		/* A box holding entire middle part. */
 	GtkWidget	*bottom;		/* Bottom part of GUI. */
 	DirPane		pane[2];
@@ -508,5 +515,5 @@ struct GuiInfo {
 /* These are defined in the main "gentoo.c" module, but really shouldn't be. Until they
 ** get a module of their own, they need to be prototyped like this. :(
 */
-void	rebuild_middle(MainInfo *min);
-void	rebuild_bottom(MainInfo *min);
+extern void	rebuild_middle(MainInfo *min);
+extern void	rebuild_bottom(MainInfo *min);
