@@ -32,46 +32,28 @@ GtkWidget * gui_arrow_button_new(GtkArrowType at)
 {
 	GtkWidget	*but, *arr;
 
-	if((but = gtk_button_new()) != NULL)
-	{
-		if((arr = gtk_arrow_new(at, GTK_SHADOW_OUT)) != NULL)
-		{
-			gtk_container_add(GTK_CONTAINER(but), arr);
-			gtk_widget_show(arr);
-			return but;
-		}
-		gtk_widget_destroy(but);
-	}
-	return NULL;
+	but = gtk_button_new();
+	arr = gtk_arrow_new(at, GTK_SHADOW_OUT);
+	gtk_container_add(GTK_CONTAINER(but), arr);
+
+	return but;
 }
 
-/* 1999-04-26 -	Create a button with a "details" icon on it. This is typically a magnification
+/* 1999-04-26 -	Create a button with a "details" icon on it. This is typically a magnifying
 **		class. This button is a graphical replacement for a button labeled "...".
 */
 GtkWidget * gui_details_button_new(GdkWindow *win)
 {
-	GtkWidget	*btn;
+	GtkWidget	*btn, *pmap;
 	GdkPixmap	*pix;
 	GdkBitmap	*msk;
 
-	if((btn = gtk_button_new()) != NULL)
-	{
-		if((pix = gdk_pixmap_create_from_xpm_d(win, &msk, NULL, icon_details_xpm)) != NULL)
-		{
-			GtkWidget	*pmap;
-
-			if((pmap = gtk_pixmap_new(pix, msk)) != NULL)
-			{
-				gtk_container_add(GTK_CONTAINER(btn), pmap);
-				gtk_widget_show(pmap);
-
-				return btn;
-			}
-			gdk_pixmap_unref(pix);
-			gdk_bitmap_unref(msk);
-		}
-	}
-	return NULL;
+	btn = gtk_button_new();
+	pix = gdk_pixmap_create_from_xpm_d(win, &msk, NULL, icon_details_xpm);
+	pmap = gtk_pixmap_new(pix, msk);
+	gtk_container_add(GTK_CONTAINER(btn), pmap);
+	gtk_widget_show(pmap);
+	return btn;
 }
 
 /* ----------------------------------------------------------------------------------------- */
@@ -96,7 +78,7 @@ GSList * gui_radio_group_new(guint num, const gchar *label[], GtkWidget *but[])
 /* ----------------------------------------------------------------------------------------- */
 
 /* 1998-09-05 -	This gets called by the dialog module if the user hits the button. */
-static void done(gint button, gpointer user)
+static void callback_done(gint button, gpointer user)
 {
 	PBar	*pbar = user;
 
@@ -115,19 +97,16 @@ gpointer gui_progress_begin(const gchar *body, const gchar *button)
 {
 	static PBar	pbar;
 
-	pbar.vbox = gtk_vbox_new(FALSE, 0);
+	pbar.vbox  = gtk_vbox_new(FALSE, 0);
 	pbar.label = gtk_label_new(body);
 	gtk_box_pack_start(GTK_BOX(pbar.vbox), pbar.label, TRUE, TRUE, 0);
-	gtk_widget_show(pbar.label);
 	pbar.obj = gtk_label_new("");
 	gtk_box_pack_start(GTK_BOX(pbar.vbox), pbar.obj, TRUE, TRUE, 0);
-	gtk_widget_show(pbar.obj);
 	pbar.bar = gtk_progress_bar_new();
 	gtk_box_pack_start(GTK_BOX(pbar.vbox), pbar.bar, TRUE, TRUE, 5);
-	gtk_widget_show(pbar.bar);
 	pbar.terminate = FALSE;
 
-	pbar.dlg = dlg_dialog_async_new(pbar.vbox, _("Progress"), button, done, &pbar);
+	pbar.dlg = dlg_dialog_async_new(pbar.vbox, _("Progress"), button, callback_done, &pbar);
 
 	return &pbar;
 }
@@ -139,7 +118,7 @@ gpointer gui_progress_begin(const gchar *body, const gchar *button)
 */
 gboolean gui_progress_update(gpointer anchor, gfloat value, const gchar *obj)
 {
-	PBar	*pbar = (PBar *) anchor;
+	PBar	*pbar = anchor;
 
 	if(!pbar->terminate)
 	{
@@ -150,7 +129,6 @@ gboolean gui_progress_update(gpointer anchor, gfloat value, const gchar *obj)
 		while(gtk_events_pending())
 			gtk_main_iteration();
 	}
-
 	return pbar->terminate;
 }
 
@@ -159,7 +137,7 @@ gboolean gui_progress_update(gpointer anchor, gfloat value, const gchar *obj)
 */
 void gui_progress_end(gpointer anchor)
 {
-	PBar	*pbar = (PBar *) anchor;
+	PBar	*pbar = anchor;
 
 	if(!pbar->terminate)
 		dlg_dialog_async_close(pbar->dlg, -1);
@@ -176,31 +154,22 @@ void gui_progress_end(gpointer anchor)
 **		determine which item was selected, each item will have its index (from 0)
 **		set as object's user data. Not pretty, but still simple and useful.
 */
-GtkWidget * gui_build_menu(const gchar *text[], GtkSignalFunc func, gpointer data)
+GtkWidget * gui_build_menu(const gchar *text[], GtkSignalFunc func, gpointer user)
 {
 	GtkWidget	*menu, *item;
 	guint		i;
 
-	if((menu = gtk_menu_new()) != NULL)
+	menu = gtk_menu_new();
+	for(i = 0; text[i] != NULL; i++)
 	{
-		for(i = 0; text[i] != NULL; i++)
+		item = gtk_menu_item_new_with_label(_(text[i]));
+		if(func != NULL)
 		{
-			if((item = gtk_menu_item_new_with_label(_(text[i]))) != NULL)
-			{
-				if(func != NULL)
-				{
-					gtk_object_set_user_data(GTK_OBJECT(item), GUINT_TO_POINTER(i));
-					gtk_signal_connect(GTK_OBJECT(item), "activate", func, data);
-				}
-				gtk_menu_append(GTK_MENU(menu), item);
-				gtk_widget_show(item);
-			}
-			else
-			{
-				gtk_widget_destroy(menu);
-				return NULL;
-			}
+			gtk_object_set_user_data(GTK_OBJECT(item), GUINT_TO_POINTER(i));
+			gtk_signal_connect(GTK_OBJECT(item), "activate", func, user);
 		}
+		gtk_menu_append(GTK_MENU(menu), item);
+		gtk_widget_show(item);
 	}
 	return menu;
 }
