@@ -43,7 +43,7 @@ void ovw_overwrite_begin(MainInfo *min, const gchar *fmt, guint32 flags)
 		ovw_info.fmt = fmt;
 		ovw_info.flags = flags;
 		ovw_info.label = gtk_label_new("");
-		ovw_info.dlg = dlg_dialog_sync_new(ovw_info.label, _("Please Confirm"), _("OK|All|Skip|Skip All|Cancel"));
+		ovw_info.dlg = dlg_dialog_sync_new(ovw_info.label, _("Please Confirm"), _("_OK|A_ll|_Skip|Skip _All|_Cancel"));
 	}
 	else
 		fprintf(stderr, "OVERWRITE: Mismatched call (level=%d)!\n", ovw_info.level);
@@ -68,8 +68,8 @@ static void get_info(MainInfo *min, gchar *buf, gsize buf_size, const gchar *old
 
 	if((stat(old_file, &o_st) == 0) && (stat(new_file, &n_st) == 0))
 	{
-		strftime(o_date, sizeof o_date, (const char *) min->cfg.opt_overwrite.datefmt, get_date(&o_st));
-		strftime(n_date, sizeof n_date, (const char *) min->cfg.opt_overwrite.datefmt, get_date(&n_st));
+		strftime(o_date, sizeof o_date, min->cfg.opt_overwrite.datefmt, get_date(&o_st));
+		strftime(n_date, sizeof n_date, min->cfg.opt_overwrite.datefmt, get_date(&n_st));
 		g_snprintf(buf, (gulong) buf_size, _("\nOld: %u bytes, changed on %s.\n"
 						   "New: %u bytes, changed on %s."),
 						   (guint) o_st.st_size, o_date,
@@ -84,9 +84,9 @@ static gboolean check_recursion(const gchar *f_new, const gchar *f_old)
 {
 	if(f_new != NULL)
 	{
-		gint	len = strlen(f_new);
+		gint	len_old = strlen(f_old), len_new = strlen(f_new);
 
-		if(f_old[len] == G_DIR_SEPARATOR && strncmp(f_old, f_new, len) == 0)
+		if(len_old >= len_new && f_old[len_new] == G_DIR_SEPARATOR && strncmp(f_old, f_new, len_new) == 0)
 			return TRUE;
 	}
 	return FALSE;
