@@ -182,7 +182,10 @@ static void styleinfo_update(StyleInfo *si)
 	if(si != NULL)
 	{
 		if(si->freeze_count == 0)
+		{
 			g_node_children_foreach(si->styles, G_TRAVERSE_ALL, style_update, si->styles->data);
+			si->update_pending = FALSE;
+		}
 		else
 			si->update_pending = TRUE;
 	}
@@ -487,10 +490,9 @@ GList * stl_styleinfo_style_get_children(StyleInfo *si, Style *stl, gboolean inc
 			g_node_traverse(node, G_PRE_ORDER, G_TRAVERSE_ALL, -1, get_child, &list);
 			if(!include_root)
 			{
-				GList	*first;
+				GList	*first = list;
 
-				first = list;
-				list = g_list_remove_link(list, first);	/* Remove <stl> from child list. */
+				list = g_list_remove_link(list, list);	/* Remove head of child list. */
 				g_list_free(first);
 			}
 			return list;
@@ -729,11 +731,7 @@ void stl_styleinfo_destroy(StyleInfo *si)
 	{
 		stl_styleinfo_freeze(si);	/* There's no point in updating inherited stuff while destroying. */
 		if(si->styles != NULL)
-		{
-/*			fprintf(stderr, "destroying styleinfo\n");*/
 			stl_style_destroy(si->styles->data, TRUE);
-/*			fprintf(stderr, " done\n");*/
-		}		
 		g_free(si);
 	}
 }
@@ -869,13 +867,13 @@ Style * stl_style_new_unique_name(StyleInfo *si)
 	if(si != NULL)
 	{
 		gchar	buf[STL_STYLE_NAME_SIZE];
-		guint	i = 0;
+		guint	i = 0U;
 
 		do
 		{
 			g_snprintf(buf, sizeof buf, i == 0 ? _("New Style") : _("New Style %u"), i + 1);
 			i++;
-		} while((stl_styleinfo_style_find(si, buf) != NULL) && (i < 10000));	/* This might fail. Big deal. */
+		} while((stl_styleinfo_style_find(si, buf) != NULL) && (i < 10000U));	/* This might fail. Big deal. */
 
 		return stl_style_new(buf);
 	}
@@ -969,34 +967,6 @@ const gchar * stl_style_get_name(Style *stl)
 	return NULL;
 }
 
-/* 1999-05-20 -	Get the full name of <stl>. A full name contains a leading "path" of
-**		parent styles, separated with slashes. Example: "Root/Data/Images/GIF".
-**		It is the caller's responsibility to call g_string_free() on the returned
-**		string when done with it, of course.
-** BUG BUG BUG	This is far from efficent. Beware.
-*/
-GString * stl_style_get_name_full(Style *stl)
-{
-	if(stl != NULL)
-	{
-		GNode	*node;
-
-		if((node = styleinfo_style_get_node(stl->styleinfo, stl)) != NULL)
-		{
-			GString	*str;
-
-			str = g_string_new(stl->name);
-			for(node = node->parent; node != NULL; node = node->parent)
-			{
-				g_string_prepend_c(str, '/');
-				g_string_prepend(str, ((Style *) node->data)->name);
-			}
-			return str;
-		}
-	}
-	return NULL;
-}
-
 /* 1999-06-12 -	Get the branch that begins with the root style and ends with <stl>. */
 static GSList * style_get_branch(Style *stl)
 {
@@ -1015,8 +985,7 @@ static GSList * style_get_branch(Style *stl)
 
 /* 1999-05-20 -	Compare hierarchial names of <stla> and <stlb>. Used by dirpane module,
 **		to sort on styles. Comparison is done so that styles with common parents
-**		are grouped. Does NOT use a simple pair of stl_style_get_name_full() calls,
-**		since that tends to kill performance completely.
+**		are grouped.
 */
 gint stl_style_compare_hierarchy(Style *stla, Style *stlb)
 {
@@ -1063,7 +1032,7 @@ static StlPVal * value_new(StlPType type)
 			val->data.color.red   = 0U;
 			val->data.color.green = 0U;
 			val->data.color.blue  = 0U;
-			val->data.color.pixel = 0LU;
+			val->data.color.pixel = 0UL;
 			break;
 		case SPT_ICON:
 			val->data.icon[0] = '\0';
@@ -1103,7 +1072,7 @@ static StlProp * style_get_property(Style *stl, const gchar *name, StlPType type
 	if((pr = g_hash_table_lookup(stl->properties, name)) != NULL)
 	{
 		if(pr->value->type != type)
-			fprintf(stderr, "STYLES: Style '%s' has wrong-type property '%s' (%u, not %u)\n", stl->name, name, pr->value->type, type);
+			fprintf(stderr, "STYLES: Style '%s' has wrong-type property '%s' (%d, not %d)\n", stl->name, name, pr->value->type, type);
 		if(pr->override == TRUE)
 			return pr;
 	}
