@@ -104,7 +104,7 @@ static void really_destroy(GtkWidget *wid)
 }
 
 /* 1998-05-19 -	This simplistic thing causes the textviewing window to close when needed. */
-static gint evt_delete(GtkWidget *wid, GdkEvent *evt, gpointer data)
+static gint evt_delete(GtkWidget *wid, GdkEvent *evt, gpointer user)
 {
 	really_destroy(wid);
 
@@ -125,7 +125,7 @@ static gint evt_keypress(GtkWidget *wid, GdkEventKey *event, gpointer user)
 /* 1999-03-02 -	Users of this module no longer have access to the GtkText widget, so
 **		this function is needed for them to be able to actually display text.
 */
-void txv_put_text(GtkWidget *wid, const char *text, int length)
+void txv_put_text(GtkWidget *wid, const gchar *text, gint length)
 {
 	TxvInfo	*txi;
 
@@ -147,7 +147,7 @@ void txv_put_text(GtkWidget *wid, const char *text, int length)
 ** 1999-03-02 -	New interface, renamed.
 ** 1999-04-06 -	Added the <buf_size> argument, which is used when mmap() isn't.
 */
-int txv_load_text(GtkWidget *wid, const char *name, gboolean use_mmap, gsize buf_size)
+gint txv_load_text(GtkWidget *wid, const gchar *name, gboolean use_mmap, gsize buf_size)
 {
 	gchar	*buf;
 	gint	fd, ret = 0;
@@ -181,7 +181,7 @@ int txv_load_text(GtkWidget *wid, const char *name, gboolean use_mmap, gsize buf
 		close(fd);
 	}
 	if(errno != 0)
-		err_set(txi->min, errno, _("File reading"), (char *) name);
+		err_set(txi->min, errno, _("File reading"), (gchar *) name);
 
 	return ret;
 }
@@ -191,17 +191,17 @@ int txv_load_text(GtkWidget *wid, const char *name, gboolean use_mmap, gsize buf
 /* 1998-12-18 -	Build "groups" of actual hex data, from <data> and <bytes> bytes on. */
 static void build_hex_group(const guchar *data, gchar *group, gsize bytes)
 {
-	gchar	*hex = "0123456789ABCDEF", *ptr = group;
-	guint8	here;
-	gsize	i;
+	const static gchar	*hex = "0123456789ABCDEF";
+	guint8			here;
+	gsize			i;
 
 	for(i = 0; i < bytes; i++)
 	{
 		if(i && !(i & 3))
-			ptr++;
+			group++;
 		here = (guint8) *data++;
-		*ptr++ = hex[here >> 4];
-		*ptr++ = hex[here & 15];
+		*group++ = hex[here >> 4];
+		*group++ = hex[here & 15];
 	}
 }
 
@@ -211,7 +211,7 @@ static void build_hex_text(const guchar *data, gchar *text, gsize bytes)
 	gsize	i;
 
 	for(i = 0; i < bytes; i++, data++)
-		*text++ = isprint((int) *data) ? *data : HEX_UNPRINTABLE;
+		*text++ = isprint((guchar) *data) ? *data : HEX_UNPRINTABLE;
 }
 
 /* 1998-12-18 -	Add contents of file <name> into GtkText widget <wid>, as a hex dump.
@@ -225,12 +225,12 @@ static void build_hex_text(const guchar *data, gchar *text, gsize bytes)
 **		Now we do plenty of small read()s instead, which I'm sure is slower, but at
 **		least doesn't waste memory.
 */
-int txv_load_text_hex(GtkWidget *win, const gchar *name)
+gint txv_load_text_hex(GtkWidget *win, const gchar *name)
 {
 	gchar		line[8 + 1 + (HEX_ROW_SIZE / 4) * 9 + 1 + HEX_ROW_SIZE + 1 + 1],
 			group[9 * (HEX_ROW_SIZE / 4) + 1], text[HEX_ROW_SIZE + 1];
 	guchar		buf[HEX_ROW_SIZE];
-	gint		fd;
+	gint		fd, len;
 	gsize		chunk, offset = 0;
 	TxvInfo		*txi;
 
@@ -239,13 +239,12 @@ int txv_load_text_hex(GtkWidget *win, const gchar *name)
 
 	txv_set_label(win, name);
 
-	err_clear(txi->min);
-
 	memset(group, ' ', sizeof group - 1);
 	group[sizeof group - 1] = '\0';
 	memset(text, ' ', sizeof text - 1);
 	text[sizeof text - 1]   = '\0';
 
+	err_clear(txi->min);
 	gtk_text_freeze(GTK_TEXT(txi->text));
 	if((fd = open(name, O_RDONLY)) >= 0)
 	{
@@ -258,8 +257,8 @@ int txv_load_text_hex(GtkWidget *win, const gchar *name)
 			}
 			build_hex_group(buf, group, chunk);
 			build_hex_text(buf, text, chunk);
-			g_snprintf(line, sizeof line, "%08lX %s %s\n", (unsigned long) offset, group, text);
-			gtk_text_insert(GTK_TEXT(txi->text), NULL, NULL, NULL, line, strlen(line));
+			len = g_snprintf(line, sizeof line, "%08lX %s %s\n", (unsigned long) offset, group, text);
+			gtk_text_insert(GTK_TEXT(txi->text), NULL, NULL, NULL, line, len);
 			offset += chunk;
 		}
 		close(fd);
@@ -267,8 +266,7 @@ int txv_load_text_hex(GtkWidget *win, const gchar *name)
 	gtk_text_thaw(GTK_TEXT(txi->text));
 
 	if(errno != 0)
-		err_set(txi->min, errno, _("File reading"), (char *) name);
-
+		err_set(txi->min, errno, _("File reading"), (gchar *) name);
 	return errno != 0;
 }
 
@@ -290,14 +288,13 @@ static void reset_search(TxvInfo *txi)
 **		should probably also include the number of lines available, but that is a
 **		rather fuzzy quantity, so I'll just leave that for now.
 */
-static gint evt_pos_changed(GtkWidget *wid, gpointer data)
+static gint evt_pos_changed(GtkAdjustment *adj, gpointer user)
 {
-	TxvInfo		*txi = (TxvInfo *) data;
-	GtkAdjustment	*adj = (GtkAdjustment *) wid;
+	TxvInfo		*txi = user;
 	GtkStyle	*stl;
 	gchar		buf[64];
-	gfloat		p;
 	gint		fh = 10;
+	gfloat		p;
 
 	if((stl = gtk_widget_get_style(txi->text)) != NULL)
 		fh = stl->font->ascent + stl->font->descent;
@@ -310,7 +307,7 @@ static gint evt_pos_changed(GtkWidget *wid, gpointer data)
 	}
 	else
 		p = 0.0f;
-	g_snprintf(buf, sizeof buf, _("Line %d (%.0f%%)"), (int) adj->value / fh, p);
+	g_snprintf(buf, sizeof buf, _("Line %d (%.0f%%)"), (gint) adj->value / fh, p);
 	gtk_label_set_text(GTK_LABEL(txi->bplabel), buf);
 
 	return TRUE;
@@ -319,9 +316,9 @@ static gint evt_pos_changed(GtkWidget *wid, gpointer data)
 /* 1999-01-05 -	User clicked on one of the four basic movement buttons. Determine which, and
 **		make the text move accordingly.
 */
-static gint evt_move_clicked(GtkWidget *wid, gpointer data)
+static gint evt_move_clicked(GtkWidget *wid, gpointer user)
 {
-	TxvInfo		*txi = (TxvInfo *) data;
+	TxvInfo		*txi = user;
 	GtkAdjustment	*adj;
 	GtkStyle	*stl;
 	gint		fh = 10;
@@ -330,7 +327,7 @@ static gint evt_move_clicked(GtkWidget *wid, gpointer data)
 		fh = stl->font->ascent + stl->font->descent;
 	if((adj = gtk_scrolled_window_get_vadjustment(GTK_SCROLLED_WINDOW(txi->scwin))) != NULL)
 	{
-		switch((MoveID) gtk_object_get_user_data(GTK_OBJECT(wid)))
+		switch((MoveID) GPOINTER_TO_INT(gtk_object_get_user_data(GTK_OBJECT(wid))))
 		{
 			case MVE_UP:
 				gtk_adjustment_set_value(adj, adj->value - fh);
@@ -347,8 +344,6 @@ static gint evt_move_clicked(GtkWidget *wid, gpointer data)
 				gtk_adjustment_set_value(adj, adj->upper - adj->page_size);
 				reset_search(txi);
 				break;
-			default:
-				;
 		}
 	}
 	return TRUE;
@@ -367,7 +362,7 @@ static void evt_goto_clicked(GtkWidget *wid, gpointer user)
 	gtk_box_pack_start(GTK_BOX(goto_dialog.vbox), goto_dialog.entry, FALSE, FALSE, 0);
 
 	gtk_widget_show_all(goto_dialog.vbox);
-	goto_dialog.dlg = dlg_dialog_sync_new(goto_dialog.vbox, _("Goto"), _("OK|Cancel"));
+	goto_dialog.dlg = dlg_dialog_sync_new(goto_dialog.vbox, _("Goto"), NULL);
 	gtk_widget_grab_focus(goto_dialog.entry);
 	if((dlg_dialog_sync_wait(goto_dialog.dlg) == DLG_POSITIVE) &&
 	   (adj = gtk_scrolled_window_get_vadjustment(GTK_SCROLLED_WINDOW(txi->scwin))) != NULL)
@@ -403,9 +398,9 @@ static void evt_goto_clicked(GtkWidget *wid, gpointer user)
 	}
 }
 
-static gint evt_close_clicked(GtkWidget *wid, gpointer data)
+static gint evt_close_clicked(GtkWidget *wid, gpointer user)
 {
-	TxvInfo		*txi = (TxvInfo *) data;
+	TxvInfo		*txi = user;
 	GtkWidget	*win;
 
 	if(txi != NULL)
@@ -413,7 +408,6 @@ static gint evt_close_clicked(GtkWidget *wid, gpointer data)
 		win = txi->win;
 		really_destroy(win);
 	}
-
 	return TRUE;
 }
 
@@ -458,7 +452,7 @@ static void show_text(TxvInfo *txi, gint pos)
 **		should really begin with '(', and with ')', and don't contain any non-escaped
 **		parentheses within!
 */
-static void do_search(TxvInfo *txi, char *re_src, int nocase, int nolf)
+static void do_search(TxvInfo *txi, const gchar *re_src, gboolean nocase, gboolean nolf)
 {
 	regex_t		re;
 	regmatch_t	rm[1];
@@ -486,16 +480,16 @@ static void do_search(TxvInfo *txi, char *re_src, int nocase, int nolf)
 	}
 	else
 	{
-		stu_strncpy(ebuf, _("Malformed regular expression:\n"), sizeof ebuf);
+		stu_strncpy(ebuf, _("Regular expression error:\n"), sizeof ebuf);
 		regerror(cerr, &re, ebuf + strlen(ebuf), sizeof ebuf - strlen(ebuf) - 1);
 		dlg_dialog_async_new_error(ebuf);
 	}
 }
 
 /* 1999-01-07 -	User hit the "Search" button, so let's pop up something to request parameters. */
-static void evt_search_clicked(GtkWidget *wid, gpointer data)
+static void evt_search_clicked(GtkWidget *wid, gpointer user)
 {
-	TxvInfo	*txi = (TxvInfo *) data;
+	TxvInfo	*txi = user;
 
 	srch_dialog.vbox  = gtk_vbox_new(FALSE, 0);
 	srch_dialog.label = gtk_label_new(_("Enter Text to Search For (RE)"));
@@ -514,7 +508,7 @@ static void evt_search_clicked(GtkWidget *wid, gpointer data)
 	gtk_box_pack_start(GTK_BOX(srch_dialog.vbox), srch_dialog.hbox, FALSE, FALSE, 0);
 
 	gtk_widget_show_all(srch_dialog.vbox);
-	srch_dialog.dlg = dlg_dialog_sync_new(srch_dialog.vbox, _("Search"), _("OK|Cancel"));
+	srch_dialog.dlg = dlg_dialog_sync_new(srch_dialog.vbox, _("Search"), NULL);
 	gtk_widget_grab_focus(srch_dialog.entry);
 	if(dlg_dialog_sync_wait(srch_dialog.dlg) == DLG_POSITIVE)
 	{
@@ -571,8 +565,8 @@ GtkWidget * txv_open(MainInfo *min, const gchar *label)
 	txi->keypress_handler = 0;
 
 	txi->win = win_window_open(min->cfg.wininfo, WIN_TEXTVIEW);
-	gtk_object_set_user_data(GTK_OBJECT(txi->win), (gpointer) txi);
-	gtk_signal_connect(GTK_OBJECT(txi->win), "delete_event", GTK_SIGNAL_FUNC(evt_delete), (gpointer) txi);
+	gtk_object_set_user_data(GTK_OBJECT(txi->win), txi);
+	gtk_signal_connect(GTK_OBJECT(txi->win), "delete_event", GTK_SIGNAL_FUNC(evt_delete), txi);
 	txv_connect_keypress(txi->win, NULL, NULL);
 
 	vbox = gtk_vbox_new(FALSE, 0);
@@ -581,12 +575,12 @@ GtkWidget * txv_open(MainInfo *min, const gchar *label)
 	gtk_text_set_word_wrap(GTK_TEXT(txi->text), FALSE);
 	gtk_text_set_editable(GTK_TEXT(txi->text), FALSE);
 	gtk_widget_set_name(txi->text, "txvText");
-	gtk_signal_connect(GTK_OBJECT(GTK_TEXT(txi->text)->vadj), "value_changed", GTK_SIGNAL_FUNC(evt_pos_changed), (gpointer) txi);
+	gtk_signal_connect(GTK_OBJECT(GTK_TEXT(txi->text)->vadj), "value_changed", GTK_SIGNAL_FUNC(evt_pos_changed), txi);
 
 	txi->scwin = gtk_scrolled_window_new(NULL, NULL);
 	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(txi->scwin), GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
 	gtk_signal_connect(GTK_OBJECT(gtk_scrolled_window_get_vadjustment(GTK_SCROLLED_WINDOW(txi->scwin))),
-				"value_changed", GTK_SIGNAL_FUNC(evt_pos_changed), (gpointer) txi);
+				"value_changed", GTK_SIGNAL_FUNC(evt_pos_changed), txi);
 	gtk_container_add(GTK_CONTAINER(txi->scwin), txi->text);
 
 	gtk_box_pack_start(GTK_BOX(vbox), txi->scwin, TRUE, TRUE, 0);
@@ -606,33 +600,33 @@ GtkWidget * txv_open(MainInfo *min, const gchar *label)
 	gtk_box_pack_start(GTK_BOX(txi->bhbox), algn, TRUE, TRUE, 0);
 
 	btn = gui_arrow_button_new(GTK_ARROW_UP);
-	gtk_object_set_user_data(GTK_OBJECT(btn), (gpointer) MVE_UP);
-	gtk_signal_connect(GTK_OBJECT(btn), "clicked", GTK_SIGNAL_FUNC(evt_move_clicked), (gpointer) txi);
+	gtk_object_set_user_data(GTK_OBJECT(btn), GINT_TO_POINTER(MVE_UP));
+	gtk_signal_connect(GTK_OBJECT(btn), "clicked", GTK_SIGNAL_FUNC(evt_move_clicked), txi);
 	gtk_box_pack_start(GTK_BOX(txi->bhbox), btn, FALSE, FALSE, 0);
 	btn = gui_arrow_button_new(GTK_ARROW_DOWN);
-	gtk_object_set_user_data(GTK_OBJECT(btn), (gpointer) MVE_DOWN);
-	gtk_signal_connect(GTK_OBJECT(btn), "clicked", GTK_SIGNAL_FUNC(evt_move_clicked), (gpointer) txi);
+	gtk_object_set_user_data(GTK_OBJECT(btn), GINT_TO_POINTER(MVE_DOWN));
+	gtk_signal_connect(GTK_OBJECT(btn), "clicked", GTK_SIGNAL_FUNC(evt_move_clicked), txi);
 	gtk_box_pack_start(GTK_BOX(txi->bhbox), btn, FALSE, FALSE, 0);
 	btn = gtk_button_new_with_label(_("Top"));
-	gtk_object_set_user_data(GTK_OBJECT(btn), (gpointer) MVE_TOP);
-	gtk_signal_connect(GTK_OBJECT(btn), "clicked", GTK_SIGNAL_FUNC(evt_move_clicked), (gpointer) txi);
+	gtk_object_set_user_data(GTK_OBJECT(btn), GINT_TO_POINTER(MVE_TOP));
+	gtk_signal_connect(GTK_OBJECT(btn), "clicked", GTK_SIGNAL_FUNC(evt_move_clicked), txi);
 	gtk_box_pack_start(GTK_BOX(txi->bhbox), btn, FALSE, FALSE, 0);
 	btn = gtk_button_new_with_label(_("Bottom"));
-	gtk_object_set_user_data(GTK_OBJECT(btn), (gpointer) MVE_BOTTOM);
-	gtk_signal_connect(GTK_OBJECT(btn), "clicked", GTK_SIGNAL_FUNC(evt_move_clicked), (gpointer) txi);
+	gtk_object_set_user_data(GTK_OBJECT(btn), GINT_TO_POINTER(MVE_BOTTOM));
+	gtk_signal_connect(GTK_OBJECT(btn), "clicked", GTK_SIGNAL_FUNC(evt_move_clicked), txi);
 	gtk_box_pack_start(GTK_BOX(txi->bhbox), btn, FALSE, FALSE, 0);
 
 	bhbox = gtk_hbox_new(FALSE, 0);
 	btn = gtk_button_new_with_label(_("Goto..."));
-	gtk_signal_connect(GTK_OBJECT(btn), "clicked", GTK_SIGNAL_FUNC(evt_goto_clicked), (gpointer) txi);
+	gtk_signal_connect(GTK_OBJECT(btn), "clicked", GTK_SIGNAL_FUNC(evt_goto_clicked), txi);
 	gtk_box_pack_start(GTK_BOX(bhbox), btn, FALSE, FALSE, 0);
 	btn = gtk_button_new_with_label(_("Search..."));
-	gtk_signal_connect(GTK_OBJECT(btn), "clicked", GTK_SIGNAL_FUNC(evt_search_clicked), (gpointer) txi);
+	gtk_signal_connect(GTK_OBJECT(btn), "clicked", GTK_SIGNAL_FUNC(evt_search_clicked), txi);
 	gtk_box_pack_start(GTK_BOX(bhbox), btn, FALSE, FALSE, 0);
 	gtk_box_pack_start(GTK_BOX(txi->bhbox), bhbox, FALSE, FALSE, 5);
 
 	btn = gtk_button_new_with_label(_("Close"));
-	gtk_signal_connect(GTK_OBJECT(btn), "clicked", GTK_SIGNAL_FUNC(evt_close_clicked), (gpointer) txi);
+	gtk_signal_connect(GTK_OBJECT(btn), "clicked", GTK_SIGNAL_FUNC(evt_close_clicked), txi);
 	gtk_box_pack_start(GTK_BOX(txi->bhbox), btn, FALSE, TRUE, 0);
 	gtk_widget_set_sensitive(txi->bhbox, FALSE);
 	gtk_box_pack_start(GTK_BOX(vbox), txi->bhbox, FALSE, FALSE, 0);
@@ -683,7 +677,7 @@ void txv_set_label(GtkWidget *wid, const gchar *text)
 
 	if(wid != NULL && (txi = gtk_object_get_user_data(GTK_OBJECT(wid))) != NULL && text != NULL)
 	{
-		gtk_window_set_title(GTK_WINDOW(wid), text);
+		win_window_set_title(wid, text);
 		gtk_label_set_text(GTK_LABEL(txi->blabel), text);
 	}
 }
