@@ -21,7 +21,6 @@
 
 #include "gentoo.h"
 
-#include <assert.h>
 #include <ctype.h>
 #include <fcntl.h>
 #include <stdlib.h>
@@ -43,7 +42,7 @@
 */
 static GList	*file_list = NULL;
 /* Another global (shudder), this one signals if there *are* any 'file'-using types around. */
-static gint	file_used = 0;
+static gboolean	file_used = FALSE;
 
 /* ----------------------------------------------------------------------------------------- */
 
@@ -55,7 +54,7 @@ static gint	file_used = 0;
 **		arguments; pretty close to my personal limit. :)
 ** 1999-05-29 -	Removed the <style> argument again, since it was too complex. Use typ_type_set_style().
 */
-FType * typ_type_new(CfgInfo *cfg, const gchar *name, mode_t mode, int perm, const gchar *suffix, const gchar *name_re, const gchar *file_re)
+FType * typ_type_new(CfgInfo *cfg, const gchar *name, mode_t mode, gint perm, const gchar *suffix, const gchar *name_re, const gchar *file_re)
 {
 	FType	*type;
 
@@ -78,7 +77,6 @@ FType * typ_type_new(CfgInfo *cfg, const gchar *name, mode_t mode, int perm, con
 			stu_strncpy(type->suffix, suffix, sizeof type->suffix);
 			type->flags |= FTFL_REQSUFFIX;
 		}
-	
 		if(name_re != NULL)
 		{
 			stu_strncpy(type->name_re_src, name_re, sizeof type->name_re_src);
@@ -115,7 +113,7 @@ FType * typ_type_copy(FType *old)
 			g_free(old->file_re);
 			old->file_re = NULL;
 		}
-		memcpy(nt, old, sizeof *nt);
+		*nt = *old;
 	}
 	return nt;
 }
@@ -135,15 +133,18 @@ void typ_type_destroy(FType *type)
 	g_free(type);
 }
 
+/* ----------------------------------------------------------------------------------------- */
+
+
 /* ----------------------------------------------------------------------------------------- */
 
 /* 1998-09-07 -	Check if the file described by <line> matches the permissions requirements
 **		in <type>. Returns 1 if so, otherwise 0.
 ** 1999-03-14 -	Made the access to the DirRow (previously DirLine) a lot more abstract.
 */
-static int check_perm(FType *type, DirRow *row)
+static gint check_perm(FType *type, DirRow *row)
 {
-	int	pr = 1, pw = 1, px = 1;
+	gint	pr = 1, pw = 1, px = 1;
 	mode_t	mode = 0;
 	uid_t	uid;
 	gid_t	gid;
@@ -179,9 +180,9 @@ static int check_perm(FType *type, DirRow *row)
 /* 1998-09-15 -	Check the RE in <re>, which has source <re_src>, against <string>.
 **		Returns 1 on match, 0 on failure.
 */
-static int check_re(const gchar *re_src, regex_t **re, gboolean glob, gboolean nocase, const gchar *string)
+static gint check_re(const gchar *re_src, regex_t **re, gboolean glob, gboolean nocase, const gchar *string)
 {
-	const gchar	*glob_re = NULL;
+	gchar	*glob_re = NULL;
 
 	if(*re == NULL)					/* RE not compiled? */
 	{
@@ -193,11 +194,11 @@ static int check_re(const gchar *re_src, regex_t **re, gboolean glob, gboolean n
 		*re = g_malloc(sizeof **re);
 		regcomp(*re, re_src, REG_EXTENDED | REG_NOSUB | (nocase ? REG_ICASE : 0));
 		if(glob_re)
-			g_free((void *) glob_re);			/* Free the globbed version. */
+			g_free(glob_re);			/* Free the globbed version. */
 	}
 	if(*re != NULL)
 		return regexec(*re, string, 0, NULL, 0) != REG_NOMATCH;
-	return REG_NOMATCH;
+	return REG_NOMATCH;	/* FIXME: Weird! */
 }
 
 /* 1998-08-02 -	Attempt to apply the identification rules given in by the type <type> to the
@@ -211,10 +212,10 @@ static int check_re(const gchar *re_src, regex_t **re, gboolean glob, gboolean n
 ** 1998-09-15 -	Moved the actual RE checking out to a function of its own, since it was getting complex.
 ** 1999-03-14 -	Replaced DirLine with new, opaque, DirRow.
 */
-static int check_type(FType *type, DirRow *row, const gchar *fout)
+static gint check_type(FType *type, DirRow *row, const gchar *fout)
 {
-	guint	tries = 0, hits = 0;
-	gchar	*name;
+	guint		tries = 0, hits = 0;
+	const gchar	*name;
 
 	/* Everything matches the unknown type, and that's a fact. */
 	if(TYP_IS_UNKNOWN(type))
@@ -249,14 +250,12 @@ static int check_type(FType *type, DirRow *row, const gchar *fout)
 		tries++;
 		hits += check_re(type->name_re_src, &type->name_re, type->flags & FTFL_NAMEGLOB, type->flags & FTFL_NAMENOCASE, name);
 	}
-
 	if(type->flags & FTFL_FILEMATCH)
 	{
 		tries++;
 		if(fout != NULL)
 			hits += check_re(type->file_re_src, &type->file_re, type->flags & FTFL_FILEGLOB, type->flags & FTFL_FILENOCASE, fout);
 	}
-
 	return tries == hits;
 }
 
@@ -279,7 +278,7 @@ static gint identify(MainInfo *min, DirRow *row)
 		if(check_type((FType *) here->data, row, NULL))
 			break;
 	}
-	DP_ROW_TYPE(row) = (FType *) here->data;
+	DP_ROW_TYPE(row) = here->data;
 	return !TYP_IS_UNKNOWN((FType *) here->data);
 }
 
@@ -288,7 +287,7 @@ static gint identify(MainInfo *min, DirRow *row)
 /* 1998-08-26 -	A rewrite of the much of the (three weeks old) file typing architecture, in
 **		order to accomodate use of 'file' in a way even resembling something efficient.
 **		Rather than just calling typ_identify() on a dirline, you call first this
-**		function, which initializes some state. You then, repeatedly, call typ_identify_line()
+**		function, which initializes some state. You then, repeatedly, call typ_identify()
 **		for all lines you wish to type. Then call typ_identify_end(). Done!
 */
 void typ_identify_begin(MainInfo *min)
@@ -303,7 +302,7 @@ void typ_identify_begin(MainInfo *min)
 		if(((FType *) here->data)->flags & FTFL_FILEMATCH)
 			break;
 	}
-	file_used = (here != NULL) ? TRUE : FALSE;
+	file_used = (here != NULL);
 }
 
 /* ----------------------------------------------------------------------------------------- */
@@ -315,7 +314,7 @@ void typ_identify_begin(MainInfo *min)
 void typ_identify(MainInfo *min, DirRow *row)
 {
 	if((identify(min, row) == FALSE) && (file_used == TRUE))
-		file_list = g_list_prepend(file_list, (gpointer) row);
+		file_list = g_list_prepend(file_list, row);
 }
 
 /* ----------------------------------------------------------------------------------------- */
@@ -345,70 +344,78 @@ static FType * match_file(GList *list, DirRow *row, const gchar *fout)
 */
 void typ_identify_end(MainInfo *min, const gchar *path)
 {
-	GList	*here;
-	gchar	buf[MAXNAMLEN + 2], *temp_name, line[MAXNAMLEN + 256], *fout;
-	gint	fd[2], len, status;
-	pid_t	file_pid;
-	FType	*type;
-	FILE	*in;
+	const GList	*here;
+	gchar		buf[MAXNAMLEN + 2], *temp_name;
+	gint		fd[2], len, status;
+	pid_t		file_pid;
+	FType		*type;
 
 	if(file_list == NULL)
 		return;
 	if((temp_name = tmpnam(NULL)) == NULL)
 		return;
+	if(pipe(fd) != 0)
+		return;
 
-	if(pipe(fd) == 0)
+	file_pid = fork();
+	if(file_pid == 0)
 	{
-		switch((file_pid = fork()))
-		{
-			case 0:
-				assert(close(STDIN_FILENO) == 0);			/* Kill stdin. */
-				assert(dup(fd[STDIN_FILENO]) == STDIN_FILENO);		/* Replace with pipe from main. */
-				assert(close(fd[STDIN_FILENO]) == 0);			/* Close the pipe. */
-				assert(close(STDOUT_FILENO) == 0);			/* Kill stdout. */
-				assert(open(temp_name, O_CREAT|O_RDWR, S_IRWXU) == STDOUT_FILENO);
-				assert(close(fd[STDOUT_FILENO]) == 0);			/* Close unused pipe. */
-				assert(chdir(path) == 0);				/* Change to the path in question. */
-				execlp("file", "file", "-f", "-", NULL);
-				exit(EXIT_FAILURE);
-			case -1:
-				fprintf(stderr, "fork() failed! Damn!\n");
-				exit(EXIT_FAILURE);
-		}
-		assert(close(fd[STDIN_FILENO]) == 0);		/* Close input part of pipe (used by child). */
-		for(here = file_list; here != NULL; here = g_list_next(here))
-		{
-			len = g_snprintf(buf, sizeof buf, "%s\n", DP_SEL_NAME(here));
-			write(fd[STDOUT_FILENO], buf, len);
-		}
+		guint	bits = 0U;
 
-		close(fd[STDOUT_FILENO]);
-		waitpid(file_pid, &status, 0);
-
-		if(WIFEXITED(status))
+		if(close(STDIN_FILENO) == 0)
 		{
-			if((in = fopen(temp_name, "rt")) != NULL)
-			{
-				for(here = file_list; here != NULL && fgets(line, sizeof line, in); here = g_list_next(here))
-				{
-					if((fout = strchr(line, ':')) == NULL)
-						continue;
-					for(fout++; isspace((int) *fout); fout++)
-						;
-					if((type = match_file(min->cfg.type, DP_SEL_ROW(here), fout)) != NULL)
-						DP_SEL_TYPE(here) = type;
-				}
-				fclose(in);
-			}
-			remove(temp_name);
+			if(dup(fd[STDIN_FILENO]) == STDIN_FILENO)
+				bits |= (close(fd[STDIN_FILENO]) == 0);
 		}
-		else
-			fprintf(stderr, "Failure!!\n");
-		g_list_free(file_list);
-		file_list = NULL;
+		if(close(STDOUT_FILENO) == 0)
+		{
+			if(open(temp_name, O_CREAT | O_RDWR, S_IRWXU) == STDOUT_FILENO)
+				bits |= (close(fd[STDOUT_FILENO]) == 0) << 1;
+		}
+		if(bits == 3U && chdir(path) == 0)
+			execlp("file", "file", "-f", "-", NULL);
+		_exit(EXIT_FAILURE);
 	}
-	else
-		fprintf(stderr, "No pipe!\n");
+	else if(file_pid < 0)
+	{
+		g_warning("Couldn't fork() 'file' for file typing");
+		return;
+	}
+	/* We don't need the input part of the pipe, so close it. */
+	close(fd[STDIN_FILENO]);
+	/* Now, we have file listening on pipe, so write all filenames to it. */
+	for(here = file_list; here != NULL; here = g_list_next(here))
+	{
+		len = g_snprintf(buf, sizeof buf, "%s\n", DP_SEL_NAME(here));
+		write(fd[STDOUT_FILENO], buf, len);
+	}
+	close(fd[STDOUT_FILENO]);
+	waitpid(file_pid, &status, 0);
+
+	if(WIFEXITED(status))
+	{
+		FILE	*in;
+
+		if((in = fopen(temp_name, "rt")) != NULL)
+		{
+			const gchar	*fout;
+			gchar		line[MAXNAMLEN + 256];
+
+			for(here = file_list; here != NULL && fgets(line, sizeof line, in); here = g_list_next(here))
+			{
+				if((fout = strchr(line, ':')) == NULL)
+					continue;
+				for(fout++; isspace((guchar) *fout); fout++)
+					;
+				if((type = match_file(min->cfg.type, DP_SEL_ROW(here), fout)) != NULL)
+					DP_SEL_TYPE(here) = type;
+			}
+			fclose(in);
+		}
+	}
+	g_list_free(file_list);
+	file_list = NULL;
+	remove(temp_name);
 }
 
 /* ----------------------------------------------------------------------------------------- */
@@ -481,6 +488,30 @@ GList * typ_type_remove(GList *list, FType *type)
 	return list;
 }
 
+/* 2002-03-17 -	Do a globbed lookup of a named type. Returns a list of matching FTypes, which
+**		will be a sublist of the grand input <list> of all types. The list, of course,
+**		will need to be freed when you're done. The data in it, however, will not.
+*/
+GList * typ_type_lookup_glob(const GList *list, const gchar *glob)
+{
+	GList		*ret = NULL;
+	gchar		*re_src;
+	regex_t		re;
+
+	re_src = stu_glob_to_re(glob);
+	if(regcomp(&re, re_src, REG_EXTENDED | REG_NOSUB | REG_ICASE) == 0)
+	{
+		for(; list != NULL; list = g_list_next(list))
+		{
+			if(regexec(&re, ((FType *) list->data)->name, 0, NULL, 0) == 0)
+				ret = g_list_append(ret, list->data);
+		}
+		regfree(&re);
+	}
+	g_free(re_src);
+	return ret;
+}
+
 /* ----------------------------------------------------------------------------------------- */
 
 /* 1998-09-17 -	Change the name of given type. Since the name is used as a fall-back
