@@ -44,19 +44,18 @@ static UsrInfo	the_usr = { NULL, NULL, NULL };
 
 /* ----------------------------------------------------------------------------------------- */
 
-static gint string_num_cmp(gconstpointer a, gconstpointer b)
+static gint cmp_string_num(gconstpointer a, gconstpointer b)
 {
 	gint	ia, ib;
 
-	sscanf((gchar *) a, "%d", &ia);
-	sscanf((gchar *) b, "%d", &ib);
-
-	return (ia < ib) ? -1 : (ia > ib);
+	if(sscanf(a, "%d", &ia) == 1 && sscanf(b, "%d", &ib) == 1)
+		return (ia < ib) ? -1 : (ia > ib);
+	return 0;
 }
 
 static void list_visit(gpointer key, gpointer value, gpointer user)
 {
-	*((GList **) user) = g_list_insert_sorted(*((GList **) user), g_strdup_printf("%d %s", GPOINTER_TO_INT(key), (gchar *) value), (GCompareFunc) string_num_cmp);
+	*(GList **) user = g_list_insert_sorted(*(GList **) user, g_strdup_printf("%d %s", GPOINTER_TO_INT(key), (const gchar *) value), cmp_string_num);
 }
 
 /* 2000-03-21 -	Build a list of strings (not GStrings, but plain old dynamic gchar pointers) of
@@ -68,8 +67,7 @@ GList * usr_string_list_create(UsrCategory category, gint id, gint *index)
 {
 	GList	*list = NULL, *iter;
 
-	g_hash_table_foreach(category == UIC_USER ? the_usr.dict_uname : the_usr.dict_gname,
-				list_visit, &list);
+	g_hash_table_foreach(category == UIC_USER ? the_usr.dict_uname : the_usr.dict_gname, list_visit, &list);
 
 	if(index != NULL)		/* Do we need to search? */
 	{
@@ -77,7 +75,7 @@ GList * usr_string_list_create(UsrCategory category, gint id, gint *index)
 
 		for(iter = list, pos = 0; iter != NULL; iter = g_list_next(iter), pos++)
 		{
-			if((sscanf((gchar *) iter->data, "%d", &tmp) == 1) && (tmp == id))
+			if((sscanf(iter->data, "%d", &tmp) == 1) && (tmp == id))
 				*index = pos;
 		}
 	}
@@ -107,24 +105,24 @@ static void scan_uid(void)
 	setpwent();
 	while((pw = getpwent()) != NULL)
 	{
-		if((old = g_hash_table_lookup(the_usr.dict_uname, GINT_TO_POINTER((int) pw->pw_uid))) != NULL)
+		if((old = g_hash_table_lookup(the_usr.dict_uname, GINT_TO_POINTER((gint) pw->pw_uid))) != NULL)
 		{
-			if((home = g_hash_table_lookup(the_usr.dict_uhome, (gpointer) old)) != NULL)
+			if((home = g_hash_table_lookup(the_usr.dict_uhome, old)) != NULL)
 			{
-				g_hash_table_remove(the_usr.dict_uname, GINT_TO_POINTER((int) pw->pw_uid));
+				g_hash_table_remove(the_usr.dict_uname, GINT_TO_POINTER((gint) pw->pw_uid));
 				g_hash_table_remove(the_usr.dict_uhome, old);
 				g_free(old);
 				g_free(home);
 			}
 			else
-				fprintf(stderr, "USERINFO: passwd data for '%s' (%d) has no home dir!\n", old, (int) pw->pw_uid);
+				fprintf(stderr, "USERINFO: passwd data for '%s' (%d) has no home dir!\n", old, (gint) pw->pw_uid);
 		}
 		if((name = g_strdup(pw->pw_name)) != NULL)
 		{
 			if((home = g_strdup(pw->pw_dir)) != NULL)
 			{
-				g_hash_table_insert(the_usr.dict_uname, GINT_TO_POINTER((int) pw->pw_uid), (gpointer) name);
-				g_hash_table_insert(the_usr.dict_uhome, (gpointer) name, (gpointer) home);
+				g_hash_table_insert(the_usr.dict_uname, GINT_TO_POINTER((gint) pw->pw_uid), name);
+				g_hash_table_insert(the_usr.dict_uhome, name, home);
 			}
 			else
 			{
@@ -149,13 +147,13 @@ static void scan_gid(void)
 	setgrent();
 	while((gr = getgrent()) != NULL)
 	{
-		if((old = g_hash_table_lookup(the_usr.dict_gname, GINT_TO_POINTER((int) gr->gr_gid))) != NULL)
+		if((old = g_hash_table_lookup(the_usr.dict_gname, GINT_TO_POINTER((gint) gr->gr_gid))) != NULL)
 		{
-			g_hash_table_remove(the_usr.dict_gname, GINT_TO_POINTER((int) gr->gr_gid));
+			g_hash_table_remove(the_usr.dict_gname, GINT_TO_POINTER((gint) gr->gr_gid));
 			g_free(old);
 		}
 		if((name = g_strdup(gr->gr_name)) != NULL)
-			g_hash_table_insert(the_usr.dict_gname, GINT_TO_POINTER((int) gr->gr_gid), name);
+			g_hash_table_insert(the_usr.dict_gname, GINT_TO_POINTER((gint) gr->gr_gid), name);
 	}
 	endgrent();
 }
@@ -167,7 +165,7 @@ static void scan_gid(void)
 */
 const gchar * usr_lookup_uname(uid_t uid)
 {
-	return (const char *) g_hash_table_lookup(the_usr.dict_uname, GINT_TO_POINTER((int) uid));
+	return g_hash_table_lookup(the_usr.dict_uname, GINT_TO_POINTER((gint) uid));
 }
 
 struct id_lookup {
@@ -182,8 +180,8 @@ static void cb_lookup_id(gpointer key, gpointer value, gpointer user)
 {
 	struct id_lookup	*idl = user;
 
-	if(strcmp((gchar *) value, idl->name) == 0)
-		idl->value = (long) key;
+	if(strcmp(value, idl->name) == 0)
+		idl->value = GPOINTER_TO_INT(key);
 }
 
 /* 2000-03-23 -	This does the reverse lookup of the function above. I don't really like the
@@ -194,7 +192,7 @@ long usr_lookup_uid(const gchar *name)
 {
 	struct id_lookup	idl;
 
-	idl.name = name;
+	idl.name  = name;
 	idl.value = -1;
 
 	g_hash_table_foreach(the_usr.dict_uname, cb_lookup_id, &idl);
@@ -206,14 +204,14 @@ long usr_lookup_uid(const gchar *name)
 **		or NULL if there is no such user. If <uname> is "" or NULL, returns the home
 **		directory of the current effective user.
 */
-const gchar * usr_lookup_uhome(const char *uname)
+const gchar * usr_lookup_uhome(const gchar *uname)
 {
 	if(uname == NULL || *uname == '\0')
 	{
 		if((uname = usr_lookup_uname(geteuid())) != NULL)
-			return g_hash_table_lookup(the_usr.dict_uhome, (gpointer) uname);
+			return g_hash_table_lookup(the_usr.dict_uhome, uname);
 	}
-	return g_hash_table_lookup(the_usr.dict_uhome, (gpointer) uname);
+	return g_hash_table_lookup(the_usr.dict_uhome, uname);
 }
 
 /* 1998-05-29 -	Rewritten & renamed. This routine looks up the name of the group with the given
@@ -221,7 +219,7 @@ const gchar * usr_lookup_uhome(const char *uname)
 */
 const gchar * usr_lookup_gname(gid_t gid)
 {
-	return g_hash_table_lookup(the_usr.dict_gname, GINT_TO_POINTER((int) gid));
+	return g_hash_table_lookup(the_usr.dict_gname, GINT_TO_POINTER(gid));
 }
 
 /* 2000-03-23 -	This does the reverse lookup of the function above. I don't really like the
@@ -230,9 +228,9 @@ const gchar * usr_lookup_gname(gid_t gid)
 */
 long usr_lookup_gid(const gchar *name)
 {
-	struct id_lookup	idl;
+	struct id_lookup idl;
 
-	idl.name = name;
+	idl.name  = name;
 	idl.value = -1;
 
 	g_hash_table_foreach(the_usr.dict_gname, cb_lookup_id, &idl);
@@ -242,9 +240,10 @@ long usr_lookup_gid(const gchar *name)
 
 /* ----------------------------------------------------------------------------------------- */
 
-static gint id_equal(gpointer a, gpointer b)
+/* Compare two pointers as integers. Can't use g_int_equal(), since that dereferences. */
+static gint cmp_int(gconstpointer a, gconstpointer b)
 {
-	return GPOINTER_TO_UINT(a) == GPOINTER_TO_UINT(b);
+	return GPOINTER_TO_INT(a) == GPOINTER_TO_INT(b);
 }
 
 /* 1998-12-22 -	Initialize the userinfo module. Calling any of the functions exported from this
@@ -253,11 +252,11 @@ static gint id_equal(gpointer a, gpointer b)
 */
 gboolean usr_init(void)
 {
-	if((the_usr.dict_uname = g_hash_table_new(g_direct_hash, (GCompareFunc) id_equal)) != NULL)
+	if((the_usr.dict_uname = g_hash_table_new(g_direct_hash, cmp_int)) != NULL)
 	{
 		if((the_usr.dict_uhome = g_hash_table_new(g_str_hash, g_str_equal)) != NULL)
 		{
-			if((the_usr.dict_gname = g_hash_table_new(g_direct_hash, (GCompareFunc) id_equal)) != NULL)
+			if((the_usr.dict_gname = g_hash_table_new(g_direct_hash, cmp_int)) != NULL)
 			{
 				scan_uid();
 				scan_gid();
