@@ -15,7 +15,10 @@
 **		prefixing the actual value with the letter 'u'.
 */
 
+#include "gentoo.h"
+
 #include <ctype.h>
+#include <locale.h>
 #include <stdio.h>
 #include <stdarg.h>
 #include <stdlib.h>
@@ -88,7 +91,7 @@ struct XmlNode {
 #define	XML_FALSE		"FALSE"
 #define	XML_UNSIGNED		"u"		/* Prefix for unsigned integers. */
 #define	XML_COLOR		"C"		/* Prefix for GdkColor RGB triplets. */
-#define	NODE_INTERNAL(tag)	(isupper((unsigned char) tag[0]) || (tag[0] == '/' && isupper((unsigned char) tag[1])))
+#define	NODE_INTERNAL(tag)	(isupper((guchar) tag[0]) || (tag[0] == '/' && isupper((guchar) tag[1])))
 #define	NODE_COMMENT(tag)	(tag[0] == '?')
 #define	SIGN(x)	(((x) == 0) ? 0 : (((x) > 0) ? 1 : -1))
 
@@ -110,11 +113,11 @@ static	struct {
 /* This little global variable (shudder) holds the nesting level for tags in the file being
 ** written by xml_put_XXX() functions.
 */
-static int	put_level = 0;
+static gint	put_level = 0;
 
 /* ----------------------------------------------------------------------------------------- */
 
-static int put_string(FILE *out, const char *str);
+static void	put_string(FILE *out, const gchar *str);
 
 /* ----------------------------------------------------------------------------------------- */
 
@@ -193,11 +196,34 @@ void xml_put_uinteger(FILE *out, const gchar *name, guint value)
 	fprintf(out, "<%s>%s%u</%s>\n", name, XML_UNSIGNED, value, name);
 }
 
-/* 1998-07-27 -	Output a leaf with a real number. */
+/* 1998-07-27 -	Output a leaf with a real number.
+** 2002-06-16 -	Significantly more complex, since we need to do this in ASCII. glib 2.0, please.
+*/
 void xml_put_real(FILE *out, const gchar *name, gfloat value)
 {
+	struct lconv	*lconv;
+	gchar		buf[64], *ptr;
+	gssize		dpl;
+
 	indent(out);
-	fprintf(out, "<%s>%f</%s>\n", name, value, name);
+	g_snprintf(buf, sizeof buf, "%f", value);
+	lconv = localeconv();
+	dpl   = strlen(lconv->decimal_point);
+	if(dpl && (ptr = strchr(buf, *lconv->decimal_point)) != NULL)
+	{
+		if(strncmp(ptr, lconv->decimal_point, dpl) == 0)
+		{
+			gssize	tl;
+
+			*ptr++ = '.';
+			tl = strlen(ptr + dpl - 1);
+			memmove(ptr, ptr + dpl - 1, tl);
+			ptr[tl] = '\0';
+		}
+	}
+	else
+		g_warning(_("XML module emitted locale-dependant real number"));
+	fprintf(out, "<%s>%s</%s>\n", name, buf, name);
 }
 
 /* 1998-07-27 -	Output a leaf with a boolean in it. */
@@ -232,7 +258,7 @@ void xml_put_color(FILE *out, const gchar *name, const GdkColor *color)
 **		we don't want appearing between tags (such as < and >). This is overly
 **		paranoid; usually you don't need to protect the '>' character.
 */
-static int put_string(FILE *out, const gchar *str)
+static void put_string(FILE *out, const gchar *str)
 {
 	gchar	here;
 	gint	i;
@@ -255,7 +281,6 @@ static int put_string(FILE *out, const gchar *str)
 				putc(here, out);
 		}
 	}
-	return 1;
 }
 
 /* ----------------------------------------------------------------------------------------- */
@@ -277,7 +302,7 @@ static const XmlNode * get_node(const XmlNode *node, const gchar *name, XVType t
 **		steps fail, return 0. Can be called with <n> == NULL to check for the presence
 **		of a correctly typed and named value.
 */
-gboolean xml_get_integer(const XmlNode *node, const gchar *name, int *n)
+gboolean xml_get_integer(const XmlNode *node, const gchar *name, gint *n)
 {
 	const XmlNode	*data;
 
@@ -306,7 +331,7 @@ gboolean xml_get_uinteger(const XmlNode *node, const gchar *name, guint *n)
 /* 1998-12-25 -	Search through <node>'s children for a real-valued node named <name>, and store
 **		the number in question at <x>. Returns 1 if value was found, 0 otherwise.
 */
-gboolean xml_get_real(const XmlNode *node, const gchar *name, float *x)
+gboolean xml_get_real(const XmlNode *node, const gchar *name, gfloat *x)
 {
 	const XmlNode	*data;
 
@@ -407,10 +432,10 @@ static void xml_error(gchar *fmt, ...)
 **		caller when done. Also returns an integer in <what> indicating the type of
 **		the string returned; this is 0 for user text, 1 for a tag, and -1 on error.
 */
-static GString * get_token(FILE *in, int *what)
+static GString * get_token(FILE *in, gint *what)
 {
 	GString	*str;
-	int	here, prev = 0, in_tag = 0, found_nsc = 0, in_string = 0;
+	gint	here, in_tag = 0, found_nsc = 0, in_string = 0;
 
 	if(what == NULL)
 		return NULL;
@@ -449,12 +474,18 @@ static GString * get_token(FILE *in, int *what)
 			}
 			else
 			{
+				if(here == '\\')
+				{
+					if((here = fgetc(in)) < 0)
+						break;
+					g_string_append_c(str, '\\');
+					g_string_append_c(str, here);
+					continue;
+				}
 				g_string_append_c(str, here);
-				found_nsc |= !isspace((unsigned char) here);
-				if(here == '"' && prev != '\\')
-					in_string ^= 1;
+				found_nsc |= !isspace((guchar) here);
+				in_string ^= (here == '"');
 			}
-			prev = here;
 		}
 	}
 	g_string_free(str, TRUE);
@@ -505,12 +536,12 @@ void xml_node_add_child(XmlNode *node, XmlNode *child)
 **		to use different callback functions for different nodes.
 ** 1999-01-04 -	This too used the old bad way of using glib's list functions. Fixed.
 */
-void xml_node_visit_children(const XmlNode *node, void (*func)(const XmlNode *child, gpointer user_data), gpointer user_data)
+void xml_node_visit_children(const XmlNode *node, void (*func)(const XmlNode *child, gpointer user), gpointer user)
 {
-	GSList	*iter;
+	const GSList	*iter;
 
 	for(iter = node->children; iter != NULL; iter = g_slist_next(iter))
-		func((XmlNode *) iter->data, user_data);
+		func(iter->data, user);
 }
 
 /* 1998-07-26 -	Destroy a node. Knows how to destroy string data. You really shouldn't destroy a
@@ -543,7 +574,7 @@ static gint match_tags(const gchar *beg, const gchar *end)
 */
 static gint is_word(const gchar *str, const gchar *word)
 {
-	while(isspace((unsigned char) *str))
+	while(isspace((guchar) *str))
 		str++;
 	return strcmp(str, word) == 0;
 }
@@ -557,40 +588,40 @@ static gint is_word(const gchar *str, const gchar *word)
 **		thus clearly blurring the line between text and floats. This means that
 **		reals are now required to contain at least one digit before the dot.
 */
-static int get_real(gchar *str, gfloat *p)
+static gboolean get_real(const gchar *str, gfloat *p)
 {
-	gchar	*beg = str;
-	gfloat	num = 0.0f, fract, weight;
-	gint	sign = 1;
+	const gchar	*beg = str;
+	gfloat		num = 0.0f, fract, weight;
+	gint		sign = 1;
 
 	if(*str == '-')
 	{
 		sign = -1;
 		str++;
 	}
-	while(isdigit((unsigned char) *str))
+	while(isdigit((guchar) *str))
 	{
 		num *= 10;
 		num += *str - '0';
 		str++;
 	}
 	if(str == beg)		/* If no digits were found, it's not a real anymore. */
-		return 0;
+		return FALSE;
 	if(*str == '.')
 	{
-		for(str++, fract = 0.0f, weight = 0.1f; isdigit((unsigned char) *str); str++, weight /= 10.0f)
+		for(str++, fract = 0.0f, weight = 0.1f; isdigit((guchar) *str); str++, weight /= 10.0f)
 			fract += (*str - '0') * weight;
 		*p = sign * (num + fract);
-		return 1;
+		return TRUE;
 	}
-	return 0;
+	return FALSE;
 }
 
-/* 1998-07-27 -	Create a duplicate of the string data in <in>, while taking care to replace
-**		any &-escaped "sequences" (such as "&amp;") with their single-char equivalences.
-**		Note that we don't even pretend to support all XML sequences.
+/* 1998-07-27 -	Create a duplicate of the string data in <in>, while taking care to replace any &-escaped
+**		"sequences" (such as "&amp;") with their single-character equivalences. Note that we don't
+**		even pretend to support all XML sequences.
 */
-static GString * get_text_data(gchar *in)
+static GString * get_text_data(const gchar *in)
 {
 	GString	*str;
 	gchar	here, next;
@@ -645,9 +676,9 @@ static GString * get_text_data(gchar *in)
 **		FALSE if it was not. Uses built-in knowledge about how different types of
 **		data look in the XML file to determine the type of data at <str>.
 */
-static gint get_data(XmlNode *node, gchar *str)
+static gint get_data(XmlNode *node, const gchar *str)
 {
-	unsigned int	red, green, blue;
+	guint	red, green, blue;
 
 	node->data.type = XVT_NONE;
 
@@ -695,7 +726,7 @@ static XmlNode * read_tree(FILE *in, XmlNode *root)
 {
 	GString	*str, *data, *end;
 	XmlNode	*node, *tree;
-	int	type, data_type, end_type;
+	gint	type, data_type, end_type;
 
 	while((str = get_token(in, &type)) != NULL)
 	{
@@ -740,7 +771,7 @@ static XmlNode * read_tree(FILE *in, XmlNode *root)
 						if(get_data(node, data->str))
 							xml_node_add_child(root, node);
 						else
-							xml_error("Couldn't get data for '%s'", node->name);
+							xml_error("Couldn't get data for '%s' (unknown type)", node->name);
 					}
 					else
 						xml_error("Overlapping tag found (%s,%s)", str->str, end->str);
@@ -749,7 +780,7 @@ static XmlNode * read_tree(FILE *in, XmlNode *root)
 					xml_error("Missing end tag for '%s'", node->name);
 			}
 			else
-				xml_error("Couldn't get data for '%s'", node->name);
+				xml_error("Couldn't get data for '%s' (no token)", node->name);
 		}
 		else
 			xml_error("Bare data found");
@@ -769,12 +800,12 @@ static XmlNode * read_tree(FILE *in, XmlNode *root)
 */
 void xml_tree_destroy(XmlNode *node)
 {
-	GSList	*iter;
+	const GSList	*iter;
 
 	if(node != NULL)
 	{
 		for(iter = node->children; iter != NULL; iter = g_slist_next(iter))
-			xml_tree_destroy((XmlNode *) iter->data);
+			xml_tree_destroy(iter->data);
 		xml_node_destroy(node);
 	}
 }
@@ -783,9 +814,9 @@ void xml_tree_destroy(XmlNode *node)
 
 #if 0		/* xml_tree_print() */
 /* 1998-07-27 -	Prints an outline of a tree. Quite handy when debugging. */
-void xml_tree_print(const XmlNode *root, gpointer data)
+void xml_tree_print(const XmlNode *root, gpointer user)
 {
-	int	i, level = (int) data;
+	gint	i, level = (gint) user;
 
 	if(root != NULL)
 	{
@@ -818,7 +849,7 @@ void xml_tree_print(const XmlNode *root, gpointer data)
 			}
 		}
 		putchar('\n');
-		g_slist_foreach(root->children, (GFunc) xml_tree_print, (gpointer) (level + 1));
+		g_slist_foreach(root->children, (GFunc) xml_tree_print, GINT_TO_POINTER((level + 1)));
 	}
 }
 #endif
@@ -830,7 +861,7 @@ void xml_tree_print(const XmlNode *root, gpointer data)
 */
 const XmlNode * xml_tree_search(const XmlNode *root, const gchar *name)
 {
-	GSList		*iter;
+	const GSList	*iter;
 	const XmlNode	*node;
 
 	if(root != NULL)
@@ -839,7 +870,7 @@ const XmlNode * xml_tree_search(const XmlNode *root, const gchar *name)
 			return root;
 		for(iter = root->children; iter != NULL; iter = g_slist_next(iter))
 		{
-			if((node = xml_tree_search((XmlNode *) iter->data, name)) != NULL)
+			if((node = xml_tree_search(iter->data, name)) != NULL)
 				return node;
 		}
 	}
